Package: CausalR
Type: Package
Title: Causal network analysis methods
Version: 1.45.0
Date: 2016-11-14
Author: Glyn Bradley, Steven Barrett, Chirag Mistry, Mark Pipe, David Wille, David Riley, Bhushan Bonde, Peter Woollard
Maintainer: Glyn Bradley <glyn.x.bradley@gsk.com>, Steven Barrett <steven.j.barrett@gsk.com>
Description: Causal network analysis methods for regulator prediction and network reconstruction from genome scale data.
Depends: R (>= 3.2.0)
Imports: igraph
Suggests: knitr, RUnit, BiocGenerics
VignetteBuilder: knitr
biocViews: ImmunoOncology, SystemsBiology, Network, GraphAndNetwork,
        Network Inference, Transcriptomics, Proteomics,
        DifferentialExpression, RNASeq, Microarray
License: GPL (>= 2)
NeedsCompilation: no
RoxygenNote: 5.0.1
git_url: https://git.bioconductor.org/packages/CausalR
git_branch: devel
git_last_commit: 41ac129
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 21:40:02 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(AnalysePredictionsList)
export(CalculateEnrichmentPValue)
export(CalculateSignificance)
export(CalculateSignificanceUsingCubicAlgorithm)
export(CalculateSignificanceUsingCubicAlgorithm1b)
export(CalculateSignificanceUsingQuarticAlgorithm)
export(CompareHypothesis)
export(CreateCCG)
export(CreateCG)
export(GetNodeName)
export(GetNumberOfPositiveAndNegativeEntries)
export(GetShortestPathsFromCCG)
export(MakePredictions)
export(MakePredictionsFromCCG)
export(MakePredictionsFromCG)
export(PlotGraphWithNodeNames)
export(RankTheHypotheses)
export(ReadExperimentalData)
export(ScoreHypothesis)
export(WriteAllExplainedNodesToSifFile)
export(WriteExplainedNodesToSifFile)
export(runSCANR)
import(igraph)

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-02 11:21:09 +0000 (Wed, 02 Jan 2013) $
# Changed By:  	$Author: MISC $
# Version:		$Revision: 351 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkPredictions/DetermineInteractionTypeOfPath.r $
# 
# *** Summary ***
# Description: Given a path and through the network, this function will determine if the path results in activation or inhibition.
# Activation is indicated by 1, inhibition by -1 

DetermineInteractionTypeOfPath <- function(network, nodesInPath){
# network must be an igraph
# nodesInPath is an ordered list of the nodes visited on the path - note that these contain numbers which use R's internal reference to the edges

numEdges <- length(nodesInPath[[1]])-1
# Declare a variable that will store the numerical references for the edges in a path
nodeEdges <- matrix(0,1,numEdges)

for (i in 1:numEdges){
	# Get the R's edge number for two successive nodes in nodesInPath
	nodeEdges[i] <- E(network)[nodesInPath[[1]][i]%->%nodesInPath[[1]][i+1]][[1]]
}
# Take the product of the weights for each edge in the path to get the overall interaction
netPathInteraction = prod(get.edge.attribute(network, "Weight", nodeEdges))
	
return(netPathInteraction)
}

# Test functions for the MakePredictionsFromCG functionality

# Copyright: Copyright 2012 GSK. All rights reserved

# *** SVN version information *** $Date: 2015-12-15 11:36:58 +0000 (Tue, 15 Dec 2015) $ $Revision: 553 $ $Author: pipm $ $HeadURL:
# https://stevenagefs:7777/7412/trunk/NetworkPredictions/test_CausalReasoningPredictions.R $

CreateSimpleNetworkForTest_ComputationalCausalGraphCreator <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    network <- igraph::set_edge_attr(network, "Weight", 1, 1)
    network <- igraph::set_edge_attr(network, "Weight", 2, -1)
    network <- igraph::set_vertex_attr(network, "ID", 1:3, 1:3)
    
    return(network)
}

test_RemoveIDsNotInExperimentalDataProducesTheCorrectOutputList <- function() {
    connectedNodes <- c(1, 2, 3, 4, 5, 6)
    expData <- matrix(c(1, 2, 5, 6), ncol = 2)
    connectedNodesInExpData <- CausalR:::RemoveIDsNotInExperimentalData(connectedNodes, expData)
    checkEquals(length(connectedNodesInExpData), 4)
    checkEquals(connectedNodesInExpData[1], 1)
    checkEquals(connectedNodesInExpData[2], 2)
    checkEquals(connectedNodesInExpData[3], 5)
    checkEquals(connectedNodesInExpData[4], 6)
}

test_DetermineInteractionTypeOfPathProducesTheCorrectOutput <- function() {
    test_network <- CreateSimpleNetworkForTest_ComputationalCausalGraphCreator()
    path <- igraph::shortest_paths(test_network, 1, 2, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path$vpath)) {
        path <- lapply(path$vpath, as.vector)
    }
    interaction1 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path)
    checkEquals(interaction1, 1)
    path2 <- igraph::shortest_paths(test_network, 1, 3, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path2$vpath)) {
        path2 <- lapply(path2$vpath, as.vector)
    }
    interaction2 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path2)
    checkEquals(interaction2, -1)
}

test_DetermineInteractionTypeOfPathForCyclicNetwork <- function() {
    test_network <- igraph::graph_from_literal("node0" - +"node1", "node1" - +"node0")
    test_network <- igraph::set_edge_attr(test_network, "Weight", 1, 1)
    test_network <- igraph::set_edge_attr(test_network, "Weight", 2, -1)
    path <- igraph::shortest_paths(test_network, 1, 2, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path$vpath)) {
        path <- lapply(path$vpath, as.vector)
    }
    interaction1 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path)
    checkEquals(interaction1, 1)
    path2 <- igraph::shortest_paths(test_network, 2, 1, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path2$vpath)) {
        path2 <- lapply(path2$vpath, as.vector)
    }
    interaction2 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path2)
    checkEquals(interaction2, -1)
    
    # Change the order in which the edges are given, and consequently the edge numbers given to them by R
    test_network <- igraph::graph_from_literal("node1" - +"node0", "node0" - +"node1")
    test_network <- igraph::set_edge_attr(test_network, "Weight", 1, -1)
    test_network <- igraph::set_edge_attr(test_network, "Weight", 2, 1)
    path <- igraph::shortest_paths(test_network, 1, 2, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path$vpath)) {
        path <- lapply(path$vpath, as.vector)
    }
    interaction3 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path)
    checkEquals(interaction3, -1)
    path2 <- igraph::shortest_paths(test_network, 2, 1, "out")
    # Fix for igraph 1.0.1
    if (!is.null(path2$vpath)) {
        path2 <- lapply(path2$vpath, as.vector)
    }
    interaction4 <- CausalR:::DetermineInteractionTypeOfPath(test_network, path2)
    checkEquals(interaction4, 1)
}

test_GetMatrixOfCausalRelationshipsProducesTheCorrectOutput <- function() {
    test_network <- CreateSimpleNetworkForTest_ComputationalCausalGraphCreator()
    idsOfConnectedNodes <- c(1, 2, 3)
    matrixOfComputationalCausalRelationships <- CausalR:::GetMatrixOfCausalRelationships(1, test_network, idsOfConnectedNodes)
    checkEquals(typeof(matrixOfComputationalCausalRelationships), "double")
    checkEquals(length(matrixOfComputationalCausalRelationships), 3)
    checkEquals(matrixOfComputationalCausalRelationships[1], 1)
    checkEquals(matrixOfComputationalCausalRelationships[2], 1)
    checkEquals(matrixOfComputationalCausalRelationships[3], -1)
}

test_GetMatrixOfCausalRelationshipsWhenIDsOfNodesInNetworkDontMatchTheInternalReference <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    network <- igraph::set_edge_attr(network, "Weight", 1, 1)
    network <- igraph::set_edge_attr(network, "Weight", 2, -1)
    network <- igraph::set_vertex_attr(network, "ID", 1:3, c(17, 6, 54))
    idsOfConnectedNodes <- c(1, 2, 3)
    matrixOfComputationalCausalRelationships <- CausalR:::GetMatrixOfCausalRelationships(1, network, idsOfConnectedNodes)
    checkEquals(typeof(matrixOfComputationalCausalRelationships), "double")
    checkEquals(length(matrixOfComputationalCausalRelationships), 3)
    checkEquals(matrixOfComputationalCausalRelationships[1], 1)
    checkEquals(matrixOfComputationalCausalRelationships[2], 1)
    checkEquals(matrixOfComputationalCausalRelationships[3], -1)
}

test_NetworkWhereConflictOccursInSecondLevel <- function() {
    test_network <- CausalR::CreateCG(system.file("testData", "network_with_conflict.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2, 3, 4)
    predictions <- CausalR::MakePredictionsFromCG(1, +1, test_network, 2, nodesInExperimentalData)
    # Node 3 is activated by the path 0 -> 1 -> 3 and inhibited by the path 0 -> 2 -| 3 Therefore it should have a value of 0 in the predictions
    checkEquals(predictions[4, 1], 4)
    checkEquals(predictions[4, 2], 0)
}

test_NetworkWhereConflictOccursInFirstLevel <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2", "node1" - +"node3", "node2" - +"node3")
    network <- igraph::set_edge_attr(network, "Weight", 1, 1)
    network <- igraph::set_edge_attr(network, "Weight", 2, -1)
    network <- igraph::set_edge_attr(network, "Weight", 3, 1)
    network <- igraph::set_edge_attr(network, "Weight", 4, 1)
    network <- igraph::set_vertex_attr(network, "ID", 1:4, 1:4)
    # The network has a conflict: node0 activates node3 via node1, but inhibits it via node2: Check that there are paths of length 2 between node0 and
    # node3:
    checkEquals(length(igraph::all_shortest_paths(network, "node0", "node3", "out")[[1]][[1]]), 3)
    # Check that weights are the same for node0 -> node1 and node1 -> node3
    checkEquals(igraph::edge_attr(network, "Weight", igraph::E(network)[igraph::`%->%`("node0", "node1")]), igraph::edge_attr(network, "Weight", igraph::E(network)[igraph::`%->%`("node1", 
        "node3")]))
    # Check that weights are opposite for node0 -> node2 and node2 -> node3
    checkEquals(igraph::edge_attr(network, "Weight", igraph::E(network)[igraph::`%->%`("node0", "node2")]), -igraph::edge_attr(network, "Weight", igraph::E(network)[igraph::`%->%`("node2", 
        "node3")]))
    # Now find the predictions, and check that it gives the right prediction for node3
    predictions <- CausalR::MakePredictionsFromCG("node0", +1, network, 2, 4)
    checkEquals(igraph::V(network)$name[predictions[1, 1]], "node3")
    checkEquals(predictions[1, 2], 0)
}



test_ConflictingData_difflength <- function() {
    test_network <- CausalR::CreateCG(system.file("testData", "network_with_difflength_conflict.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2, 3)
    predictions <- CausalR::MakePredictionsFromCG(1, +1, test_network, 2, nodesInExperimentalData)
    # Node 2 is activated by the path 0 -> 1 -> 2 and inhibited by the path 0 -| 2 Therefore it should have a value of -1 in the predictions (shorter
    # path)
    checkEquals(predictions[3, 1], 3)
    checkEquals(predictions[3, 2], -1)
}

test_SuppliedNodeNotInNetwork <- function() {
    test_network <- CausalR::CreateCG(system.file("testData", "network_with_conflict.sif", package = "CausalR"))
    checkException(predictions <- CausalR::MakePredictionsFromCG(10, +1, test_network, 2, nodesInExperimentalData))
}

test_MakePredictionsFromCGProducesTheCorrectNodeListWithPositiveRegulation <- function() {
    test_network <- CreateSimpleNetworkForTest_ComputationalCausalGraphCreator()
    nodesInExperimentalData <- c(1, 2, 3)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCG(1, +1, test_network, 3, nodesInExperimentalData)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 3)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 1)
    checkEquals(matrixOfCausalRelationships[2, 1], 2)
    checkEquals(matrixOfCausalRelationships[3, 1], 3)
    checkEquals(matrixOfCausalRelationships[1, 2], 1)
    checkEquals(matrixOfCausalRelationships[2, 2], 1)
    checkEquals(matrixOfCausalRelationships[3, 2], -1)
}

test_MakePredictionsFromCGProducesTheCorrectNodeListWithNegativeRegulation <- function() {
    test_network <- CreateSimpleNetworkForTest_ComputationalCausalGraphCreator()
    nodesInExperimentalData <- c(1, 2, 3)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCG(1, -1, test_network, 3, nodesInExperimentalData)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 3)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 1)
    checkEquals(matrixOfCausalRelationships[2, 1], 2)
    checkEquals(matrixOfCausalRelationships[3, 1], 3)
    checkEquals(matrixOfCausalRelationships[1, 2], -1)
    checkEquals(matrixOfCausalRelationships[2, 2], -1)
    checkEquals(matrixOfCausalRelationships[3, 2], 1)
}

test_MakePredictionsFromCGProducesTheCorrectNodeListWhenTheHypothesisNodeIsNotConnectedToAnyOther <- function() {
    # By 'not connected to any other', it is meant that this node has nodes following on from it in the directed graph
    test_network <- CreateSimpleNetworkForTest_ComputationalCausalGraphCreator()
    nodesInExperimentalData <- c(1, 2, 3)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCG(2, +1, test_network, 3, nodesInExperimentalData)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 1)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 2)
    checkEquals(matrixOfCausalRelationships[1, 2], 1)
}

test_MakePredictionsFromCGWorksWithBrokenLoops <- function() {
    test_network <- CausalR::CreateCG(system.file("testData", "network_with_loop.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2)
    # Network is 0 -> 1 -> 0. predictions should break the loop i.e. return 0 -> 1 even if asked to go to depth > 1.  First check that it works for
    # following one link:
    predictions <- CausalR::MakePredictionsFromCG(1, +1, test_network, 1, nodesInExperimentalData)
    checkEquals(typeof(predictions), "double")
    checkEquals(ncol(predictions), 2)
    # Then check we get the same answer following two links:
    predictions <- CausalR::MakePredictionsFromCG(1, +1, test_network, 2, nodesInExperimentalData)
    checkEquals(ncol(predictions), 2)
}

test_MakePredictionsFromCGReturnsNullWhenANodeWithNoConnectionsInTheExperimentalDataIsInput <- function() {
    # The function MakePredictionsFromCG should return null when supplied with a node with no connected nodes in the experimental data list (this
    # includes the root node itself)
    test_network <- igraph::graph_from_literal("node0" - +"node1", "node1" - +"node0")
    test_network <- igraph::set_edge_attr(test_network, "Weight", 1, 1)
    test_network <- igraph::set_edge_attr(test_network, "Weight", 2, -1)
    nodesInExperimentalData <- c(3, 4)
    checkTrue(is.null(CausalR::MakePredictionsFromCG(1, +1, test_network, 1, nodesInExperimentalData)))
}

test_FindIdsOfConnectedNodesInSubgraphWorksCorrectly <- function() {
    subgraph <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    subgraph <- igraph::set_edge_attr(subgraph, "Weight", 1, 1)
    subgraph <- igraph::set_edge_attr(subgraph, "Weight", 2, -1)
    subgraph <- igraph::set_vertex_attr(subgraph, "ID", 1:3, c(10, 19, 37))
    connectedNodes <- c(19, 37, 10)
    idsOfNodesInSubgraph <- CausalR:::FindIdsOfConnectedNodesInSubgraph(connectedNodes, subgraph)
    checkEquals(length(idsOfNodesInSubgraph), 3)
    checkEquals(idsOfNodesInSubgraph[1], 2)
    checkEquals(idsOfNodesInSubgraph[2], 3)
    checkEquals(idsOfNodesInSubgraph[3], 1)
}

test_MakePredictionsFromCCGProducesTheCorrectNodeListWithPositiveRegulation <- function() {
    test_double_network <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2, 3)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCCG(1, +1, test_double_network, 3, nodesInExperimentalData)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 3)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 1)
    checkEquals(matrixOfCausalRelationships[2, 1], 2)
    checkEquals(matrixOfCausalRelationships[3, 1], 3)
    checkEquals(matrixOfCausalRelationships[1, 2], 1)
    checkEquals(matrixOfCausalRelationships[2, 2], 1)
    checkEquals(matrixOfCausalRelationships[3, 2], -1)
}


test_MakePredictionsFromCCGProducesTheCorrectNodeListWhenTheHypothesisNodeIsNotConnectedToAnyOther <- function() {
    # By 'not connected to any other', it is meant that this node has nodes following on from it in the directed graph
    test_double_network <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2, 3)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCCG(2, +1, test_double_network, 3, nodesInExperimentalData)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 1)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 2)
    checkEquals(matrixOfCausalRelationships[1, 2], 1)
}

test_MakePredictionsFromCCGWorksWithBrokenLoops <- function() {
    test_double_network <- CausalR::CreateCG(system.file("testData", "network_with_loop.sif", package = "CausalR"))
    nodesInExperimentalData <- c(1, 2)
    # Network is 0 -> 1 -> 0. predictions should break the loop i.e. return 0 -> 1 even if asked to go to depth > 1.  First check that it works for
    # following one link:
    predictions <- CausalR::MakePredictionsFromCCG(1, +1, test_double_network, 1, nodesInExperimentalData)
    checkEquals(typeof(predictions), "double")
    checkEquals(ncol(predictions), 2)
    # Then check we get the same answer following two links:
    predictions <- CausalR::MakePredictionsFromCCG(1, +1, test_double_network, 2, nodesInExperimentalData)
    checkEquals(ncol(predictions), 2)
}

test_MakePredictionsFromCCGReturnsNullWhenANodeWithNoConnectionsInTheExperimentalDataIsInput <- function() {
    # The function MakePredictionsFromCG should return null when supplied with a node with no connected nodes in the experimental data list (this
    # includes the root node itself)
    test_double_network <- igraph::graph_from_literal("node0+" - +"node1+", "node1+" - +"node0-", "node0-" - +"node1-", "node1-" - +"node0+")
    nodesInExperimentalData <- c(3, 4)
    checkTrue(is.null(CausalR::MakePredictionsFromCCG(1, +1, test_double_network, 1, nodesInExperimentalData)))
}

test_MakePredictionsFromCCGWorksOnANetworkWithMoreThanOneLevel <- function() {
    test_CCG_with_two_levels <- CausalR::CreateCCG(system.file("testData", "CCGWithTwoLevels.txt", package = "CausalR"))
    
    # Test when delta = 1 (signOfHypothesis = 1)
    matrixOfCausalRelationships <- CausalR::MakePredictionsFromCCG("node0", +1, test_CCG_with_two_levels, 1)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships)[1], 3)
    checkEquals(dim(matrixOfCausalRelationships)[2], 2)
    checkEquals(matrixOfCausalRelationships[1, 1], 1)
    checkEquals(matrixOfCausalRelationships[2, 1], 2)
    checkEquals(matrixOfCausalRelationships[3, 1], 3)
    checkEquals(matrixOfCausalRelationships[1, 2], 1)
    checkEquals(matrixOfCausalRelationships[2, 2], 1)
    checkEquals(matrixOfCausalRelationships[3, 2], 1)
    
    # Test when delta = 2 (signOfHypothesis = -1)
    matrixOfCausalRelationships2 <- CausalR::MakePredictionsFromCCG("node0", -1, test_CCG_with_two_levels, 2)
    checkEquals(typeof(matrixOfCausalRelationships), "double")
    checkEquals(dim(matrixOfCausalRelationships2)[1], 6)
    checkEquals(dim(matrixOfCausalRelationships2)[2], 2)
    checkEquals(matrixOfCausalRelationships2[1, 1], 1)
    checkEquals(matrixOfCausalRelationships2[2, 1], 2)
    checkEquals(matrixOfCausalRelationships2[3, 1], 3)
    checkEquals(matrixOfCausalRelationships2[4, 1], 4)
    checkEquals(matrixOfCausalRelationships2[5, 1], 5)
    checkEquals(matrixOfCausalRelationships2[6, 1], 6)
    checkEquals(matrixOfCausalRelationships2[1, 2], -1)
    checkEquals(matrixOfCausalRelationships2[2, 2], -1)
    checkEquals(matrixOfCausalRelationships2[3, 2], -1)
    checkEquals(matrixOfCausalRelationships2[4, 2], 1)
    checkEquals(matrixOfCausalRelationships2[5, 2], 1)
    checkEquals(matrixOfCausalRelationships2[6, 2], -1)
} 

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-06 21:19:06 +0000 (Sun, 06 Jan 2013) $
# Changed By:  	$Author: rild $
# Version:		$Revision: 389 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkFileReader/CreateTestTableForTest_SifFileReader.r $
# 
# *** Summary ***
# Description: Create a table to be used in the testsuite for CreateCG

CreateTestTableForTest_SifFileReader <- function(){
  matrix1 <- matrix(c("node0","node0","Activates","Inhibits","node1","node2"), nrow =2)
  tab <- as.table(matrix1)
  
  return(tab)
}

# Copyright: Copyright 2012 GSK. All rights reserved *** SVN *** LastChanged: $Date: 2016-09-21 17:28:25 +0100 (Wed, 21 Sep 2016) $ Changed By:
# $Author: pipm $ Version: $Revision: 636 $ Source: $HeadURL: https://stevenagefs:7777/7412/trunk/NetworkFileReader/test_SifFileReader.R $ ***
# Summary *** Description: A test function for the CreateCG and CreateCCG functions

# Create a table to be used in the testsuite for CreateCG
CreateTestTableForTest_SifFileReader <- function() {
    matrix1 <- matrix(c("node0", "node0", "Activates", "Inhibits", "node1", "node2"), nrow = 2)
    tab <- as.table(matrix1)
    
    return(tab)
}

# Create a network for use in the testsuite for CreateCG
CreateNetworkForTest_SifFileReader <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    
    return(network)
}

# Create interactionInfo table for use in the testsuite for CreateCG
CreateInteractionInfoTableForTest_SifFileReader <- function() {
    matrix1 <- matrix(c("Activates", "Inhibits"))
    tab <- as.table(matrix1)
    
    return(tab)
}

# A helper file for the function Test_SifFileReader
CheckEdgeForTest_SifFileReader <- function(edge, vertex1, vertex2) {
    checkEquals(edge[1], vertex1) && checkEquals(edge[2], vertex2)
}

test_ReadSifFileToTableReadsInformationCorrectly <- function() {
    tableOfConnections <- CausalR:::ReadSifFileToTable(system.file("testData", "test_network.sif", package = "CausalR"))
    checkEquals(dim(tableOfConnections)[1], 2)
    checkEquals(dim(tableOfConnections)[2], 3)
    checkEquals(tableOfConnections[1, 1], "node0")
    checkEquals(tableOfConnections[1, 1], "node0")
    checkEquals(tableOfConnections[1, 2], "Activates")
    checkEquals(tableOfConnections[1, 3], "node1")
    checkEquals(tableOfConnections[2, 1], "node0")
    checkEquals(tableOfConnections[2, 2], "Inhibits")
    checkEquals(tableOfConnections[2, 3], "node2")
}

test_CreateNetworkFromTableProducesCorrectNetwork <- function() {
    tableOfConnections <- CreateTestTableForTest_SifFileReader()
    test_network <- CausalR:::CreateNetworkFromTable(tableOfConnections)
    networkEdges = igraph::E(test_network)
    networkEdgeList = igraph::as_edgelist(test_network)
    checkEquals(dim(networkEdgeList)[1], 2) && checkEquals(dim(networkEdgeList)[2], 2)
    # Check node 0 is connected to node 1
    CheckEdgeForTest_SifFileReader(networkEdgeList[1, ], "node0", "node1")
    # Check node 0 is connected to node 2
    CheckEdgeForTest_SifFileReader(networkEdgeList[2, ], "node0", "node2")
    # Check incorrect request fails
    checkTrue(!(networkEdgeList[1, 2] == "node3" && networkEdgeList[1, 1] == "node0"))
}

test_GetInteractionInformationProducesCorrectTable <- function() {
    tableOfConnections <- CreateTestTableForTest_SifFileReader()
    interactionInfo <- CausalR:::GetInteractionInformation(tableOfConnections)
    checkEquals(interactionInfo[[1]], "Activates")
    checkEquals(interactionInfo[[2]], "Inhibits")
    checkEquals(length(interactionInfo), 2)
}

test_GetWeightsFromInteractionInformation <- function() {
    interactionInfo <- c("Activates", "Inhibits", "Activates", "Activates", "Inhibits")
    weightInformation <- CausalR:::GetWeightsFromInteractionInformation(interactionInfo)
    checkEquals(nrow(weightInformation), 5)
    checkEquals(ncol(weightInformation), 1)
    checkEquals(weightInformation[1], 1)
    checkEquals(weightInformation[2], -1)
    checkEquals(weightInformation[3], 1)
    checkEquals(weightInformation[4], 1)
    checkEquals(weightInformation[5], -1)
}

test_AddEdgeInformationAssignsCorrectWeights <- function() {
    tableOfConnections <- CreateTestTableForTest_SifFileReader()
    test_network <- CreateNetworkForTest_SifFileReader()
    interactionInformation <- CreateInteractionInfoTableForTest_SifFileReader()
    test_network <- CausalR:::AddWeightsToEdges(test_network, interactionInformation)
    edge1Weight = igraph::edge_attr(test_network, "Weight", 1)
    checkEquals(edge1Weight, 1)
    edge2Weight = igraph::edge_attr(test_network, "Weight", 2)
    checkEquals(edge2Weight, -1)
}

test_AddIDsToVerticesWorksCorrectly <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node1" - +"node3", "node1" - +"node2")
    # To make the test realistic the edge attributes are added to the network
    network <- igraph::set_edge_attr(network, "Weight", 1, 1)
    network <- igraph::set_edge_attr(network, "Weight", 2, -1)
    network <- igraph::set_edge_attr(network, "Weight", 3, 1)
    network <- CausalR:::AddIDsToVertices(network)
    checkEquals(igraph::vertex_attr(network, "ID", "node0"), 1)
    checkEquals(igraph::vertex_attr(network, "ID", "node1"), 2)
    checkEquals(igraph::vertex_attr(network, "ID", "node3"), 3)
    checkEquals(igraph::vertex_attr(network, "ID", "node2"), 4)
}

test_ReadSifFileFailsWhenGivenANonsenseFile <- function() {
    checkException(CausalR::CreateCG("nonExistentFile.sif"))
}

test_ReadSifFileCreatesTheCorrectNetwork <- function() {
    network <- CausalR::CreateCG(system.file("testData", "test_network.sif", package = "CausalR"))
    checkEquals(igraph::gorder(network), 3)
    checkEquals(igraph::gsize(network), 2)
    checkEquals(igraph::vertex_attr(network, "ID", "node0"), 1)
    checkEquals(igraph::vertex_attr(network, "ID", "node1"), 2)
    checkEquals(igraph::vertex_attr(network, "ID", "node2"), 3)
    checkEquals(igraph::edge_attr(network, "Weight", 1), 1)
    checkEquals(igraph::edge_attr(network, "Weight", 2), -1)
}

test_CreateDoubleNetworkWorksCorrectly. <- function() {
    network <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    checkEquals(igraph::gorder(network), 6)
    checkEquals(igraph::gsize(network), 4)
    checkEquals(igraph::V(network)[1]$name, "node0+")
    checkEquals(igraph::V(network)[2]$name, "node1+")
    checkEquals(igraph::V(network)[3]$name, "node2+")
    checkEquals(igraph::V(network)[4]$name, "node0-")
    checkEquals(igraph::V(network)[5]$name, "node1-")
    checkEquals(igraph::V(network)[6]$name, "node2-")
}

test_CreateDoubleNetworkFailsWhenGivenANonsenseFile <- function() {
    checkException(CausalR::CreateCCG("nonExistentFile.sif"))
} 

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-06 21:19:06 +0000 (Sun, 06 Jan 2013) $
# Changed By:  	$Author: rild $
# Version:		$Revision: 389 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkFileReader/CreateNetworkForTest_SifFileReader.r $
# 
# *** Summary ***
# Description: Create a network for use in the testsuite for CreateCG

CreateNetworkForTest_SifFileReader <- function(){
  network <- graph.formula( "node0" -+ "node1", "node0" -+ "node2")
  return(network)
}

CreateCGForTest_RankTheHypotheses <- function(){

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-16 10:51:07 +0000 (Wed, 16 Jan 2013) $
# Changed By:  	$Author: MISC $
# Version:		$Revision: 429 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/RankingHypotheses/CreateCGForTest_RankTheHypotheses.r $
# 
# *** Summary ***
# Description: A helper function that will create the CG for the purpose of testing in RankTheHypotheses

  # Create test_network
  test_network <- graph.formula( "node0" -+ "node1", "node0" -+ "node2")
  test_network <- set.edge.attribute(test_network,"Interaction", 1, "Activates")
  test_network <- set.edge.attribute(test_network,"Interaction", 2, "Inhibits")
  test_network <- set.edge.attribute(test_network,"Weight", 1, 1)
  test_network <- set.edge.attribute(test_network,"Weight", 2, -1)
  test_network <- set.vertex.attribute(test_network,"ID", 1:3, 1:3)
  test_network$isCCG <- FALSE
  
  return(test_network)
}

# Test suite for the hypothesis ranking and ordering function (which also acts as the top-level interface to the whole calculation)

# Copyright: Copyright 2012 - 2013 GSK. All rights reserved

# *** SVN *** LastChanged: $Date: 2016-08-24 08:36:18 +0100 (Wed, 24 Aug 2016) $ Changed By: $Author: pipm $ Version: $Revision: 627 $ Source:
# $HeadURL: https://stevenagefs:7777/7412/trunk/RankingHypotheses/test_RankTheHypotheses.R $

# A helper function that will create the CG for the purpose of testing in RankTheHypotheses
CreateCGForTest_RankTheHypotheses <- function() {
    
    # Create test_network
    test_network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    test_network <- igraph::set_edge_attr(test_network, "Interaction", 1, "Activates")
    test_network <- igraph::set_edge_attr(test_network, "Interaction", 2, "Inhibits")
    test_network <- igraph::set_edge_attr(test_network, "Weight", 1, 1)
    test_network <- igraph::set_edge_attr(test_network, "Weight", 2, -1)
    test_network <- igraph::set_vertex_attr(test_network, "ID", 1:3, 1:3)
    test_network$isCCG <- FALSE
    
    return(test_network)
}

test_RankTheHypothesesProducesTheCorrectOutputWithCG <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesProducesTheCorrectOutputWithCCG <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}


test_RankTheHypothesesWritesFileWhenWriteFileIsTrue <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    
    CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, outputDir = outputDir)
    
    # Check one file is produced
    checkEquals(length(list.files(outputDir)), 1)
    
    # Check content is correct
    resultsTable <- read.table(paste0(outputDir, "//ResultsTable-test_network-test_ExperimentalData-delta2.txt"), stringsAsFactors = FALSE, header = TRUE, sep="\t")
    
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 9)
    checkEquals(resultsTable[1, 1], "node0")
    checkEquals(resultsTable[1, 2], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 3], 1)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 3)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], 0)
    checkEquals(resultsTable[1, 8], (1/3))
    checkEquals(resultsTable[1, 9], 1)
    checkEquals(resultsTable[2, 1], "node1")
    checkEquals(resultsTable[2, 2], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 1)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 0)
    checkEquals(resultsTable[2, 8], 2/3)
    checkEquals(resultsTable[2, 9], 1)
    checkEquals(resultsTable[3, 1], "node2")
    checkEquals(resultsTable[3, 2], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 0)
    checkEquals(resultsTable[3, 8], 1/3)
    checkEquals(resultsTable[3, 9], 1)
    checkEquals(resultsTable[4, 1], "node2")
    checkEquals(resultsTable[4, 2], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 3], 1)
    checkEquals(resultsTable[4, 4], -1)
    checkEquals(resultsTable[4, 5], 0)
    checkEquals(resultsTable[4, 6], 1)
    checkEquals(resultsTable[4, 7], 0)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[4, 9], 1)
    checkEquals(resultsTable[5, 1], "node1")
    checkEquals(resultsTable[5, 2], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], -1)
    checkEquals(resultsTable[5, 5], 0)
    checkEquals(resultsTable[5, 6], 1)
    checkEquals(resultsTable[5, 7], 0)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[5, 9], 1)
    checkEquals(resultsTable[6, 1], "node0")
    checkEquals(resultsTable[6, 2], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 3], -1)
    checkEquals(resultsTable[6, 4], -3)
    checkEquals(resultsTable[6, 5], 0)
    checkEquals(resultsTable[6, 6], 3)
    checkEquals(resultsTable[6, 7], 0)
    checkEquals(resultsTable[6, 8], 1)
    checkEquals(resultsTable[6, 9], 1)
}


test_RankTheHypothesesDoesntWriteFileWhenWriteFileIsFalse <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    
    CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, outputDir = outputDir, writeFile = FALSE)
    
    # Check no file is produced
    checkEquals(length(list.files(outputDir)), 0)
}


test_RankTheHypothesesProducesTheCorrectOutputWithCGUsingQuarticAlgorithm <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, useCubicAlgorithm = FALSE)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesProducesTheCorrectOutputWithCCGUsingQuarticAlgorithm <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, useCubicAlgorithm = FALSE)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_OrderHypothesesProducesTheCorrectResult <- function() {
    matrix1 <- matrix(c("GeneA", "GeneB", "GeneC", 1, -1, 1, 1, 3, 2), nrow = 3)
    orderedTable <- CausalR:::OrderHypotheses(matrix1)
    checkEquals(orderedTable[1, 1], "GeneB")
    checkEquals(orderedTable[1, 2], "-1")
    checkEquals(orderedTable[1, 3], "3")
    checkEquals(orderedTable[2, 1], "GeneC")
    checkEquals(orderedTable[2, 2], "1")
    checkEquals(orderedTable[2, 3], "2")
    checkEquals(orderedTable[3, 1], "GeneA")
    checkEquals(orderedTable[3, 2], "1")
    checkEquals(orderedTable[3, 3], "1")
}

test_AnalyseExperimentalData <- function() {
    expData <- matrix(c(1, 2, 3, 4, 5, 6, 7, -1, 1, 1, 0, -1, 1, +1), ncol = 2)
    expDataStats <- CausalR:::AnalyseExperimentalData(expData)
    checkEquals(expDataStats[1], 4)
    checkEquals(expDataStats[2], 2)
    checkEquals(expDataStats[3], 1)
}

test_RankTheHypothesesWillWorkWithANonSymmetricCCG <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1+"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2-"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2+"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1-"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -2)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 2)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenASingleNodeIDInTheSpecifiedListOfNodes <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 1)
    checkEquals(resultsTable[1, 4], 1)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (2/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -1)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 1)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenASingleNameInTheSpecifiedListOfNodes <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = "node0")
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -3)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 3)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenMoreThanOneNodeInTheSpecifiedListOfNodes <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = c("node0", "node1"))
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -3)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 3)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenASingleNodeIDInTheSpecifiedListOfNodes <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 1)
    checkEquals(resultsTable[1, 4], 1)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (2/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -1)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 1)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenASingleNameInTheSpecifiedListOfNodes <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = "node0")
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -3)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 3)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenMoreThanOneNodeInTheSpecifiedListOfNodes <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = c("node0", "node1"))
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -3)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 3)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesIsAbleToFilterByTheNumberOfCorrectPredictionsForACCG <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, correctPredictionsThreshold = 0)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkTrue(is.na(resultsTable[4, 7]))
    checkTrue(is.na(resultsTable[4, 8]))
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkTrue(is.na(resultsTable[5, 7]))
    checkTrue(is.na(resultsTable[5, 8]))
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkTrue(is.na(resultsTable[6, 7]))
    checkTrue(is.na(resultsTable[6, 8]))
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithSingleNodeIDInTheSpecifiedNodesList <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = 1)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -2)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 2)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithSingleNameInTheSpecifiedNodesList <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = "node0")
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -2)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 2)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithMoreThanOneNodeInTheSpecifiedNodesList <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = c(1, 2))
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1+"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node1-"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -2)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 2)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesHandlesNoOverlap <- function() {
    # Test that RankTheHypotheses works correctly if there is no overlap between the predictions and the nodes in the experimental data.
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # call RankTheHypotheses looking at node2, but only providing experimental data on node0 and node1
    output <- CausalR::RankTheHypotheses(test_CCG, experimentalData[1:2, ], 2, listOfNodes = "node2")
    # the calculated part of output should be all zeros, with 1 for the p values
    checkTrue(all(output[, 3:6] == 0))
    checkTrue(all(output[, 7:8] == 1))
} 


###########################################################
## Same again in parallel
## Note: CRAN checks only allow tests and examples to use 2 cores
###########################################################

test_RankTheHypothesesProducesTheCorrectOutputWithCGInParallelMode <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesProducesTheCorrectOutputWithCCGInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesProducesTheCorrectOutputWithCGUsingQuarticAlgorithmInParallelMode <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, useCubicAlgorithm = FALSE, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_TheHypothesesProducesTheCorrectOutputWithCCGUsingQuarticAlgorithmInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, useCubicAlgorithm = FALSE, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesWillWorkWithANonSymmetricCCGInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1+"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2-"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2+"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1-"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkEquals(resultsTable[5, 7], 1)
    checkEquals(resultsTable[5, 8], 1)
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -2)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 2)
    checkEquals(resultsTable[6, 6], 0)
    checkEquals(resultsTable[6, 7], 1)
    checkEquals(resultsTable[6, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenASingleNodeIDInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = 2, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 1)
    checkEquals(resultsTable[1, 4], 1)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (2/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -1)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 1)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenASingleNameInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = "node0", doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -3)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 3)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACCGWhenGivenMoreThanOneNodeInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, listOfNodes = c("node0", "node1"), doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -3)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 3)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenASingleNodeIDInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = 2, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 1)
    checkEquals(resultsTable[1, 4], 1)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (2/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -1)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 1)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenASingleNameInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = "node0", doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -3)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 3)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithACGWhenGivenMoreThanOneNodeInTheSpecifiedListOfNodesInParallelMode <- function() {
    test_CG <- CreateCGForTest_RankTheHypotheses()
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CG)
    resultsTable <- CausalR::RankTheHypotheses(test_CG, experimentalData, 2, listOfNodes = c("node0", "node1"), doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CG, "node1"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CG, "node0"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -3)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 3)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesIsAbleToFilterByTheNumberOfCorrectPredictionsForACCGInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, correctPredictionsThreshold = 0, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 6)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], 1)
    checkEquals(resultsTable[3, 4], 1)
    checkEquals(resultsTable[3, 5], 0)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1/3)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node2"))
    checkEquals(resultsTable[4, 2], 1)
    checkEquals(resultsTable[4, 3], -1)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 1)
    checkEquals(resultsTable[4, 6], 0)
    checkTrue(is.na(resultsTable[4, 7]))
    checkTrue(is.na(resultsTable[4, 8]))
    checkEquals(resultsTable[5, 1], CausalR:::GetNodeID(test_CCG, "node1"))
    checkEquals(resultsTable[5, 2], -1)
    checkEquals(resultsTable[5, 3], -1)
    checkEquals(resultsTable[5, 4], 0)
    checkEquals(resultsTable[5, 5], 1)
    checkEquals(resultsTable[5, 6], 0)
    checkTrue(is.na(resultsTable[5, 7]))
    checkTrue(is.na(resultsTable[5, 8]))
    checkEquals(resultsTable[6, 1], CausalR:::GetNodeID(test_CCG, "node0"))
    checkEquals(resultsTable[6, 2], -1)
    checkEquals(resultsTable[6, 3], -3)
    checkEquals(resultsTable[6, 4], 0)
    checkEquals(resultsTable[6, 5], 3)
    checkEquals(resultsTable[6, 6], 0)
    checkTrue(is.na(resultsTable[6, 7]))
    checkTrue(is.na(resultsTable[6, 8]))
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithSingleNodeIDInTheSpecifiedNodesListInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = 1, doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -2)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 2)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithSingleNameInTheSpecifiedNodesListInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = "node0", doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 2)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[2, 2], -1)
    checkEquals(resultsTable[2, 3], -2)
    checkEquals(resultsTable[2, 4], 0)
    checkEquals(resultsTable[2, 5], 2)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 1)
    checkEquals(resultsTable[2, 8], 1)
}

test_RankTheHypothesesWorksWithNonSymetricCCGWithMoreThanOneNodeInTheSpecifiedNodesListInParallelMode <- function() {
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # Delete an edge to make the graph non-symmetric
    test_CCG <- igraph::delete_edges(test_CCG, 4)
    resultsTable <- CausalR::RankTheHypotheses(test_CCG, experimentalData, 2, symmetricCCG = FALSE, listOfNodes = c(1, 2), doParallel = TRUE, numCores = 2)
    checkEquals(dim(resultsTable)[1], 4)
    checkEquals(dim(resultsTable)[2], 8)
    checkEquals(resultsTable[1, 1], CausalR:::GetNodeID(test_CCG, "node0+"))
    checkEquals(resultsTable[1, 2], 1)
    checkEquals(resultsTable[1, 3], 3)
    checkEquals(resultsTable[1, 4], 3)
    checkEquals(resultsTable[1, 5], 0)
    checkEquals(resultsTable[1, 6], 0)
    checkEquals(resultsTable[1, 7], (1/3))
    checkEquals(resultsTable[1, 8], 1)
    checkEquals(resultsTable[2, 1], CausalR:::GetNodeID(test_CCG, "node1+"))
    checkEquals(resultsTable[2, 2], 1)
    checkEquals(resultsTable[2, 3], 1)
    checkEquals(resultsTable[2, 4], 1)
    checkEquals(resultsTable[2, 5], 0)
    checkEquals(resultsTable[2, 6], 0)
    checkEquals(resultsTable[2, 7], 2/3)
    checkEquals(resultsTable[2, 8], 1)
    checkEquals(resultsTable[3, 1], CausalR:::GetNodeID(test_CCG, "node1-"))
    checkEquals(resultsTable[3, 2], -1)
    checkEquals(resultsTable[3, 3], -1)
    checkEquals(resultsTable[3, 4], 0)
    checkEquals(resultsTable[3, 5], 1)
    checkEquals(resultsTable[3, 6], 0)
    checkEquals(resultsTable[3, 7], 1)
    checkEquals(resultsTable[3, 8], 1)
    checkEquals(resultsTable[4, 1], CausalR:::GetNodeID(test_CCG, "node0-"))
    checkEquals(resultsTable[4, 2], -1)
    checkEquals(resultsTable[4, 3], -2)
    checkEquals(resultsTable[4, 4], 0)
    checkEquals(resultsTable[4, 5], 2)
    checkEquals(resultsTable[4, 6], 0)
    checkEquals(resultsTable[4, 7], 1)
    checkEquals(resultsTable[4, 8], 1)
}

test_RankTheHypothesesHandlesNoOverlapInParallelMode <- function() {
    # Test that RankTheHypotheses works correctly if there is no overlap between the predictions and the nodes in the experimental data.
    test_CCG <- CausalR::CreateCCG(system.file("testData", "test_network.sif", package = "CausalR"))
    experimentalData <- CausalR::ReadExperimentalData(system.file("testData", "test_ExperimentalData.txt", package = "CausalR"), test_CCG)
    # call RankTheHypotheses looking at node2, but only providing experimental data on node0 and node1
    output <- CausalR::RankTheHypotheses(test_CCG, experimentalData[1:2, ], 2, listOfNodes = "node2", doParallel = TRUE, numCores = 2)
    # the calculated part of output should be all zeros, with 1 for the p values
    checkTrue(all(output[, 3:6] == 0))
    checkTrue(all(output[, 7:8] == 1))
} 

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-03 13:22:53 +0000 (Thu, 03 Jan 2013) $
# Changed By:  	$Author: MISC $
# Version:		$Revision: 357 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkPredictions/CreateSimpleNetworkForTest_ComputationalCausalGraphCreator.r $
# 
# *** Summary ***
# Description: A helper function for Test_ComputationalCausalGraphCreator. It gets a simple network for the test

CreateSimpleNetworkForTest_ComputationalCausalGraphCreator <- function(){
network <- graph.formula( "node0" -+ "node1", "node0" -+ "node2")
network <- set.edge.attribute(network,"Weight", 1, 1)
network <- set.edge.attribute(network,"Weight", 2, -1)
network <- set.vertex.attribute(network,"ID", 1:3, 1:3)

return(network)
}

GetMatrixOfCausalRelationships <- function(hypothesis, network, idsOfConnectedNodesFromSubgraph){
  # Get matrix of Causal Relationships from the network and the IDs of connected nodes
  
  # *** SVN version information ***
  # $Date: 2013-01-03 18:03:15 +0000 (Thu, 03 Jan 2013) $
  # $Revision: 366 $
  # $Author: rild $
  # $HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkPredictions/GetMatrixOfCausalRelationships.r $
  
  # Copyright 2012 GSK. All rights reserved
  
  nNodes <- length(idsOfConnectedNodesFromSubgraph)
  
  causalRelationships <- array(0, nNodes)
  
  for (iNode in 1:nNodes){
    if (hypothesis == idsOfConnectedNodesFromSubgraph[iNode]){
      #  Hypothesis node's interaction with itself has to be 1.
      causalRelationships[iNode] <- 1
    }
    else{
      shortestPaths <- get.all.shortest.paths(network, hypothesis, idsOfConnectedNodesFromSubgraph[iNode], "out")
      numberOfShortestPaths <- length(shortestPaths[[1]])
      # Determine interaction type for first path
      interactionValue <- DetermineInteractionTypeOfPath(network, shortestPaths[[1]][1])
      
      if (numberOfShortestPaths > 1){
        # if there is more than one shortest path, two things can happen:
        # 	1. they could all agree, in which case we can take any of the values in the vector to populate the data frame
        # 	2. they could disagree, in this case the interaction given the value 0 (as opposed to 1/-1 for activation/inhibition.
        
        for (counter in 2:numberOfShortestPaths){
          interactionValueForAlternativePath <- DetermineInteractionTypeOfPath(network, shortestPaths[[1]][counter])
          if (interactionValueForAlternativePath != interactionValue){
            interactionValue <- 0
            break()
          }
        }  
      }
	  causalRelationships[iNode] <- interactionValue
    }
  }
  
  return(causalRelationships) 
}

RemoveIDsNotInExperimentalData <- function(connectedNodes, nodesInExperimentalData){

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2013-01-03 14:30:21 +0000 (Thu, 03 Jan 2013) $
# Changed By:  	$Author: MISC $
# Version:		$Revision: 361 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkPredictions/RemoveIDsNotInExperimentalData.r $
# 
# *** Summary ***
# Description: Takes in a list of connected nodes and removes
# those not in the experimental data.

connectedNodesInExperimentalData <- intersect(connectedNodes, nodesInExperimentalData)
return(connectedNodesInExperimentalData)
}

#' @title Get explained nodes of CCG
#' @description
#' Returns a table of node names and values for explained nodes, I.e. nodes that appear in both network and data with the same sign.
#' The table contain the name in column 1 and the value (1 or -1) in column 2

#' @param hypothesisnode a hypothesis node
#' @param signOfHypothesis the direction of change of hypothesis node
#' @param network a computational causal graph
#' @param experimentalData The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.
#' @param delta the number of edges across which the hypothesis should be followed
#' @return vector of explained nodes

GetExplainedNodesOfCCG <- function(hypothesisnode, signOfHypothesis, network, experimentalData, delta) {
    
    signOfHypothesis <- as.integer(signOfHypothesis)
    delta <- as.integer(delta)
    
    predictions <- MakePredictionsFromCCG(hypothesisnode, signOfHypothesis, network, delta)
    experimentalDataByName <- GetNodeName(network, experimentalData)
    predictionsByName <- GetNodeName(network, predictions)
    
    # Set the column names - needed before merge
    colnames(experimentalDataByName) <- c("name", "exp")
    colnames(predictionsByName) <- c("name", "pred")
    
    ## Merge the experimental data and prediction data tables by first (name) column to create a
    ## table containing all nodes from both the experimental and prediction data
    tableOfMatches <- merge.default(experimentalDataByName, predictionsByName, all=TRUE, by = c(TRUE, FALSE))

    # Tidy up tableOfMatches - convert from factors to strings then to integers
    i <- sapply(tableOfMatches, is.factor)
    tableOfMatches[i] <- lapply(tableOfMatches[i], as.character)
    tableOfMatches[,"name"] <- as.character(tableOfMatches[,"name"])
    tableOfMatches[,"exp"] <- as.integer(tableOfMatches[,"exp"])
    tableOfMatches[,"pred"] <- as.integer(tableOfMatches[,"pred"])

    # Filter to only nodes that appear in both experimental and predictions
    tableOfMatches <- tableOfMatches[apply(tableOfMatches, 1, function(x) !any(is.na(x))),]
    
    # Get nodes that have the same regulation value for experimental and predictions
    corExplainedNodes <- tableOfMatches[tableOfMatches[,"exp"]*tableOfMatches[,"pred"] == 1,]
    #corExplainedNodes <- corExplainedNodes[corExplainedNodes[,"name"] != hypothesisnode,]
    
    # Get nodes that have different and non-zero values for experimental and predictions
    incorExplainedNodes <- tableOfMatches[tableOfMatches[,"exp"]*tableOfMatches[,"pred"] == -1,]
    
    # Get nodes that are in experimental but not in predictions 
    ambExplainedNodes <- tableOfMatches[tableOfMatches[,"pred"] == 0 & tableOfMatches[,"exp"] != 0,]
    # ambExplainedNodes <- tableOfMatches[apply(tableOfMatches, 1, function(x) any(is.na(x))),]
    
    
    ## In table of matches column 1 contains the name, column 2 contains experimental data and
    ## column 3 contains predictions data.
    ## Explained nodes are those where the experimental value matches the prediction.
    ## Get a table of explained nodes with column 1 containing the names of the nodes
    ## and column 2 containing the value (+1 or -1)
    #explainedNodes <- tableOfMatches[tableOfMatches[,2]==tableOfMatches[,3],1:2]
    
    explainedNodes <- list(corExplainedNodes = corExplainedNodes, incorExplainedNodes = incorExplainedNodes, ambExplainedNodes = ambExplainedNodes)
    
    return(explainedNodes)
}

#' @title Get scores for single node
#' @description
#' A helper function for RankTheHypotheses to calculate a line of the scoresMatrix table
#' @param iNode this node
#' @param timeToRunSoFar the time to run so far
#' @param nodesToBeTested List of all nodes to be tested
#' @param network Computational Causal Graph, as an igraph.
#' @param delta Distance to search within the causal graph.
#' @param processedExperimentalData The processed experimental data
#' @param numPredictions The number of predictions
#' @param epsilon The threshold that is used when calculating the p-value using the cubic algorithm (see 'Assessing statistical significance in causal graphs').
#' @param useCubicAlgorithm An indicator specifying which algorithm will be used to calculate the p-value. The default is set as useCubicAlgorithm = TRUE which uses the cubic algorithm. If this value is set as FALSE, the algorithm will use the much slower quartic algorithm which does compute the exact answer, as opposed to using approximations like the cubic algorithm.
#' @param use1bAlgorithm An indicator specifying whether the 1a or 1b (default, faster) variant of the cubic algorithm described in Chindelevitch's paper will be used to calculate the p-value.
#' @param symmetricCCG This flag specifies whether the CCG is assumed to be symmetric. The value is set as TRUE as a default. If this is the case the running time of the algorithm is reduced since the negative node values can be calculated using symmetry and the results of calculations performed for the positive node
#' @param correctPredictionsThreshold A threshold on the number of correct predictions for a given hypothesis. If a hypothesis produces fewer correct predictions than predictionsThreshold then the algorithm will not calculate the two p-values. Instead 'NA' will be displayed in the final two columns of the corresponding row of the results table. As a default correctPredictionsThreshold is set as -Inf, so that the p-values are calculated for all specified hypotheses. Note: Set to Inf to turn off p-value calculations entirely.
#' @param experimentalDataStats Stats from the experimental data
#' @param quiet a flag to supress progress output
#' @return If symmetricCCG is false, this returns a single line of the scoreMatrix for the 'iNode'th node in nodesToBeTested. If symmetricCCG is true this returns two lines. The first of which corresponds to the positive node and the second the negative node.

GetScoresForSingleNode <- function(iNode, timeToRunSoFar, nodesToBeTested, network, delta, processedExperimentalData, numPredictions, epsilon, useCubicAlgorithm,
                                   use1bAlgorithm, symmetricCCG, correctPredictionsThreshold, experimentalDataStats, quiet) {
    
    ## Set up a matrix to hold rows that will be added to scoresMatrix in RankTheHypothesis
    if (symmetricCCG) {
        scoresMatrixRow <- matrix(0, 2, 8)
    } else {
        scoresMatrixRow <- matrix(0, 1, 8)
    }
    colnames(scoresMatrixRow) <- c("NodeID", "Regulation", "Score", "Correct", "Incorrect", "Ambiguous", "p-value", "Enrichment p-value")
    
    ## For this node get the predictions from the network within a given delta
    nodeID <- nodesToBeTested[iNode]
    
    scoresMatrixRow[1, "NodeID"] <- nodesToBeTested[iNode]
    
    ## Second column of scores matrix contains the value +1/-1 depending on whether we are testing the hypothesis node to be upregulated or downregulated.
    ## If not a symmetricCCG the second half of nodesToBeTested are downregulated
    if (!symmetricCCG && iNode > length(nodesToBeTested)/2) {
        scoresMatrixRow[1, "Regulation"] <- -1
    } else {
        scoresMatrixRow[1, "Regulation"] <- 1
    }
    
    prediction <- MakePredictions(hypothesisnode = scoresMatrixRow[1, "NodeID"], signOfHypothesis = 1,
                                  network = network, delta = delta, nodesInExperimentalData = processedExperimentalData[, 1])
    
    if (!is.null(prediction)) {
        # Get the values of q+, q-, q0 and get scoreBreakdown by comparing prediction to data
        predictionListStats <- AnalysePredictionsList(prediction, numPredictions)
        scoreBreakdown <- ScoreHypothesis(prediction, processedExperimentalData)

        scoresMatrixRow[1, c("Score", "Correct", "Incorrect", "Ambiguous")] <- scoreBreakdown
        
        # Only calculate remaining values if the number of correct predictions is below correctPredictionsThreshold
        if (scoresMatrixRow[1, "Correct"] > correctPredictionsThreshold) {
            scoresMatrixRow[1, "p-value"] <- CalculateSignificance(scoresMatrixRow[1, "Score"], predictionListStats, experimentalDataStats, epsilon, useCubicAlgorithm, use1bAlgorithm)
            scoresMatrixRow[1, "Enrichment p-value"] <- CalculateEnrichmentPValue(prediction, processedExperimentalData)
        } else {
            # If there aren't enough correct predictions, don't calculate the p-values just report NA
            scoresMatrixRow[1, "p-value"] <- NA
            scoresMatrixRow[1, "Enrichment p-value"] <- NA
        }
    } else {
        # No overlap between predictions and data, score will be 0 and this is the only score that can be achieved so P values should be 1
        scoresMatrixRow[1, "p-value"] <- 1
        scoresMatrixRow[1, "Enrichment p-value"] <- 1
    }
    
    
    ## If the network is symmetric the second half of the network (downregulated nodes) can be obtained by symmetry without having to re-calculate
    ## using MakePredictions. So if symmetric CCg network add another row and fill it with the scores. 
    if (symmetricCCG) {

        scoresMatrixRow[2, "NodeID"] <- scoresMatrixRow[1, "NodeID"]
        scoresMatrixRow[2, "Regulation"] <- -scoresMatrixRow[1, "Regulation"]
        
        # Second half of the table: downregulate the hypothesis node.  Predictions and scores are obtained by symmetry. The quantity
        # iNode+numNodesToBeTested is the iNode for the corresponding node in the CCG i.e. it would give the id of node1- for given the id of node1+
        if (!is.null(prediction)) {
            prediction[, 2] <- -prediction[, 2]
            scoresMatrixRow[2, "Score"] <- -scoresMatrixRow[1, "Score"]
            # For the opposite hypothesis the number of correct predictions becomes the number of incorrect and vice versa
            scoresMatrixRow[2, c("Correct", "Incorrect", "Ambiguous")] <- scoresMatrixRow[1, c("Incorrect", "Correct", "Ambiguous")]
            
            # Check the number of correct predictions, in the case of a symmetric CCG this is just the number of incorrect predictions for the opposite
            # hypothesis
            if (scoresMatrixRow[2, "Correct"] > correctPredictionsThreshold) {
                scoresMatrixRow[2, "p-value"] <- CalculateSignificance(scoresMatrixRow[2, "Score"], predictionListStats[c(2, 1, 3)], experimentalDataStats,
                    epsilon, useCubicAlgorithm, use1bAlgorithm
                )
                scoresMatrixRow[2, "Enrichment p-value"] <- CalculateEnrichmentPValue(prediction, processedExperimentalData)
            } else {
                # If there aren't enough correct predictions, don't calculate the p-values just report NA
                scoresMatrixRow[2, "p-value"] <- NA
                scoresMatrixRow[2, "Enrichment p-value"] <- NA
            }
        } else {
            # No overlap between predictions and data, score will be 0 and this is the only score that can be achieved so P values should be 1
            scoresMatrixRow[2, "p-value"] <- 1
            scoresMatrixRow[2, "Enrichment p-value"] <- 1
        }
    }
    
    if (!quiet && iNode %% 100 == 0) {
        #cat(paste0("Node Reached: ", iNode, ". "))
        #timeToRunNextHundred <- proc.time()[1]
        #cat(paste("Time to run this hundred nodes:", (timeToRunNextHundred - timeToRunSoFar), "s\n"))
        timeDiff <- Sys.time() - timeToRunSoFar
        cat(paste0("Time taken to reach Node ", iNode, ": ", round(timeDiff, 2), " ", units(timeDiff), "\n"))
        
        #timeToRunSoFar <- timeToRunNextHundred
    }
    
    return(scoresMatrixRow)
}

#' @title remove IDs not in experimental data
#' @description
#' Takes in a list of connected nodes and removes those not in the experimental data.
#' @param connectedNodes a list of connected nodes
#' @param nodesInExperimentalData a list of nodes in the experimental data
#' @return connectedNodesInExperimentalData a list of connected nodes with the redundant nodes removed


RemoveIDsNotInExperimentalData <- function(connectedNodes, nodesInExperimentalData) {
    
    connectedNodesInExperimentalData <- intersect(connectedNodes, nodesInExperimentalData)
    return(connectedNodesInExperimentalData)
} 

#' @title determine interaction type of path
#' @description 
#' Determines the sign of a given path.
#' Given a path and through the network, this function will determine if the path results in activation or inhibition.
#' Activation is indicated by 1, inhibition by -1 

#' @param network an igraph representing the network
#' @param nodesInPath an ordered list of the nodes visited on the path - note that these contain numbers which use 
#'     R's internal reference to the edges
#' @return a signed integer representing the paths sign

DetermineInteractionTypeOfPath <- function(network, nodesInPath) {
    # network must be an igraph nodesInPath is an ordered list of the nodes visited on the path - note that these contain numbers which use R's internal
    # reference to the edges
    
    numEdges <- length(nodesInPath[[1]]) - 1
    # Declare a variable that will store the numerical references for the edges in a path
    nodeEdges <- matrix(0, 1, numEdges)
    
    for (i in 1:numEdges) {
        # Get the R's edge number for two successive nodes in nodesInPath
        nodeEdges[i] <- igraph::E(network)[igraph::`%->%`(nodesInPath[[1]][i], nodesInPath[[1]][i + 1])][[1]]
    }
    # Take the product of the weights for each edge in the path to get the overall interaction
    netPathInteraction = prod(igraph::edge_attr(network, "Weight", nodeEdges))
    
    return(netPathInteraction)
}

 

#' @title get weight for numbers of correct and incorrect predictions
#' @description 
#' Gets the weight based on the values of n++, n+-, n-+ and n--.
#' @param n_pp the contingency table entry n++
#' @param n_pm the contingency table entry n+-
#' @param n_mp the contingency table entry n-+
#' @param n_mm the contingency table entry n--
#' @param predictionDataStats prediction data statistics
#' @param experimentalDataStats  experimental data statistics
#' @param logOfFactorialOfPredictionListStats  log of factorial of prediction list stats
#' @param returnlog true if the result should be returned as a log
#' @return none

GetWeightForNumbersOfCorrectandIncorrectPredictions <- function(n_pp, n_pm, n_mp, n_mm, predictionDataStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
    returnlog = FALSE) {
    
    # contingencyTableValues contains the values of n++, n+-, n-+ and n-- (in that order) - the top left of the contingency table
    threeByThreeContingencyTable <- PopulateTheThreeByThreeContingencyTable(n_pp, n_pm, n_mp, n_mm, predictionDataStats, experimentalDataStats)
    
    if (returnlog) {
        return(sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable)))
    } else {
        return(exp(sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable))))
    }
} 

# A test function for functions in Significance Calculator

test_GetNumberOfPositiveAndNegativeEntries <- function() {
    dataList <- matrix(c(1, 2, 3, 4, 5, 6, 7, -1, 1, 1, 0, -1, 1, +1), ncol = 2)
    dataStats <- CausalR:::GetNumberOfPositiveAndNegativeEntries(dataList)
    checkEquals(dataStats[1], 4)
    checkEquals(dataStats[2], 2)
}

test_AnalysePredictionsList <- function() {
    predictionData <- matrix(c(1, 2, 3, 4, 5, 6, 7, -1, 1, 1, 0, -1, -1, +1), ncol = 2)
    predictionDataStats <- CausalR:::AnalysePredictionsList(predictionData, length(predictionData[, 1]))
    checkEquals(predictionDataStats[1], 3)
    checkEquals(predictionDataStats[2], 3)
    checkEquals(predictionDataStats[3], 1)
}

test_GetScoreForNumbersOfCorrectandIncorrectPredictionsProducesCorrectOutput <- function() {
    checkEquals(CausalR:::GetScoreForNumbersOfCorrectandIncorrectPredictions(c(5, 2, 1, 2)), 4)
    checkEquals(CausalR:::GetScoreForNumbersOfCorrectandIncorrectPredictions(c(5, 7, 1, 2)), -1)
}

test_GetWeightForNumbersOfCorrectandIncorrectPredictionsProducesCorrectOutputWhenReturnLogIsFalse <- function() {
    logOfFactorialOfPredictionListStats <- c(log(factorial(3)), log(factorial(2)), log(factorial(0)))
    checkEquals(CausalR:::GetWeightForNumbersOfCorrectandIncorrectPredictions(1, 0, 1, 1, c(3, 2, 0), c(2, 1, 2), logOfFactorialOfPredictionListStats, 
        returnlog = FALSE), 6)
    checkEquals(CausalR:::GetWeightForNumbersOfCorrectandIncorrectPredictions(2, 1, 0, 0, c(3, 2, 0), c(2, 1, 2), logOfFactorialOfPredictionListStats, 
        returnlog = FALSE), 3)
}

test_GetWeightForNumbersOfCorrectandIncorrectPredictionsProducesCorrectOutputWhenReturnLogIsTrue <- function() {
    logOfFactorialOfPredictionListStats <- c(log(factorial(3)), log(factorial(2)), log(factorial(0)))
    checkEquals(CausalR:::GetWeightForNumbersOfCorrectandIncorrectPredictions(1, 0, 1, 1, c(3, 2, 0), c(2, 1, 2), logOfFactorialOfPredictionListStats, 
        returnlog = TRUE), log(6))
    checkEquals(CausalR:::GetWeightForNumbersOfCorrectandIncorrectPredictions(2, 1, 0, 0, c(3, 2, 0), c(2, 1, 2), logOfFactorialOfPredictionListStats, 
        returnlog = TRUE), log(3))
}

test_PopulateTheThreeByThreeContingencyTablePopulatesTableCorrectly <- function() {
    # Test a symmetric case
    predictionDataStats <- c(2, 3, 2)
    experimentalDataStats <- c(2, 3, 2)
    threeByThreeTable <- CausalR:::PopulateTheThreeByThreeContingencyTable(1, 1, 1, 1, predictionDataStats, experimentalDataStats)
    checkEquals(length(threeByThreeTable), 9)
    checkEquals(threeByThreeTable[1], 1)
    checkEquals(threeByThreeTable[2], 1)
    checkEquals(threeByThreeTable[3], 0)
    checkEquals(threeByThreeTable[4], 1)
    checkEquals(threeByThreeTable[5], 1)
    checkEquals(threeByThreeTable[6], 1)
    checkEquals(threeByThreeTable[7], 0)
    checkEquals(threeByThreeTable[8], 1)
    checkEquals(threeByThreeTable[9], 1)
    predictionDataStats2 <- c(4, 3, 2)
    experimentalDataStats2 <- c(2, 3, 4)
    threeByThreeTable2 <- CausalR:::PopulateTheThreeByThreeContingencyTable(1, 1, 1, 1, predictionDataStats2, experimentalDataStats2)
    checkEquals(length(threeByThreeTable), 9)
    checkEquals(threeByThreeTable2[1], 1)
    checkEquals(threeByThreeTable2[2], 1)
    checkEquals(threeByThreeTable2[3], 2)
    checkEquals(threeByThreeTable2[4], 1)
    checkEquals(threeByThreeTable2[5], 1)
    checkEquals(threeByThreeTable2[6], 1)
    checkEquals(threeByThreeTable2[7], 0)
    checkEquals(threeByThreeTable2[8], 1)
    checkEquals(threeByThreeTable2[9], 1)
}

test_ComputePValueFromDistributionTableFromDistributionTableWorksGivenADistributionTable <- function() {
    matrix1 <- matrix(c(1, 2, 3, 0.5, 0.4, 0.1), ncol = 2)
    totalWeights <- sum(matrix1[, 2])
    checkEquals(CausalR:::ComputePValueFromDistributionTable(1, matrix1, totalWeights), 1)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(2, matrix1, totalWeights), 0.5)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(3, matrix1, totalWeights), 0.1)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(4, matrix1, totalWeights), 0)
}

test_ComputePValueFromDistributionTableWorksGivenUnnormalisedDistributionTable <- function() {
    matrix1 <- matrix(c(1, 2, 3, 5, 4, 1), ncol = 2)
    totalWeights <- sum(matrix1[, 2])
    checkEquals(CausalR:::ComputePValueFromDistributionTable(1, matrix1, totalWeights), 1)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(2, matrix1, totalWeights), 0.5)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(3, matrix1, totalWeights), 0.1)
    checkEquals(CausalR:::ComputePValueFromDistributionTable(4, matrix1, totalWeights), 0)
}

test_CalculateSignificanceCallsCalculateSignificanceUsingCubicAlgorithm <- function() {
    # Check that CalculateSignificance calls CalculateSignificanceUsingCubicAlgorithm correctly
    pValue_direct <- CausalR::CalculateSignificanceUsingCubicAlgorithm(5, c(7, 4, 9), c(6, 6, 8), 1e-05)
    pValue_indirect <- CausalR:::CalculateSignificance(5, c(7, 4, 9), c(6, 6, 8), 1e-05, TRUE)
    checkEquals(pValue_direct, pValue_indirect)
}

test_CalculateSignificanceCallsCalculateSignificanceUsingQuarticAlgorithm <- function() {
    # Check that CalculateSignificance calls CalculateSignificanceUsingQuarticAlgorithm correctly
    pValue_direct <- CausalR::CalculateSignificanceUsingQuarticAlgorithm(5, c(7, 4, 9), c(6, 6, 8))
    pValue_indirect <- CausalR:::CalculateSignificance(5, c(7, 4, 9), c(6, 6, 8), useCubicAlgorithm = FALSE)
    checkEquals(pValue_direct, pValue_indirect)
} 

#' @title rank the hypotheses
#' @description
#' Rank the hypotheses in the causal network. This function can be run with parallelisation using the doParallel flag.
#' @export
#' @concept CausalR
#' @param network Computational Causal Graph, as an igraph.
#' @param experimentalData The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.
#' @param delta Distance to search within the causal graph.
#' @param epsilon The threshold that is used when calculating the p-value using the cubic algorithm (see 'Assessing statistical significance in causal graphs').
#' @param useCubicAlgorithm An indicator specifying which algorithm will be used to calculate the p-value. The default is set as useCubicAlgorithm = TRUE which uses the cubic algorithm. If this value is set as FALSE, the algorithm will use the much slower quartic algorithm which does compute the exact answer, as opposed to using approximations like the cubic algorithm.
#' @param use1bAlgorithm An indicator specifying whether the 1a or 1b (default, faster) variant of the cubic algorithm described in Chindelevitch's paper will be used to calculate the p-value.
#' @param symmetricCCG This flag specifies whether the CCG is assumed to be symmetric. The value is set as TRUE as a default. If this is the case the running time of the algorithm is reduced since the bottom half of the table can be filled in using the results of calculations performed earlier.
#' @param listOfNodes A list of nodes specified by the user. The algorithm will only calculate and store the results for the nodes in the specified list. The default value is NULL; here the algorithm will calculate and store results for all the nodes in the network.
#' @param correctPredictionsThreshold A threshold on the number of correct predictions for a given hypothesis. If a hypothesis produces fewer correct predictions than predictionsThreshold then the algorithm will not calculate the two p-values. Instead 'NA' will be displayed in the final two columns of the corresponding row of the results table. As a default correctPredictionsThreshold is set as -Inf, so that the p-values are calculated for all specified hypotheses.
#' @param quiet a flag to suppress output to console. FALSE by default.
#' @param doParallel A flag for running RankTheHypothesis in parallel mode.
#' @param numCores Number of cores to use if using parallel mode. If the default value of NULL is used, it will attempt to detect the number of cores available and use all of them bar one.
#' @param writeFile A flag for determining if the output should be written to a file in the working directory. Default is TRUE.
#' @param outputDir the directory to output the files to. Default is the working directory
#' @return A data frame containing the results of the algorithm.
#' @examples
#' #get path to example network file
#' networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' #create ccg
#' network <- CreateCCG(networkFile)
#' #get path to example experimental data
#' experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData.txt')
#' #read in experimetal data
#' experimentalData <- ReadExperimentalData(experimentalDataFile, network)
#' #run in single threaded mode
#' RankTheHypotheses(network, experimentalData, 2)
#' #run in parallel mode
#' RankTheHypotheses(network, experimentalData, 2, doParallel=TRUE, numCores=2)

#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.

RankTheHypotheses <- function(network, experimentalData, delta, epsilon = 1e-05, useCubicAlgorithm = TRUE, use1bAlgorithm = TRUE, symmetricCCG = TRUE,
                              listOfNodes = NULL, correctPredictionsThreshold = -Inf, quiet = FALSE, doParallel = FALSE, numCores = NULL, writeFile = TRUE, outputDir = getwd()) {
    
    delta <- as.integer(delta)
    
    # If outputDir doesn't exist, create it
    if (!file.exists(file.path(outputDir))){
        dir.create(file.path(outputDir))
    }
    
    # If CreateCCG was used to create 'network' it should have a name attribute containing the filename used to produce it.
    # If ReadExperimentalData was used to create 'experimentalData' the name of the first column should be equal to the filename used to produce it
    fileOutputName <- paste0(network$name, "-", colnames(experimentalData)[1], "-delta", delta)
    
    # Set up timer
        timeToRunSoFar <- Sys.time()
        
        if (is.numeric(experimentalData[1, 1])) {
            # experimentalData is already processed (i.e. has node IDs rather than text node names)
            processedExperimentalData <- experimentalData
        } else {
            processedExperimentalData <- ProcessExperimentalData(experimentalData, network)
        }
        
        isCCG <- network$isCCG
        if (is.null(isCCG)) {
            # Network doesn't specify whether it's a CCG or not - assume it's a CG
            isCCG <- FALSE
        }
        
        ## By default, if in parallel mode use all cores bar one
        if (doParallel && is.null(numCores)) {
            numCores <- parallel::detectCores() - 1
            if (numCores < 1)
                numCores <- 1
        }
        
        if (isCCG) {
            numNodes <- igraph::gorder(network) / 2
        } else {
            numNodes <- igraph::gorder(network)
        }
        
        # If listOfNodes is null test all the hypotheses, otherwise only test the hypotheses in the list
        if (is.null(listOfNodes)) {
            numNodesToBeTested <- numNodes
            nodesToBeTested <- c(1:numNodesToBeTested)
        } else {
            numNodesToBeTested <- length(listOfNodes)
            # If listOfNodes is a character array convert to nodeIDs
            if (typeof(listOfNodes) == "character") {
                nodesToBeTested <- GetNodeID(network, listOfNodes)
            } else {
                nodesToBeTested <- listOfNodes
            }
        }
        
        if(!quiet) {
            cat(paste("Number of Nodes to analyse:", numNodesToBeTested, "\n"))
        }
        
        numPredictions <- nrow(processedExperimentalData)
        # Get the values of n+, n- and n0
        experimentalDataStats <- AnalyseExperimentalData(processedExperimentalData)
        
        if (!symmetricCCG) {
            # Add the downregulated nodes on to the list of hypotheses to test
            nodesToBeTested <- c(nodesToBeTested, nodesToBeTested + numNodes)
            numNodesToBeTested <- numNodesToBeTested * 2
        }

        ## Use lapply functions with GetScoresForSingleNode function to build scoresMatrix
        if (doParallel) {

            if (.Platform$OS.type == "windows") {
                ## Using parallel processing on windows. Windows does not have fork() so can not use mclapply. Use parLapply instead
                
                ## Create temp file for output  
                tempOutputFile <- paste0(tempdir(), "\\progressOutfile.txt")
                if(file.exists(tempOutputFile)) {
                    file.remove(tempOutputFile)
                }
				
                cat(paste("Parallel processing. Number of cores:", numCores, "\n"))
                if (!quiet && numNodesToBeTested > 100) {
                    ## Progress ouput only occurs every 100 nodes 
                    cat(paste("Writing progress statements to temporary file:", tempOutputFile, "\n"))
                }
                
                ## For n cores parLapply distributes the elements of nodeIndices to the cores using parallel::splitIndices (roughly, but not quite
                ## the first 1/n elements of nodeIndices to core 1, the second 1/n to core 2 etc.). Since the the low indices take longer to run than the
                ## higher indices arrange indices, here we put nodeIndices in order so they are they are evenly distributed, one by one with increasing nodeID
                ## across cores.
                ## Note that it is not currently known why the lower indices take longer than higher indices. It is suspected they are ordered
                ## by level of connectivity somehow/somewhere.
                nodeIndices <- parallel::splitIndices(numNodesToBeTested, numCores)
                x <- 1
                for (i in 1:max(do.call("c", lapply(nodeIndices, length)))) {
                    ## if there are less nodes to be tested than cores, only loop over that number
                    for (j in 1:min(numCores, numNodesToBeTested)) {
                        if (!is.na(nodeIndices[[j]][i])) {
                            nodeIndices[[j]][i] <- x
                            x <- x+1
                        }
                    }
                }
                nodeIndices <- do.call("c", nodeIndices)
                
                ## Manually set up the clusters, passing required packages with makeCluster and required functions and variables with clusterExport
                cl <- parallel::makeCluster(numCores, outfile = tempOutputFile)
#                parallel::clusterCall(cl, function() {requireNamespace("igraph")})
                parallel::clusterCall(cl, function() {requireNamespace("igraph"); requireNamespace("CausalR")})
                parallel::clusterExport(
                    cl, varlist = c(
                        ## CausalR functions - export these separately (rather than exporting the whole package so this can be run outside of the package)
#                         "GetScoresForSingleNode", "RemoveIDsNotInExperimentalData", "MakePredictions", "MakePredictionsFromCG", "MakePredictionsFromCCG",
#                         "AnalysePredictionsList","GetNumberOfPositiveAndNegativeEntries", "ScoreHypothesis", "CalculateSignificance",
#                         "CalculateSignificanceUsingCubicAlgorithm1b", "CalculateSignificanceUsingQuarticAlgorithm", "FindMaximumDValue", "FindApproximateValuesThatWillMaximiseDValue",
#                         "GetMaximumDValueFromTwoByTwoContingencyTable", "GetAllPossibleRoundingCombinations", "PopulateTheThreeByThreeContingencyTable",
#                         "CalculateWeightGivenValuesInThreeByThreeContingencyTable", "GetWeightsAboveHypothesisScoreForAThreeByTwoTable",
#                         "GetMaxDValueForAThreeByTwoFamily", "GetApproximateMaximumDValueFromThreeByTwoContingencyTable", "CalculateEnrichmentPValue",
#                         "GetSetOfSignificantPredictions", "GetSetOfDifferentiallyExpressedGenes", "FindIdsOfConnectedNodesInSubgraph",
#                         "GetMatrixOfCausalRelationships", "DetermineInteractionTypeOfPath", "GetCombinationsOfCorrectandIncorrectPredictions", "CheckPossibleValuesAreValid",
#                         "GetScoresWeightsMatrix", "GetScoreForNumbersOfCorrectandIncorrectPredictions", "GetWeightForNumbersOfCorrectandIncorrectPredictions",
#                         "ComputePValueFromDistributionTable",
                        ## Variables
                        "timeToRunSoFar", "nodesToBeTested", "network", "delta", "processedExperimentalData", "numPredictions", "epsilon",
                        "useCubicAlgorithm", "use1bAlgorithm", "symmetricCCG", "correctPredictionsThreshold", "experimentalDataStats", "quiet"
                    ), envir = environment()
                )
                
                ## get scoresMatrix by applying GetScoresForSingleNode to each node and joining with rbind
                scoresMatrix <- do.call(rbind, parallel::parLapply(cl, nodeIndices, function(x) {
                    GetScoresForSingleNode(
                        x, timeToRunSoFar, nodesToBeTested, network, delta, processedExperimentalData,
                        numPredictions, epsilon, useCubicAlgorithm, use1bAlgorithm, symmetricCCG,
                        correctPredictionsThreshold, experimentalDataStats, quiet
                    )
                }))

                ## Stop the clusters
                parallel::stopCluster(cl)
                
            } else {
                ## Using parallel processing on Linux/Mac. Uses fork() - use mclapply.
                nodeIndices <- 1:numNodesToBeTested
                
                ## get scoresMatrix by applying GetScoresForSingleNode to each node and joining with rbind
                scoresMatrix <- do.call(
                    rbind, parallel::mclapply(nodeIndices, function(x)
                        GetScoresForSingleNode(x, timeToRunSoFar, nodesToBeTested, network, delta, processedExperimentalData,
                                               numPredictions, epsilon, useCubicAlgorithm, use1bAlgorithm, symmetricCCG,
                                               correctPredictionsThreshold, experimentalDataStats, quiet
                        ), mc.cores = numCores)
                )
            }
        } else {
            ## Single threaded processing - use lapply.
            nodeIndices <- 1:numNodesToBeTested
            
            ## get scoresMatrix by applying GetScoresForSingleNode to each node and joining with rbind
            scoresMatrix <- do.call(rbind, lapply(nodeIndices, function(x)
                GetScoresForSingleNode(x, timeToRunSoFar, nodesToBeTested, network, delta, processedExperimentalData,
                                       numPredictions, epsilon, useCubicAlgorithm, use1bAlgorithm, symmetricCCG,
                                       correctPredictionsThreshold, experimentalDataStats, quiet
                )
            )
            )
        }
        
        ## First column of scores matrix contains the NodeID
        unsignedNodeNames <- GetNodeName(network, scoresMatrix[,1])
        
        nodeNames <- unsignedNodeNames
        nodeNames[scoresMatrix[,2] == 1] <- paste0(nodeNames[scoresMatrix[,2] == 1], "+")
        nodeNames[scoresMatrix[,2] == -1] <- paste0(nodeNames[scoresMatrix[,2] == -1], "-")
            
        # Sort the output table in order of score (decreasing), then regulation (decreasing), then nodeID (increasing)
        orderHypotheses <- order(-scoresMatrix[, 3], -scoresMatrix[, 2], scoresMatrix[, 1])
        rankedHypotheses <- data.frame(scoresMatrix, row.names = nodeNames)
        colnames(rankedHypotheses) <- c("NodeID", "Regulation", "Score", "Correct", "Incorrect", "Ambiguous", "p-value", "Enrichment p-value")
        
        rankedHypotheses <- rankedHypotheses[orderHypotheses,]
        
        # Save results to an text file
        if(writeFile) {
            rhoutput <- scoresMatrix[, c(1, 1:8)]
            rhoutput[, 1] <- unsignedNodeNames
            rhoutput <- rhoutput[orderHypotheses,]
            colnames(rhoutput) <- c("NodeName", "NodeID", "Regulation", "Score", "Correct", "Incorrect", "Ambiguous", "p-value", "Enrichment p-value")
            utils::write.table(rhoutput, paste0(outputDir, "/ResultsTable-", fileOutputName, ".txt"), sep = "\t", row.names = FALSE, quote = FALSE)
        }
        
        return(rankedHypotheses)
    }

#' @title Write all explained nodes to Sif file
#' @description
#' Outputs networks of all explained nodes in .sif file format, named by node name with sign of regulation, each with a corresponding annotation file for producing visualisations using Cytoscape.  
#' @param scanResults a results object produced by ScanR
#' @param network a computational causal graph
#' @param experimentalData The experimental data read in using \link{ReadExperimentalData}. 
#' @param delta the number of edges across which the hypothesis should be followed, the setting should be that used to generate the input ScanR object.
#' @param correctlyExplainedOnly if TRUE network files will only be produced for correctly explained nodes. If FALSE network files will be produced for each of correctly explained, incorrectly explained and ambiguously explained nodes.
#' @param quiet a flag to suppress output to console. FALSE by default.
#' @return files containing paths from hypothesis node to explained nodes in sif format and corresponding annotation (_anno.txt) files
#' @export
#' @concept CausalR
#' @examples
#' networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
#' network <- CreateCCG(networkFile)
#' experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
#' experimentalData <- ReadExperimentalData(experimentalDataFile, network)
#' delta <- 2
#' scanResults <- runSCANR(network, experimentalData, numberOfDeltaToScan = delta,
#'     topNumGenes = 2, writeResultFiles = FALSE, writeNetworkFiles = "none",
#'     quiet = FALSE, doParallel = TRUE, numCores = 2)
#' WriteAllExplainedNodesToSifFile(scanResults, network, experimentalData, delta,
#'     correctlyExplainedOnly = TRUE, quiet = TRUE)

WriteAllExplainedNodesToSifFile <- function(scanResults, network, experimentalData, delta, correctlyExplainedOnly = TRUE, quiet = TRUE){
    
    NumCommonNodes <-length(scanResults$commonNodes)
    if (NumCommonNodes == 0) {
        stop("ScanR returned no common nodes at given delta")
    }
    
    for(i in 1:NumCommonNodes){
        
        # get current hypothesis node 'i' name concatenated with it's sign of regulation
        
        NodeiWithSign <- scanResults$commonNodes[i]    
        
        # separate NodeiWithSign into nodename and sign
        
        Nodeiname <- substr(NodeiWithSign, 1, nchar(NodeiWithSign)-1)
        NodeiSign <- substr(NodeiWithSign, nchar(NodeiWithSign), nchar(NodeiWithSign))
        
        if (NodeiSign=='+') {
            signOfHypothesis <- paste(+ 1, sep='')
        } else {
            signOfHypothesis <- paste(- 1, sep='')
        } 
        
        WriteExplainedNodesToSifFile(hypothesisnode = Nodeiname, signOfHypothesis , network, experimentalData, delta, outputFilesName = NodeiWithSign, correctlyExplainedOnly=TRUE, quiet=TRUE)
    } # end for
}

#' @title get maximum D value for three-by-two a family
#' @description
#' Returns the maximum D value for a particular family as described as D_fam 
#'  on pages 6 and 7 of  Assessing Statistical Significance of Causal Graphs in Assessing Statistical Signifcance of Causal Graphs
#' @param r_p a r+ row sum from the prediction-observation matrix 
#' @param r_m a  r- row sum from the prediction-observation matrix 
#' @param r_z a  r0 row sum from the prediction-observation matrix 
#' @param n_p a number of predicted increases from the prediction-observation matrix 
#' @param n_m a number of predicted decreases  from the prediction-observation matrix 
#' @param predictionListStats a vector contain the number of postive, negative and non-significant/contradictory predictions: q+, q- and q0.
#' @param logOfFactorialOfPredictionListStats a vector containing the log of the factorial for each element in the predictionListStats object
#' @param returnlog whether or not the maximum D value should be returned as a log (TRUE). Otherwise a non-logged value is returned.
#' @return Maximum D_fam Value


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.


GetMaxDValueForAThreeByTwoFamily <- function(r_p, r_m, r_z, n_p, n_m, predictionListStats, logOfFactorialOfPredictionListStats, returnlog = FALSE) {
    # Compute the maximum D value for a particular superfamily (a 3x2 matrix) - described in Assessing Statistical Significance of Causal Graphs (page
    # 6/7)
    
    # Inputs: twoByTwoContingencyTable Approximate values of n++, n+-, n-+ and n--, these values are statistical significance of causal graphs)
    # predictionListStats A vector containing the values q+, q- and q0 (the number of positive/negative/non-significant (or contradictory) predictions)
    # logOfFactorialOfPredictionListStats A vector containing the log of the factorial value for each entry in predictionListStats returnlog A boolean
    # describing whether the log of the D-Value should be returned
    
    total <- n_p + n_m
    
    # Compute the values of n++, n+-, n-+, n--, n0+, and n0- that maximise the D-value; these correspond to the following formula: n_ab is approximately
    # equal to q_a*n_b/T, where T = n+ + n-, and a,b are either + or -.  See Assessing statistical significance in causal graphs, page 7 - the formula
    # is not stated explicitly but follows from the logic of algorithm 1a).
    if (total > 0) {
        n_pp <- ceiling(r_p * n_p/total)
        n_pm <- r_p - n_pp
        n_mp <- ceiling(r_m * n_p/total)
        # Check this rounding produces a valid combination i.e. n++ + n-+ <= n+
        if ((n_mp + n_pp) > n_p) {
            n_mp <- floor(r_m * n_p/total)
        }
        n_mm <- r_m - n_mp
        n_zp <- n_p - (n_pp + n_mp)
        n_zm <- n_m - (n_pm + n_mm)
    } else {
        n_pp <- 0
        n_pm <- 0
        n_mp <- 0
        n_mm <- 0
        n_zp <- 0
        n_zm <- 0
    }
    
    threeByTwoContingencyTable <- c(n_pp, n_pm, n_mp, n_mm, n_zp, n_zm)
    
    maximumDFamValue <- GetApproximateMaximumDValueFromThreeByTwoContingencyTable(threeByTwoContingencyTable, predictionListStats, logOfFactorialOfPredictionListStats, 
        returnlog)
    
    return(maximumDFamValue)
} 

# A test file for the (faster) 1b cubic algorithm functions

test_GetApproximateMaximumDValueFromThreeByTwoContingencyTable <- function() {
    threeByTwoContingencyTable <- c(1, 3, 3, 2, 1, 4)
    predictionListStats <- c(6, 6, 6)
    logOfFactorialOfPredictionListStats <- lfactorial(c(6, 6, 6))
    checkEquals(CausalR:::GetApproximateMaximumDValueFromThreeByTwoContingencyTable(threeByTwoContingencyTable, predictionListStats, logOfFactorialOfPredictionListStats), 
        108000)
}

test_GetMaxDValueForAThreeByTwoFamily <- function() {
    predictionListStats <- c(2, 4, 1)
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    returnlog <- FALSE
    checkEquals(CausalR:::GetMaxDValueForAThreeByTwoFamily(1, 3, 1, 3, 2, predictionListStats, logOfFactorialOfPredictionListStats, returnlog = FALSE), 
        24)
}

test_TheCorrectTotalWeightForAllContingencyTablesIsCalculated <- function() {
    experimentalDataStats <- c(2, 3, 4)
    checkEquals(CausalR:::CalculateTotalWeightForAllContingencyTables(experimentalDataStats), 1260)
    checkEquals(CausalR:::CalculateTotalWeightForAllContingencyTables(experimentalDataStats, returnlog = TRUE), log(1260))
}

test_CorrectWeightsAboveHypothesisScoreAreReturnedWithDifferentScores <- function() {
    rowAndColumnSumValuesForThreeByTwoSubmatrix <- c(1, 3, 1, 3, 2)
    predictionListStats <- c(2, 4, 1)
    experimentalDataStats <- c(3, 2, 2)
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    hypothesisScore <- -2
    epsilon <- 1e-05
    logepsDMax <- log(24 * epsilon)
    logDMax <- log(24)
    weightStore <- c(0, 0)
    checkEquals(CausalR:::GetWeightsAboveHypothesisScoreForAThreeByTwoTable(weightStore, 1, 3, 1, 3, 2, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax), c(3, 10/3))
    hypothesisScore2 <- 0
    checkEquals(CausalR:::GetWeightsAboveHypothesisScoreForAThreeByTwoTable(weightStore, 1, 3, 1, 3, 2, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats, hypothesisScore2, logepsDMax, logDMax), c(2, 10/3))
    hypothesisScore3 <- 2
    checkEquals(CausalR:::GetWeightsAboveHypothesisScoreForAThreeByTwoTable(weightStore, 1, 3, 1, 3, 2, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats, hypothesisScore3, logepsDMax, logDMax), c(1, 10/3))
    
}

test_CalculateSignificanceUsingCubicAlgorithm1bProducesExactAnswerWithSmallInputs <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(2, 1, 0)
    epsilon = 1e-05
    pValue <- CausalR:::CalculateSignificanceUsingCubicAlgorithm1b(-1, predictionListStats, experimentalResultStats, epsilon)
    checkEquals(pValue, 1)
    pValue2 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm1b(3, predictionListStats, experimentalResultStats, epsilon)
    checkEquals(pValue2, (1/3))
}


test_CubicAlgorithm1bProducesApproximatelyCorrectAnswerWithLargeInputs <- function() {
    # Check that the answer from the faster cubic algorithm is within a certain tolerance of the answer obtained using the quartic algorithm whose
    # answer is exact.
    epsilon = 1e-05
    pValue3 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm1b(5, c(10, 10, 20), c(10, 10, 20), epsilon)
    checkEqualsNumeric(pValue3, 0.08, tolerance = 1 * 10 - 3)
    pValue4 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm1b(5, c(7, 7, 15), c(9, 3, 17), epsilon)
    checkEqualsNumeric(pValue4, 0.024, tolerance = 1 * 10 - 3)
} 

#' @title find Ids of connected nodes in subgraph
#' @description
#' Adds the IDs of the connected nodes in a subgraph to an existing list. 
#' Given the IDs of connected nodes in the full network, this function will find
#' the corresponding IDs in the subgraph
#' 

#' @param idsOfConnectedNodes a list of connected nodes in the full graph
#' @param subgraphOfConnectedNodes a subgraph 
#' @return a list of connected nodes in the subgraph

FindIdsOfConnectedNodesInSubgraph <- function(idsOfConnectedNodes, subgraphOfConnectedNodes) {
    
    
    numConnectedNodes <- length(idsOfConnectedNodes)
    idsOfConnectedNodesInSubgraph <- array(0, length(idsOfConnectedNodes))
    ids <- igraph::V(subgraphOfConnectedNodes)$ID
    
    for (i in 1:numConnectedNodes) {
        idsOfConnectedNodesInSubgraph[i] <- which(ids == idsOfConnectedNodes[i])
    }
    
    return(idsOfConnectedNodesInSubgraph)
} 

#' @title populate the three-by-three contingency table 
#' @description
#' Populates 3x3 signed contingency table of expected versus observed changes.
#' Given the values of n++, n+-, n-+ and n--, calculates n0+, n0-, n+0, n-0 and n00. 
#' Notation from Chindelevitch et al. Causal reasoning on biological networks 
#' Bioinformatics (2012) paper.

#' @param n_pp n++ contingency table entry
#' @param n_pm n+- contingency table entry
#' @param n_mp n-+ contingency table entry
#' @param n_mm n-- contingency table entry
#' @param predictionDataStats a prediction data table.
#' @param experimentalDataStats an experimental data table 
#' @return Vector of calculated values for n0+, n0-, n+0, n-0 and n00 - See: Chindelevitch et al.Bioinformatics (2012).


PopulateTheThreeByThreeContingencyTable <- function(n_pp, n_pm, n_mp, n_mm, predictionDataStats, experimentalDataStats) {
    
    n_pz <- predictionDataStats[1] - n_pp - n_pm
    n_mz <- predictionDataStats[2] - n_mp - n_mm
    
    return(c(n_pp, n_pm, n_pz, n_mp, n_mm, n_mz, (experimentalDataStats[1] - n_pp - n_mp), (experimentalDataStats[2] - n_pm - n_mm), (experimentalDataStats[3] - 
        n_pz - n_mz)))
    
}

#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012. 

#' @title get CCG node ID
#' @description
#' Returns the CCG node ID from a node name or a vector of node names and a given direction of regulation.
#' @param network a CCG object
#' @param nodename the node name, or names, for which the ID is required
#' @param direction the direction of regulation of the required node or nodes. Maybe +1 (default) or -1.
#' @return a scalar or vector containing the node ID or IDs requested

GetNodeID <- function(network, nodename, direction = 1) {
    
    n <- length(nodename)
    if (n > 1) {
        nodeID <- array(0, n)
        for (i in 1:n) {
            nodeID[i] <- GetNodeID(network, nodename[i], direction)
        }
    } else {
        # If we're given the full name, return the associated node ID
        nodeID <- which(igraph::V(network)$name == nodename)
        
        if (length(nodeID) == 0 && network$isCCG) {
            # nodename doesn't match any of the signed node names, try the unsigned names
            if (direction > 0) {
                nodeID <- which(igraph::V(network)$unsignedName == nodename)[1]
            } else {
                nodeID <- which(igraph::V(network)$unsignedName == nodename)[2]
            }
        }
    }
    
    return(nodeID)
    
} 

#' @title returns approximate maximum D value or weight for a 3x2 superfamily
#' @description
#' Computes an approximate maximum D value (or weight) for a superfamily (3x2 table).
#' The result is only approximate as only the first valid D value that is return. This has been
#' done to speed up the overall algorithm. 
#' 
#' @param threeByTwoContingencyTable  approximate values of n++, n+-, n-+, n--, n0+ and n0-, these values are
#' calculated to optimise the D-value (see page 6 of Assessing
#' statistical significance of causal graphs)
#' @param predictionListStats a vector containing the values q+, q- and q0 
#' (the number of positive/negative/non-significant 
#' (or contradictory) predictions)
#' @param logOfFactorialOfPredictionListStats a vector containing the log of the factorial value for
#' each entry in predictionListStats
#' @param returnlog return the result as a log, default is FALSE
#' @return an approximate maximum D value or weight


GetApproximateMaximumDValueFromThreeByTwoContingencyTable <- function(threeByTwoContingencyTable, predictionListStats, logOfFactorialOfPredictionListStats, 
    returnlog = FALSE) {
    # *** Summary *** Description: A function that will compute an approximate maximum D value (or weight) for a superfamily (3x2 table).  The reason it
    # is described as approiximate is that this function returns the first valid D value that is found. This has been done to speed up the overall
    # algorithm.
    
    
    n_pz <- predictionListStats[1] - (threeByTwoContingencyTable[1] + threeByTwoContingencyTable[2])
    n_mz <- predictionListStats[2] - (threeByTwoContingencyTable[3] + threeByTwoContingencyTable[4])
    n_zz <- predictionListStats[3] - (threeByTwoContingencyTable[5] + threeByTwoContingencyTable[6])
    threeByThreeContingencyTable <- c(threeByTwoContingencyTable, n_pz, n_mz, n_zz)
    weight <- CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable, logOfFactorialOfPredictionListStats, returnlog)
    return(weight)
} 

#' @title Populate Two by Two Contingency Table
#' @description
#' Calculates a 2x2 contingency table.
#' Given the value of n++ and the row and column sums (r+, r-, c+, c-),
#' Calculates the remaining values in the 2x2 contingency table i.e.
#' n+-, n-+, and n--. See Chindelevich et al. BMC Bioinformatics (2012) paper 
#' 'Assessing Statistical significance of causal graphs' for clarification on notation.

#' @param rowAndColumnSumValues the row and column sums (r+, r-, c+, c-).
#' @param n_pp the value of n++.
#' @return  the completed 2x2 contingency table: n++, n+-, n-+, n--

#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012.   

PopulateTwoByTwoContingencyTable <- function(rowAndColumnSumValues, n_pp) {
    
    # r+
    r_p <- rowAndColumnSumValues[1]
    # r-
    r_m <- rowAndColumnSumValues[2]
    # c+
    c_p <- rowAndColumnSumValues[3]
    # c- c_m <- rowAndColumnSumValues[4]
    
    # n+-
    n_pm <- r_p - n_pp
    # n-+
    n_mp <- c_p - n_pp
    # n--
    n_mm <- r_m - n_mp
    
    contingencyTableValues <- c(n_pp, n_pm, n_mp, n_mm)
    return(contingencyTableValues)
} 

#' @title compute a p-value from the distribution table
#' @description
#' Computes the p-value of the score of an hypothesis, based on a distribution table

#' @param scoreOfHypothesis a score of hypothesis
#' @param distributionMatrix a distribution table presented as a matrix
#' @param totalWeights a matrix of total weights
#' @return a p-value


ComputePValueFromDistributionTable <- function(scoreOfHypothesis, distributionMatrix, totalWeights) {
    
    
    # Ensure distribution matrix has two columns
    distributionMatrix <- matrix(distributionMatrix, ncol = 2)
    
    pValue <- sum(distributionMatrix[distributionMatrix[, 1] >= scoreOfHypothesis, 2])/totalWeights
    
    return(pValue)
} 

#' @title get scores weight matrix
#' @description
#' Computes the score and weight for a network/set of experimental data based on the table containing possible values of n++, n+-, n-+ and n--.
#' @param  matrixOfPossibleValues values of n++, n+-, n-+ and n-- that need to be assessed 
#' @param  predictionDataStats a table of predicions
#' @param  experimentalDataStats a table of observed experimental data
#' @param  logOfFactorialOfPredictionListStats  a vector containing the log of the factorial value for
#' each entry in predictionListStats
#' @return a matrix containing scores and logs of the weights

GetScoresWeightsMatrix <- function(matrixOfPossibleValues, predictionDataStats, experimentalDataStats, logOfFactorialOfPredictionListStats) {
    
    matrixOfPossibleValues <- matrix(matrixOfPossibleValues, ncol = 4)
    
    numRows <- nrow(matrixOfPossibleValues)
    # A matrix to store the score and weight of each row in matrixOfPossibleValues
    scoresandLogOfWeightsMatrix <- matrix(0, numRows, 2)
    
    for (counter in 1:numRows) {
        # Compute the score
        score = GetScoreForNumbersOfCorrectandIncorrectPredictions(matrixOfPossibleValues[counter, ])
        scoresandLogOfWeightsMatrix[counter, 1] <- score
        # Compute D-value - all are returned as log of the actual value. This is because the actual value is usually too big and reported as Inf
        logOfWeight = GetWeightForNumbersOfCorrectandIncorrectPredictions(matrixOfPossibleValues[counter, 1], matrixOfPossibleValues[counter, 2], matrixOfPossibleValues[counter, 
            3], matrixOfPossibleValues[counter, 4], predictionDataStats, experimentalDataStats, logOfFactorialOfPredictionListStats, returnlog = TRUE)
        scoresandLogOfWeightsMatrix[counter, 2] <- logOfWeight
    }
    
    # Check total weight expectedWeight =
    # factorial(sum(predictionDataStats))/factorial(numUpregulatedGenes)*factorial(numDownregulatedGenes)*factorial(numNonResponsiveGenes)
    
    return(scoresandLogOfWeightsMatrix)
} 

# A test suite for the Quartic Algorithm

test_CalculateSignificanceUsingQuarticAlgorithmContainingSingleNode <- function() {
    # A number of very simple tests for networks with just 1,and 2 nodes Tests for network with 1 node
    predictionsListStats <- c(1, 0, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats, c(1, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats, c(0, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats, c(0, 0, 1)), 1)
}

test_CalculateSignificanceUsingQuarticAlgorithmWithNetworkContainingTwoNodes <- function() {
    # Tests involving a representative network with 2 nodes Both nodes have positive predictions
    predictionsListStats2 <- c(2, 0, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats2, c(2, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats2, c(0, 2, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats2, c(0, 0, 2)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats2, c(1, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats2, c(1, 0, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats2, c(0, 1, 1)), 1)
    # Both nodes have negative predictions
    predictionsListStats3 <- c(0, 2, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats3, c(2, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats3, c(0, 2, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats3, c(0, 0, 2)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats3, c(1, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats3, c(1, 0, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats3, c(0, 1, 1)), 1)
    # One node has a positive prediction the other is negative (this is done without loss of generality)
    predictionsListStats4 <- c(1, 1, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats4, c(2, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats4, c(0, 2, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats4, c(0, 0, 2)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats4, c(1, 1, 0)), 0.5)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats4, c(1, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats4, c(1, 0, 1)), 0.5)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats4, c(1, 0, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats4, c(0, 1, 1)), 0.5)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats4, c(0, 1, 1)), 1)
}

test_CalculateSignificanceUsingQuarticAlgorithmWithNetworkContainingThreeNodes <- function() {
    # Test involving a representative network with 3 nodes All three nodes are positive
    predictionsListStats5 <- c(3, 0, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(3, predictionsListStats5, c(3, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-3, predictionsListStats5, c(0, 3, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats5, c(0, 0, 3)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats5, c(2, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats5, c(2, 0, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats5, c(1, 2, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats5, c(1, 1, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats5, c(1, 0, 2)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats5, c(0, 2, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats5, c(0, 1, 2)), 1)
    # 2 positive nodes and one negative
    predictionsListStats6 <- c(2, 1, 0)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats6, c(3, 0, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats6, c(0, 3, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats6, c(0, 0, 3)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(3, predictionsListStats6, c(2, 1, 0)), (1/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats6, c(2, 1, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats6, c(2, 0, 1)), (1/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats6, c(2, 0, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats6, c(1, 2, 0)), (2/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-3, predictionsListStats6, c(1, 2, 0)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(2, predictionsListStats6, c(1, 1, 1)), (1/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats6, c(1, 1, 1)), (2/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats6, c(1, 1, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats6, c(1, 0, 2)), (2/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats6, c(1, 0, 2)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(0, predictionsListStats6, c(0, 2, 1)), (2/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-2, predictionsListStats6, c(0, 2, 1)), 1)
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(1, predictionsListStats6, c(0, 1, 2)), (1/3))
    checkEquals(CausalR:::CalculateSignificanceUsingQuarticAlgorithm(-1, predictionsListStats6, c(0, 1, 2)), 1)
}

test_CheckPossibleValuesAreValidPassesWhenGivenAValidCombination <- function() {
    checkTrue(CausalR:::CheckPossibleValuesAreValid(c(3, 2, 0), c(2, 1, 2), c(1, 1, 1, 0)))
}

test_CheckPossibleValuesAreValidFaislWhenGivenAnInvalidCombination <- function() {
    # There are 10 conditions that are checked in the function - each one needs to be tested individually The number of each condition is given in
    # CheckPossibleValuesAreValid Check condition 1 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(2, 0, 0), c(0, 0, 0), c(2, 1, 0, 0))))
    # Check condition 2 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 2, 0), c(0, 0, 0), c(0, 0, 2, 1))))
    # Check condition 3 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 0, 0), c(2, 0, 0), c(1, 0, 2, 0))))
    # Check condition 4 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 0, 0), c(0, 2, 0), c(0, 1, 0, 2))))
    # Check condition 5 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(4, 0, 0), c(0, 0, 1), c(1, 1, 0, 0))))
    # Check condition 6 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 4, 0), c(0, 0, 1), c(0, 0, 1, 1))))
    # Check condition 7 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 0, 1), c(4, 0, 0), c(1, 0, 1, 0))))
    # Check condition 8 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 0, 1), c(0, 4, 0), c(0, 1, 0, 1))))
    # Check condition 9 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(4, 4, 0), c(0, 0, 1), c(1, 1, 1, 1))))
    # Check condition 9 fails appropriately
    checkTrue(!(CausalR:::CheckPossibleValuesAreValid(c(0, 0, 1), c(4, 4, 0), c(1, 1, 1, 1))))
}

test_ComputeFinalDistributionAggregatesWeightsCorrectly <- function() {
    matrix1 <- matrix(c(12, 4, -1, 12, -1, 0.3, 0.1, 0.1, 0.2, 0.3), ncol = 2)
    distributionMatrix <- CausalR:::ComputeFinalDistribution(matrix1)
    checkEquals(distributionMatrix[1, 1], -1)
    checkEquals(distributionMatrix[2, 1], 4)
    checkEquals(distributionMatrix[3, 1], 12)
    checkEquals(distributionMatrix[1, 2], 0.4)
    checkEquals(distributionMatrix[2, 2], 0.1)
    checkEquals(distributionMatrix[3, 2], 0.5)
}

test_GetCombinationsOfCorrectandIncorrectPredictionsProducesCorrectOutput <- function() {
    predictionDataStats <- c(2, 1, 0)
    experimentalDataStats <- c(1, 1, 1)
    matrixWithNumbersOfCorrectandIncorrectPredictions <- CausalR:::GetCombinationsOfCorrectandIncorrectPredictions(predictionDataStats, experimentalDataStats)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[1, 1], 0)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[1, 2], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[1, 3], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[1, 4], 0)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[2, 1], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[2, 2], 0)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[2, 3], 0)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[2, 4], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[3, 1], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[3, 2], 1)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[3, 3], 0)
    checkEquals(matrixWithNumbersOfCorrectandIncorrectPredictions[3, 4], 0)
}

test_GetScoresWeightsMatrixFillsTheTwoColumnsCorrectly <- function() {
    matrix1 <- matrix(c(0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0), ncol = 4)
    predictionDataStats <- c(2, 1, 0)
    experimentalDataStats <- c(1, 1, 1)
    logOfFactorialOfPredictionListStats <- c(log(factorial(2)), log(factorial(1)), log(factorial(0)))
    matrixWithScoresAndWeights <- CausalR:::GetScoresWeightsMatrix(matrix1, predictionDataStats, experimentalDataStats, logOfFactorialOfPredictionListStats)
    checkEquals(dim(matrixWithScoresAndWeights)[1], 3)
    checkEquals(dim(matrixWithScoresAndWeights)[2], 2)
    checkEquals(matrixWithScoresAndWeights[1, 1], -2)
    checkEquals(matrixWithScoresAndWeights[2, 1], 2)
    checkEquals(matrixWithScoresAndWeights[3, 1], 0)
    checkEquals(matrixWithScoresAndWeights[1, 2], matrixWithScoresAndWeights[2, 2])
    checkEquals(matrixWithScoresAndWeights[2, 2], matrixWithScoresAndWeights[3, 2])
} 

#' @title create network from table
#' @description
#' Creates a network from an internal data table created from a .sif file:
#' this function converts the data read in from the .sif file into 
#' an igraph in R.
#' 

#' @param dataTable the data table containing the information read in from the .sif file representing the network.
#' @return an igraph network


CreateNetworkFromTable <- function(dataTable) {
    
    
    matrixOfInteractions <- as.matrix(dataTable)
    # Use the first and third colums (names of genes that define an edge) to build the network
    network <- igraph::graph_from_edgelist(matrixOfInteractions[, -2], TRUE)
    return(network)
} 

#' @title calculate significance using the quartic algorithm
#' @description
#' Computes the significance of a given hypothesis. For a detailed description of the algorithm
#' see Causal reasoning on biological networks: interpreting transcriptional changes - Chindelevitch et al., section 2.
#' from which the methods and notation is taken.


#' @export
#' @concept CausalR
#' @param hypothesisScore  the score for which a  p-value is required
#' @param predictionListStats  a vector containing the values q+, q- and q0 (the number of positive/negative/non-significant  or contradictory) predictions)
#' @param experimentalDataStats a vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or contradictory) transcripts in the results) (or contradictory) transcripts in the results)
#' @return  the corresponding p-value
#' @examples
#' CalculateSignificance(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)

#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012.

CalculateSignificanceUsingQuarticAlgorithm <- function(hypothesisScore, predictionListStats, experimentalDataStats) {
    
    # Each row of the matrix below will contain the four values: numCorrectPositivePredictions (n++), numIncorrectPositivePredictions (n+-),
    # numCorrectNegativePredictions n(-+) and numIncorrectNegativePredictions (n--)
    matrixOfIncorrectAndCorrectPredictionNumbers <- GetCombinationsOfCorrectandIncorrectPredictions(predictionListStats, experimentalDataStats)
    
    # Calculate the log of the factorial for all values in predictionListStats, this is to reduce the number of computations done later on.
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    
    scoresAndLogOfWeightsMatrix <- GetScoresWeightsMatrix(matrixOfIncorrectAndCorrectPredictionNumbers, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats)
    
    # Shift the logOfWeight values by subtracting the minimum away from all of them
    scoresAndLogOfWeightsMatrix[, 2] <- scoresAndLogOfWeightsMatrix[, 2] - min(scoresAndLogOfWeightsMatrix[, 2])
    
    # Replace the logOfWeight values with Weights by taking the exponential and store in a matrix. By subtracting a factor from all the terms in the
    # previous step we have divided all the weight values by a certain factor. This means we will get the correct answer when computing the ratio of
    # those which have a score greater than the hypothesis score (i.e. the p-value)
    scoresWeightsMatrix <- scoresAndLogOfWeightsMatrix
    scoresWeightsMatrix[, 2] <- exp(scoresWeightsMatrix[, 2])
    
    totalWeights <- sum(scoresWeightsMatrix[, 2])
    
    pValue <- ComputePValueFromDistributionTable(hypothesisScore, scoresWeightsMatrix, totalWeights)
    
    return(pValue)
} 

#' @title  check possible values are valid
#' @description
#' Checks if the a given set of possible values for n++, n+-, n-+ and n-- are agree with the predicted and experimental
#' data
#'

#' @param predictionDataStats a vector of predicted results
#' @param experimentalDataStats a vector of observed experimental results
#' @param possibleValues a vector of possible values  n++, n+-, n-+ and n--

#' @return TRUE if and only if the given vector of possible values is valid

CheckPossibleValuesAreValid <- function(predictionDataStats, experimentalDataStats, possibleValues) {
    
    numPositivePredictions <- predictionDataStats[1]
    numNegativePredictions <- predictionDataStats[2]
    numZeroPredictions <- predictionDataStats[3]
    numUpregulatedGenes <- experimentalDataStats[1]
    numDownregulatedGenes <- experimentalDataStats[2]
    numNonResponsiveGenes <- experimentalDataStats[3]
    
    # n_pp = possible value for numCorrectPositivePredictions
    n_pp <- possibleValues[1]
    # n_pm = possible value for numIncorrectPositivePredictions
    n_pm <- possibleValues[2]
    # n_mp = possible value for numIncorrectNegativePredictions
    n_mp <- possibleValues[3]
    # n_mm = possible value for numCorrectNegativePredictions
    n_mm <- possibleValues[4]
    
    # The conditions that need to be fulfilled are a series of inequalities The conditions will be number so that they can matched to the test function
    # Condition 1
    if (((n_pp + n_pm) > numPositivePredictions)) {
        return(FALSE)
    }
    # Condition 2
    if (((n_mp + n_mm) > numNegativePredictions)) {
        return(FALSE)
    }
    # Condition 3
    if (((n_pp + n_mp) > numUpregulatedGenes)) {
        return(FALSE)
    }
    # Condition 4
    if (((n_pm + n_mm) > numDownregulatedGenes)) {
        return(FALSE)
    }
    
    # n_pz (n+0) is the number of genes with positive predictions but are in the set of non-responsive genes in the results.
    n_pz <- (numPositivePredictions - n_pp - n_pm)
    # Condition 5
    if (n_pz > numNonResponsiveGenes) {
        return(FALSE)
    }
    # n_mz (n-0) is the number of genes with negative predictions but are in the set of non-responsive genes in the results.
    n_mz <- (numNegativePredictions - n_mp - n_mm)
    # Condition 6
    if (n_mz > numNonResponsiveGenes) {
        return(FALSE)
    }
    # n_zp (n0p) is the number of genes predicted to have a non-significant interaction with the hypothesis but are actually in the positively affected
    # in the results
    n_zp <- (numUpregulatedGenes - n_pp - n_mp)
    # Condition 7
    if (n_zp > numZeroPredictions) {
        return(FALSE)
    }
    # n_zm (n0m) is the number of genes predicted to have a non-significant interaction with the hypothesis but are actually in the negatively affected
    # in the results
    n_zm <- (numDownregulatedGenes - n_pm - n_mm)
    # Condition 8
    if (n_zm > numZeroPredictions) {
        return(FALSE)
    }
    # Condition 9
    if (((n_pz + n_mz) > numNonResponsiveGenes)) {
        return(FALSE)
    }
    # Condition 10
    if (((n_zp + n_zm) > numZeroPredictions)) {
        return(FALSE)
    }
    
    return(TRUE)
} 

#' @title  get weights above hypothesis score and total weights
#' @description 
#' Gets the score based on the values of n++, n+-, n-+ and n--. Used as part of a p-value calculation.
#' @param r_p the row sum r+
#' @param r_m the row sum r-
#' @param c_p the column sum c+
#' @param predictionListStats statistics for the  prediction list
#' @param experimentalDataStats statistics for the experimental data
#' @param logOfFactorialOfPredictionListStats log of factorial of prediction list stats
#' @param hypothesisScore the hypothesis score to be considered
#' @param logepsDMax Exponential of logD Maximum value 
#' @param logDMax A logD Maximum value
#' @return score data




GetWeightsAboveHypothesisScoreAndTotalWeights <- function(r_p, r_m, c_p, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
    hypothesisScore, logepsDMax, logDMax) {
    
    weights <- c(0, 0)
    
    # To reduce the overall runtime of the algorithm, the approximate maxDValue for the family is calculated, rather than the actual.
    log_approxDFamilyMax <- GetMaxDValueForAFamily(r_p, r_m, c_p, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, TRUE)
    if (log_approxDFamilyMax > logepsDMax) {
        # Calculate the value of n++ that is used to calculate approxDFamilyMax t
        total <- r_p + r_m
        
        if (total > 0) {
            n_pp <- round(r_p * c_p/total)
        } else {
            n_pp <- 0
        }
        
        # Iterate over possible values of n++, starting from the one calculated above.  Since the values decrease monotonically either side of this value,
        # we start at this value and iterate in both directions until the D value is less than a given threshold. This will give us the non-neglible parts
        # of the distribution.
        
        # An upper bound for possible values of n++ is the minimum of r+ and c+
        for (m_pp in n_pp:min(r_p, c_p)) {
            # n+-
            if (r_p >= m_pp) {
                m_pm <- r_p - m_pp
                # n-+
                if (c_p >= m_pp) {
                    m_mp <- c_p - m_pp
                    # n--
                    if (r_m >= m_mp) {
                        m_mm <- r_m - m_mp
                        logDValue <- GetWeightForNumbersOfCorrectandIncorrectPredictions(m_pp, m_pm, m_mp, m_mm, predictionListStats, experimentalDataStats, 
                                                                                         logOfFactorialOfPredictionListStats, TRUE)
                        if (logDValue > logepsDMax) {
                            # Rather than just calculate the DValue by taking the exponential of logDvalue, we first subtract a constant.  This means when we take the
                            # exponential we get the DValue divided by a different constant, but this cancels out when we take ratios later when computing the p-value
                            weight_contrib <- exp(logDValue - logDMax)
                            weights[2] <- weights[2] + weight_contrib
                            if (m_pp + m_mm - m_pm - m_mp >= hypothesisScore) {
                                weights[1] <- weights[1] + weight_contrib
                            }
                        } else {
                            break
                        }
                    }
                }
            }
        }
        
        # The loop below does exactly the same thing as the one above; it covers the cases that are not done in the loop above. A lower bound for possible
        # values of n++ is 0 To avoid n_pp become negative or double counting when n_pp = 0, we add the following if statement
        if (n_pp > 0) {
            
            for (m_pp in (n_pp - 1):0) {
                # n+-
                if (r_p >= m_pp) {
                    m_pm <- r_p - m_pp
                    # n-+
                    if (c_p >= m_pp) {
                        m_mp <- c_p - m_pp
                        # n--
                        if (r_m >= m_mp) {
                            m_mm <- r_m - m_mp
                            logDValue <- GetWeightForNumbersOfCorrectandIncorrectPredictions(m_pp, m_pm, m_mp, m_mm, predictionListStats, experimentalDataStats, 
                                                                                             logOfFactorialOfPredictionListStats, TRUE)
                            if (logDValue > logepsDMax) {
                                # Rather than just calculate the DValue by taking the exponential of logDvalue, we first subtract a constant.  This means when we take the
                                # exponential we get the DValue divided by a different constant, but this cancels out when we take ratios later when computing the p-value
                                weight_contrib <- exp(logDValue - logDMax)
                                weights[2] <- weights[2] + weight_contrib
                                if (m_pp + m_mm - m_pm - m_mp >= hypothesisScore) {
                                    weights[1] <- weights[1] + weight_contrib
                                }
                            } else {
                                break
                            }
                        }
                    }
                }
            }
        }
    }
    
    return(weights)
    
} 

# A test suite for the functions implemented in the cubic algorithm

test_FindMaximumDValueProducesTheCorrectValue <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(1, 1, 1)
    logOfFactorialOfPredictionListStats <- c(log(factorial(2)), log(factorial(1)), log(factorial(0)))
    checkEquals(CausalR:::FindMaximumDValue(predictionListStats, experimentalResultStats, logOfFactorialOfPredictionListStats), 2)
    experimentalResultStats2 <- c(2, 1, 0)
    checkEquals(CausalR:::FindMaximumDValue(predictionListStats, experimentalResultStats2, logOfFactorialOfPredictionListStats), 2)
}

test_FindApproximateValuesThatWillMaximiseDValueCalculatesOutputValuesCorrectly <- function() {
    predictionListStats <- c(7, 3, 1)
    experimentalDataStats <- c(4, 5, 2)
    values <- CausalR:::FindApproximateValuesThatWillMaximiseDValue(predictionListStats, experimentalDataStats)
    checkEquals(length(values), 4)
    checkEquals(values[1], (28/11))
    checkEquals(values[2], (35/11))
    checkEquals(values[3], (12/11))
    checkEquals(values[4], (15/11))
}

test_GetMaximumDValueFromTwoByTwoContingencyTableProducesTheCorrectDValue <- function() {
    twoByTwoContingencyTable <- c(1, 2, 0, 1)
    predictionListStats <- c(4, 3, 2)
    experimentalDataStats <- c(2, 3, 4)
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    checkEquals(CausalR:::GetMaximumDValueFromTwoByTwoContingencyTable(twoByTwoContingencyTable, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats), 
        72)
}

test_GetApproximateMaximumDValueFromTwoByTwoContingencyTableProducesTheCorrectDValue <- function() {
    n_pp <- 1
    n_pm <- 2
    n_mp <- 0
    n_mm <- 1
    predictionListStats <- c(4, 3, 2)
    experimentalDataStats <- c(2, 3, 4)
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    checkEquals(CausalR:::GetApproximateMaximumDValueFromTwoByTwoContingencyTable(n_pp, n_pm, n_mp, n_mm, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats), 72)
}

test_GetAllPossibleRoundingCombinationsWithFourNonIntegerValues <- function() {
    vect <- c(4.2, 5.7, 2.3, 0.7)
    roundingCombinations <- CausalR:::GetAllPossibleRoundingCombinations(vect)
    checkEquals(nrow(roundingCombinations), 16)
    checkEquals(ncol(roundingCombinations), 4)
    checkEquals(roundingCombinations[1, ], c(4, 5, 2, 0))
    checkEquals(roundingCombinations[2, ], c(4, 5, 2, 1))
    checkEquals(roundingCombinations[3, ], c(4, 5, 3, 0))
    checkEquals(roundingCombinations[4, ], c(4, 5, 3, 1))
    checkEquals(roundingCombinations[5, ], c(4, 6, 2, 0))
    checkEquals(roundingCombinations[6, ], c(4, 6, 2, 1))
    checkEquals(roundingCombinations[7, ], c(4, 6, 3, 0))
    checkEquals(roundingCombinations[8, ], c(4, 6, 3, 1))
    checkEquals(roundingCombinations[9, ], c(5, 5, 2, 0))
    checkEquals(roundingCombinations[10, ], c(5, 5, 2, 1))
    checkEquals(roundingCombinations[11, ], c(5, 5, 3, 0))
    checkEquals(roundingCombinations[12, ], c(5, 5, 3, 1))
    checkEquals(roundingCombinations[13, ], c(5, 6, 2, 0))
    checkEquals(roundingCombinations[14, ], c(5, 6, 2, 1))
    checkEquals(roundingCombinations[15, ], c(5, 6, 3, 0))
    checkEquals(roundingCombinations[16, ], c(5, 6, 3, 1))
}

test_GetAllPossibleRoundingCombinationsWithOneIntegerValue <- function() {
    vect <- c(4.2, 5.7, 2.3, 1)
    roundingCombinations <- CausalR:::GetAllPossibleRoundingCombinations(vect)
    checkEquals(nrow(roundingCombinations), 8)
    checkEquals(ncol(roundingCombinations), 4)
    checkEquals(roundingCombinations[1, ], c(4, 5, 2, 1))
    checkEquals(roundingCombinations[2, ], c(4, 5, 3, 1))
    checkEquals(roundingCombinations[3, ], c(4, 6, 2, 1))
    checkEquals(roundingCombinations[4, ], c(4, 6, 3, 1))
    checkEquals(roundingCombinations[5, ], c(5, 5, 2, 1))
    checkEquals(roundingCombinations[6, ], c(5, 5, 3, 1))
    checkEquals(roundingCombinations[7, ], c(5, 6, 2, 1))
    checkEquals(roundingCombinations[8, ], c(5, 6, 3, 1))
}

test_GetMaxDValueForAFamilyProducesTheCorrectDValue <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(1, 1, 1)
    logOfFactorialOfPredictionListStats <- c(log(factorial(2)), log(factorial(1)), log(factorial(0)))
    checkEquals(CausalR:::GetMaxDValueForAFamily(2, 0, 1, predictionListStats, experimentalResultStats, logOfFactorialOfPredictionListStats), 2)
}

test_GetRowAndColumnSumValuesOutputsTheCorrectCombinations <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(1, 1, 1)
    possibleRowAndColumnSumValues <- CausalR:::GetRowAndColumnSumValues(predictionListStats, experimentalResultStats)
    checkEquals(nrow(possibleRowAndColumnSumValues), 2)
    checkEquals(ncol(possibleRowAndColumnSumValues), 4)
    checkEquals(possibleRowAndColumnSumValues[1, 1], 1)
    checkEquals(possibleRowAndColumnSumValues[1, 2], 1)
    checkEquals(possibleRowAndColumnSumValues[1, 3], 1)
    checkEquals(possibleRowAndColumnSumValues[1, 4], 1)
    checkEquals(possibleRowAndColumnSumValues[2, 1], 2)
    checkEquals(possibleRowAndColumnSumValues[2, 2], 0)
    checkEquals(possibleRowAndColumnSumValues[2, 3], 1)
    checkEquals(possibleRowAndColumnSumValues[2, 4], 1)
}


test_CheckRowAndColumnSumValuesAreValidReturnsTrueForValidInput <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(1, 1, 1)
    rowAndColumnSumValues <- c(2, 0, 1, 1)
    checkTrue(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues, predictionListStats, experimentalResultStats))
}

test_CheckRowAndColumnSumValuesAreValidReturnsFalseForInvalidInput <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(1, 1, 1)
    rowAndColumnSumValues1 <- c(0, 0, 1, 0)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues1, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues2 <- c(0, 0, 0, 0)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues2, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues3 <- c(1, 0, 0, 1)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues3, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues4 <- c(2, 0, 0, 2)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues4, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues5 <- c(0, 1, 0, 1)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues5, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues6 <- c(1, 1, 0, 2)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues6, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues7 <- c(2, 1, 0, 3)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues7, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues8 <- c(0, 0, 1, -1)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues8, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues9 <- c(1, 0, 1, 0)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues9, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues10 <- c(0, 1, 1, 0)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues10, predictionListStats, experimentalResultStats)))
    rowAndColumnSumValues10 <- c(2, 1, 1, -2)
    checkTrue(!(CausalR:::CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues10, predictionListStats, experimentalResultStats)))
}

test_GetWeightsAboveHypothesisScoreAndTotalWeightsProducesCorrectDistributionTable <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(2, 1, 0)
    epsilon = 1e-05
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    hypothesisScore <- 1
    logDMax <- log(2)
    logepsDMax <- log(epsilon) + logDMax
    r_p <- 2
    r_m <- 1
    c_p <- 2
    # A 2x1 array containing the weight above the hypothesis score and the total weight
    weightAboveHypScoreAndTotalWeight <- CausalR:::GetWeightsAboveHypothesisScoreAndTotalWeights(r_p, r_m, c_p, predictionListStats, experimentalResultStats, 
        logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax)
    checkEquals(weightAboveHypScoreAndTotalWeight[1], 0.5)
    checkEquals(weightAboveHypScoreAndTotalWeight[2], 1.5)
}

test_GetWeightsAboveHypothesisScoreAndTotalWeightsDoesntDoubleCountWhenNPlusPlusIsZero <- function() {
    predictionListStats <- c(0, 1, 1)
    experimentalDataStats <- c(1, 1, 0)
    epsilon = 1e-05
    r_p <- 0
    r_m <- 1
    c_p <- 1
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    hypothesisScore <- 0
    logDMax <- 0
    logepsDMax <- log(epsilon) + logDMax
    weightAboveHypScoreAndTotalWeight <- CausalR:::GetWeightsAboveHypothesisScoreAndTotalWeights(r_p, r_m, c_p, predictionListStats, experimentalDataStats, 
        logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax)
    # There's only 1 element in the contingency table, a wrongly-predicted hypothesis, hence the score is -1, so it should contribute to the denominator
    # of the p-value but not the numerator.
    checkEquals(weightAboveHypScoreAndTotalWeight[1], 0)
    checkEquals(weightAboveHypScoreAndTotalWeight[2], 1)
}

test_CalculateSignificanceUsingCubicAlgorithmProducesExactAnswerWithSmallInputs <- function() {
    predictionListStats <- c(2, 1, 0)
    experimentalResultStats <- c(2, 1, 0)
    epsilon = 1e-05
    pValue <- CausalR:::CalculateSignificanceUsingCubicAlgorithm(-1, predictionListStats, experimentalResultStats, epsilon)
    checkEquals(pValue, 1)
    pValue2 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm(3, predictionListStats, experimentalResultStats, epsilon)
    checkEquals(pValue2, (1/3))
}

test_CalculateSignificanceUsingCubicAlgorithmProducesApproximatelyCorrectAnswerWithLargeInputs <- function() {
    # Check that the answer from the cubic algorithm is within a certain tolerance of the answer obtained using the quartic algorithm whose answer is
    # exact.
    epsilon = 1e-05
    pValue3 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm(5, c(10, 10, 20), c(10, 10, 20), epsilon)
    checkEqualsNumeric(pValue3, 0.08, tolerance = 1 * 10 - 3)
    pValue4 <- CausalR:::CalculateSignificanceUsingCubicAlgorithm(5, c(7, 7, 15), c(9, 3, 17), epsilon)
    checkEqualsNumeric(pValue4, 0.024, tolerance = 1 * 10 - 3)
}

test_CalculateWeightGivenValuesInThreeByThreeContingencyTableProducesTheCorrectOutput <- function() {
    threeByThreeContingencyTable <- c(1, 1, 0, 1, 1, 1, 0, 1, 1)
    logOfFactorialOfPredictionListStats <- c(log(factorial(2)), log(factorial(3)), log(factorial(2)))
    checkEquals(CausalR:::CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable, logOfFactorialOfPredictionListStats), 
        24)
    threeByThreeContingencyTable2 <- c(1, 1, 2, 1, 1, 1, 0, 1, 1)
    logOfFactorialOfPredictionListStats2 <- c(log(factorial(4)), log(factorial(3)), log(factorial(2)))
    checkEquals(CausalR:::CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable2, logOfFactorialOfPredictionListStats2), 
        144)
} 

#' @title calculate weight given values in three-by-three contingency table
#' @description
#' Given the values in the three by three contingency table and the values of the number of positive/negative/non-significant predictions (q+, q-, q0) this function calculates the D-value (or weight).

#' @param threeByThreeContingencyTable a 3x3 contingency table
#' @param logOfFactorialOfPredictionListStats log of Factorial of prediction statistics
#' @param returnlog should the result be returned as a log value. Default is FALSE.

#' @return a D-value (or weight)

CalculateWeightGivenValuesInThreeByThreeContingencyTable <- function(threeByThreeContingencyTable, logOfFactorialOfPredictionListStats, returnlog = FALSE) {
    
    # The D value is defined as: (q+! / (n++! * n+-! * n+0!)) * (q-! / (n-+! * n--! * n-0!)) * (q0! / (n0+! * n0-! * n00!))
    
    if (returnlog) {
        return(sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable)))
    } else {
        return(exp(sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable))))
    }
} 

#' @title read .sif to Table
#' @description
#' Reads a .sif file into a table in R
#' @param sifFile the sifFile to be read in 
#' @return a R table containing the data from the .sif file

ReadSifFileToTable <- function(sifFile) {
    
    table <- utils::read.table(sifFile, colClasses = "character")
    
    ValidateFormatOfTable(table)
    
    return(table)
} 

#' @title  get row and column sum values
#' @description
#' Returns the possible values of r+, r-, c+ and c- (the column and row sum values) following page 6 of Assessing
#' statistical  significance in causal graphs (Chindelevitch et. al)
#' @param predictionListStats a vector containing the number of postive, negative, or non-signficant/contradictory predictions (q+, q- and q0)
#' @param experimentalResultStats a vector containing the number of postive, negative, or non-signficant/contradictory observations (n+, n- and n0)
#' @return a matrix of row and sum values r+, r-, c+ and c- 


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.



GetRowAndColumnSumValues <- function(predictionListStats, experimentalResultStats) {
    
    
    # Number of positive predictions from the network. This will be an upper bound for r+ (r_p)
    q_p <- predictionListStats[1]
    # Number of negative predictions from the network. This will be an upper bound for r- (r_m)
    q_m <- predictionListStats[2]
    # Number of positive results. This will be an upper bound for c+ (c_p)
    n_p <- experimentalResultStats[1]
    # Number of negative results. This will be an upper bound for c- (c_m)
    n_m <- experimentalResultStats[2]
    
    # Pre-declare an array to store the valid column/row sum values - since we only iterate over three quantities, the array can have a maximum size
    # equal to the product of the three smallest values out of q_p+1, q_m+1, n_p+1, and n_m+1.
    maximumValue = max(q_p + 1, q_m + 1, n_p + 1, n_m + 1)
    maxNumberOfRows <- (q_p + 1) * (q_m + 1) * (n_p + 1) * (n_m + 1)/maximumValue
    tryCatch({
        possibleRowAndColumnSumValues <- matrix(0, maxNumberOfRows, 4)
        # Iterate all possible values of r+, r-, c+ and c- and check if they produce a valid contingency table
        counter <- 1
        for (r_p in 0:q_p) {
            # for 0 to the limit of negative predictions
            for (r_m in 0:q_m) {
                # for 0 to the limit of number of positive results
                for (c_p in 0:n_p) {
                    # calculate the column sum of m
                    c_m <- r_p + r_m - c_p
                    rowAndColumnSumValues <- c(r_p, r_m, c_p, c_m)
                    if (CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues, predictionListStats, experimentalResultStats)) {
                        if (counter > maxNumberOfRows) {
                            # double the size of the matrix if it doesn't have any space
                            possibleRowAndColumnSumValues <- matrix(c(possibleRowAndColumnSumValues, matrix(0, maxNumberOfRows, 4)), ncol = 4)
                        }
                    possibleRowAndColumnSumValues[counter, ] <- rowAndColumnSumValues
                    counter <- counter + 1
                    }
                }
            }
        }
        
        # Only want to keep the rows that were populated in the set of for loops above
        possibleRowAndColumnSumValues <- possibleRowAndColumnSumValues[1:counter - 1, ]
        
        return(possibleRowAndColumnSumValues)
    }, error = function(e) {
        # If there is an error, report back all valuable information to the user - this is deisgned for when there is a memory error.
        print(e)
        print(paste("Number of positive predictions:", predictionListStats[1]))
        print(paste("Number of negative predictions:", predictionListStats[2]))
        print(paste("Number of ambiguous/unknown predictions:", predictionListStats[3]))
        print(paste("Number of upregualted results:", experimentalResultStats[1]))
        print(paste("Number of down-regualted results:", experimentalResultStats[2]))
        print(paste("Number of non-significantly changed results:", experimentalResultStats[3]))
        print(paste("Number of rows in array to store all the combinations of row and column sums in upper 2x2 submatrix of contingency table:", maxNumberOfRows))
        stop()
    })
}
 

#' @title plot graph with node names
#' @description
#' Plots an igraph with the node names. Plots a igraph to the screen displaying the names of the nodes input rather than R's internal numbering.

#' @param igraph internal an igraph representation of an interaction network
#' @return network visualisation
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg <- CreateCCG(network)
#' PlotGraphWithNodeNames(ccg)




PlotGraphWithNodeNames <- function(igraph) {
    
    graphics::plot(igraph, vertex.label = igraph::V(igraph)$name)
    
} 

#' @title returns table of correct and incorrect predictions
#' @description
#' Returns the  numbers of correct and incorrect positive and negative predictions
#' 
#' @param predictionDataStats prediction data statistics table
#' @param experimentalDataStats Experimental data statistics table
#' @return a matrix the  numbers of correct and incorrect positive and negative prediction 
#' 


GetCombinationsOfCorrectandIncorrectPredictions <- function(predictionDataStats, experimentalDataStats) {
    
    numPositivePredictions <- predictionDataStats[1]
    numNegativePredictions <- predictionDataStats[2]
    numUpregulatedGenes <- experimentalDataStats[1]
    numDownregulatedGenes <- experimentalDataStats[2]
    
    # First find bounds for numCorrectPositivePredictions, numIncorrectPositivePredictions, numCorrectNegativePredictions and
    # numIncorrectNegativePredictions
    boundForNumCorrectPositivePredictions <- min(numPositivePredictions, numUpregulatedGenes)
    boundForNumIncorrectPositivePredictions <- min(numPositivePredictions, numDownregulatedGenes)
    boundForNumIncorrectNegativePredictions <- min(numNegativePredictions, numUpregulatedGenes)
    boundForNumCorrectNegativePredictions <- min(numNegativePredictions, numDownregulatedGenes)
    
    # NOTE: Incorrect predictions are those that are predicted positive, but in the results turn out to be negative or vice versa
    
    maxNumberOfRows <- (boundForNumCorrectPositivePredictions + 1) * (boundForNumIncorrectPositivePredictions + 1) * (boundForNumIncorrectNegativePredictions + 
        1) * (boundForNumCorrectNegativePredictions + 1)
    matrixOfPossibleValues <- matrix(0, maxNumberOfRows, 4)
    
    # Let n_pp = possible value for numCorrectPositivePredictions n_pm = possible value for numIncorrectPositivePredictions n_mp = possible value for
    # numIncorrectNegativePredictions n_mm = possible value for numCorrectNegativePredictions (p is used to replace the + and m to replace the -)
    
    # Iterate over possible values of n++, n+-, n-+ and n-- and check if they are valid
    counter = 1
    for (n_pp in 0:boundForNumCorrectPositivePredictions) {
        for (n_pm in 0:boundForNumIncorrectPositivePredictions) {
            for (n_mp in 0:boundForNumIncorrectNegativePredictions) {
                for (n_mm in 0:boundForNumCorrectNegativePredictions) {
                    possibleValues <- c(n_pp, n_pm, n_mp, n_mm)
                    if (CheckPossibleValuesAreValid(predictionDataStats, experimentalDataStats, possibleValues)) {
                        matrixOfPossibleValues[counter, ] <- possibleValues
                        counter = counter + 1
                    }
                }
            }
        }
    }
    matrixOfPossibleValues <- matrixOfPossibleValues[1:counter - 1, ]
    return(matrixOfPossibleValues)
} 

#' @title get shortest paths from CCG
#' @description
#' Gets the node names in the shortest path from one node in a CCG to another
#' @param network built from iGraph
#' @param hypothesisnode hypothesis node ID
#' @param targetnode target node ID
#' @param showbothdirs where multiple paths from a positive and negative node, FALSE returns only the shortest. Otherwise both are returned.
#' @param quiet a flag to suppress output to console. FALSE by default.
#' @return a list of vectors containing the nodes of individual paths
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg = CreateCCG(network)
#' hypothesisnode = 1
#' targetnode = 10
#' GetShortestPathsFromCCG (ccg, hypothesisnode, targetnode)


GetShortestPathsFromCCG <- function(network, hypothesisnode, targetnode, showbothdirs = FALSE, quiet = FALSE) {
    
    if (is.numeric(hypothesisnode)) {
        hypothesis <- hypothesisnode
    } else {
        hypothesis <- GetNodeID(network, hypothesisnode)
    }
    
    if (is.numeric(targetnode)) {
        targetID <- targetnode
    } else {
        targetID <- which(igraph::V(network)$unsignedName == targetnode)
        if (!any(targetID)) {
            targetID <- which(igraph::V(network)$name == targetnode)
        }
    }
    
    shortestPath <- igraph::distances(network, hypothesis, targetID, mode = "out")
    
    # If there are no paths from hypothesis to target (+ or -), return NULL
    if (all(shortestPath == Inf)) {
        return(NULL)
    }
    
    # If there is more than one target (i.e. a + and a - node), decide which one(s) to include: - don't include one which has no path from the
    # hypothesis node - include the one which is closer to the hypothesis node (both if the two paths are equal) - include the one which is further from
    # the hypothesis node if showbothdirs is true (i.e. flag to override the default 'show only the closer node' behaviour)
    if (length(shortestPath) > 1) {
        if (shortestPath[1] == Inf) {
            targetID <- targetID[2]
        } else {
            if (shortestPath[2] == Inf) {
                targetID <- targetID[1]
            } else {
                if (!showbothdirs) {
                    if (shortestPath[1] < shortestPath[2]) {
                        targetID <- targetID[1]
                    } else {
                        if (shortestPath[2] < shortestPath[1]) {
                            targetID <- targetID[2]
                        }
                    }
                }
            }
        }
    }
    
    # Now assemble a list of the nodes in the paths from the hypothesis to the targets selected above
    nodesinpath <- list()
    inode <- 0
    for (itarget in 1:length(targetID)) {
        shortestPaths <- igraph::all_shortest_paths(network, hypothesis, targetID[itarget], mode = "out")
        for (i in 1:length(shortestPaths$res)) {
            inode <- inode + 1
            nodesinpath[[inode]] <- igraph::V(network)$name[shortestPaths$res[[i]]]
        }
    }
    
    if (!quiet) {
        for (i in 1:length(nodesinpath)) {
            for (j in 1:length(nodesinpath[[i]])) {
                cat(nodesinpath[[i]][j], "\t", sep = "")
            }
            cat("\b\n")
        }
        return(invisible(nodesinpath))
    } else {
        return(nodesinpath)
    }
} 

#' @title compare hypothesis
#' @description
#' Compare the predictions from a hypothesis with the experimental data returning an matrix with columns for node ID, predictions, experimental results and the corresponding scores.
#' @export
#' @concept CausalR
#' @param matrixOfPredictions a matrix of predictions
#' @param matrixOfExperimentalData a matrix of experimental data
#' @param ccg a CCG network (default=NULL) 
#' @param sourceNode A starting node (default=NULL)
#' @return a matrix containing predictions, observations and scores.
#' @examples
#' predictions <- matrix(c(1,2,3,+1,0,-1),ncol=2)
#' experimentalData <- matrix(c(1,2,4,+1,+1,-1),ncol=2)
#' ScoreHypothesis(predictions,experimentalData)
#' CompareHypothesis(predictions,experimentalData)


CompareHypothesis <- function(matrixOfPredictions, matrixOfExperimentalData, ccg = NULL, sourceNode = NULL) {
    
    intersectnodes <- intersect(matrixOfPredictions[, 1], matrixOfExperimentalData[, 1])
    
    n <- length(intersectnodes)
    
    comparison <- data.frame(matrix(0, n, 4))
    colnames(comparison) <- c("NodeID", "Prediction", "Experiment", "Score")
    
    if (n > 0) {
        for (i in 1:n) {
            comparison[i, 1] <- intersectnodes[i]
            comparison[i, 2] <- matrixOfPredictions[which(matrixOfPredictions[, 1] == intersectnodes[i]), 2]
            comparison[i, 3] <- matrixOfExperimentalData[which(matrixOfExperimentalData[, 1] == intersectnodes[i]), 2]
        }
        
        comparison[, 4] <- comparison[, 2] * comparison[, 3]
    }
    
    if (!is.null(ccg)) {
        rownames(comparison) <- GetNodeName(ccg, comparison[, 1])
    }
    
    # Sort the outputs into order - correct, then 0, then wrong, and within each of these sort by experimental value then predicted value
    comparison <- comparison[order(comparison[, 4], comparison[, 3], comparison[, 2], decreasing = TRUE), ]
    
    # Save results to an text file - first create the filename
    if (is.null(sourceNode) || (is.numeric(sourceNode) && is.null(ccg))) {
        filename <- paste(getwd(), "/explained_Nodes.txt", sep = "")
    } else {
        if (is.numeric(sourceNode)) {
            filename <- paste(getwd(), "/", GetNodeName(ccg, sourceNode, signed = TRUE), "_explained_Nodes.txt", sep = "")
        } else {
            filename <- paste(getwd(), "/", sourceNode, "_explained_Nodes.txt", sep = "")
        }
    }
    
    # Format the data for saving, and save it.
    if (is.null(ccg)) {
        rhoutput <- data.frame(comparison)
        colnames(rhoutput) <- c("NodeID", "Prediction", "Experiment", "Score")
        utils::write.table(rhoutput, filename, sep = "\t", row.names = FALSE, quote = FALSE)
    } else {
        rhoutput <- data.frame(comparison[, c(1, 1:4)])
        rhoutput[, 1] <- rownames(comparison)
        colnames(rhoutput) <- c("NodeName", "NodeID", "Prediction", "Experiment", "Score")
        utils::write.table(rhoutput, filename, sep = "\t", row.names = FALSE, quote = FALSE)
    }
    
    return(comparison)
} 

# A test suite for ScoreHypothesis and CompareHypothesis functions

test_ScoreHypothesisProducesTheCorrectScoreWhenAllNodesInPredictionsAreInTheExperimentalData <- function() {
    matrixOfPredictions <- matrix(c(1, 3, 1, -1), nrow = 2)
    matrixOfExperimentalData <- matrix(c(1, 2, 3, 1, -1, 0), nrow = 3)
    # 1 is correct (predicted +, observed +) 3 doesn't appear in the table (predicted -, observed 0)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkEquals(scoreBreakdown[1], 1)
    checkEquals(scoreBreakdown[2], 1)
    checkEquals(scoreBreakdown[3], 0)
    checkEquals(scoreBreakdown[4], 0)
    
    matrixOfExperimentalData2 <- matrix(c(1, 2, 3, -1, -1, 1), nrow = 3)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData2)
    checkEquals(scoreBreakdown[1], -2)
    checkEquals(scoreBreakdown[2], 0)
    checkEquals(scoreBreakdown[3], 2)
    checkEquals(scoreBreakdown[4], 0)
}

test_ScoreHypothesisProducesTheCorrectScoreWhenThePredictionsContainsANodeNotInTheExperimentalData <- function() {
    matrixOfPredictions <- matrix(c(1, 3, 1, -1), nrow = 2)
    matrixOfExperimentalData <- matrix(c(1, 2, 1, -1), nrow = 2)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkEquals(scoreBreakdown[1], 1)
    checkEquals(scoreBreakdown[2], 1)
    checkEquals(scoreBreakdown[3], 0)
    checkEquals(scoreBreakdown[4], 0)
}

test_ScoreHypothesisProducesTheCorrectScoreWhenNodesAreMisMatched <- function() {
    matrixOfPredictions <- matrix(c(1, 2, 4, 5, -1, 1, 1, -1), nrow = 4)
    matrixOfExperimentalData <- matrix(c(1, 3, 5, 1, -1, 1), nrow = 3)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkEquals(scoreBreakdown[1], -2)
    checkEquals(scoreBreakdown[2], 0)
    checkEquals(scoreBreakdown[3], 2)
    checkEquals(scoreBreakdown[4], 0)
}

test_ScoreHypothesisCorrectlyIdentifiesAmbiguousNodes <- function() {
    matrixOfPredictions <- matrix(c(1, 2, 3, 4, 5, -1, 0, 1, 1, 0), nrow = 5)
    matrixOfExperimentalData <- matrix(c(1, 3, 5, 1, 0, 1), nrow = 3)
    # 1 is incorrect 3 doesn't appear in the table (predicted +, observed 0) 5 is ambiguous (predicted conflicted, observed +)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkEquals(scoreBreakdown[1], -1)
    checkEquals(scoreBreakdown[2], 0)
    checkEquals(scoreBreakdown[3], 1)
    checkEquals(scoreBreakdown[4], 1)
}

test_ScoreHypothesisWorksWhenNoOverlap <- function() {
    matrixOfPredictions <- matrix(c(1, 2, 3, 4, 5, -1, 0, 1, 1, 0), nrow = 5)
    matrixOfExperimentalData <- matrix(c(6, 7, 8, 1, 0, 1), nrow = 3)
    scoreBreakdown <- CausalR:::ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkTrue(all(scoreBreakdown == 0))
}

test_CompareHypothesisWorksWhenNoOverlap <- function() {
    matrixOfPredictions <- matrix(c(1, 2, 3, 4, 5, -1, 0, 1, 1, 0), nrow = 5)
    matrixOfExperimentalData <- matrix(c(6, 7, 8, 1, 0, 1), nrow = 3)
    comparison <- CausalR:::CompareHypothesis(matrixOfPredictions, matrixOfExperimentalData)
    checkEquals(dim(comparison)[1], 0)
} 

#' @title make predictions from CCG
#' @description
#' Create a matrix of predictions for a particular hypothesis starting from a network 
#' with separate nodes for up- and down-regulation (+ve and -ve).
#' The output is an array containing the relationship between each node and the hypothesis.
#' The hypothesis provided will be the vertex id of one of the nodes in the network
#' (as an integer or name including + or - for up/down regulation).
#' The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation.
#' (It generally shouldn't be necessary to reverse the sign of a node when working from
#' a CCG, but this facility is included for consistency with MakePredictionsFromCG)

#' @param hypothesisnode a hypothesis node
#' @param signOfHypothesis the direction of change of hypothesis node
#' @param network a computational causal graph
#' @param delta the number of edges across which the hypothesis should be followed
#' @param nodesInExperimentalData the number of nodes in experimental data
#' @return an matrix containing the relationship between each node and the hypothesis
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg <- CreateCCG(network)
#' MakePredictionsFromCCG('NodeA', +1, ccg, 2)


MakePredictionsFromCCG <- function(hypothesisnode, signOfHypothesis, network, delta, nodesInExperimentalData = NULL) {
    # Create a matrix of predictions for a particular hypothesis starting from a network with separate nodes for up- and down-regulation (+ve and -ve).
    # The output is an array containing the relationship between each node and the hypothesis.  The hypothesis provided will be the vertex id of one of
    # the nodes in the network (as an integer or name including + or - for up/down regulation).  The signOfHypothesis variable should be a 1 or -1,
    # indicating up/down regulation.  (It generally shouldn't be necessary to reverse the sign of a node when working from a CCG, but this facility is
    # included for consistency with MakePredictionsFromCG)
    
    signOfHypothesis <- as.integer(signOfHypothesis)
    delta <- as.integer(delta)
    
    if (is.numeric(hypothesisnode)) {
        hypothesis <- hypothesisnode
    } else {
        hypothesis <- GetNodeID(network, hypothesisnode)
    }
    
    # Check that the input hypothesis is a possible id. Ids are numbered from one to the number of vertices, so it is enough to check the hypothesis is
    # not less than one or greater than the number of vertices.
    if (is.na(hypothesis) || length(hypothesis) != 1 || hypothesis < 1 || hypothesis > igraph::gorder(network)) {
        stop("Couldn't create predictions - hypothesis ", hypothesisnode, " is not present in the network")
    }
    nNodes <- igraph::gorder(network)
    
    # Array for the distance from the hypothesis to each node. Default value of (delta+1), i.e. too far away to matter
    distance <- array(delta + 1, nNodes)
    
    # Number of nodes at each distance from the hypothesis
    n <- array(0, delta + 1)
    
    # Interaction of the hypothesis node with itself
    distance[hypothesis] <- 0
    n[1] <- 1
    
    if (delta > 0) {
        for (currentStep in 1:delta) {
            # As of igraph V1.0.0 functions return sequences instead of numeric ids. as.vector() converts these back to ids.
            nn <- as.vector(igraph::ego(network, currentStep, hypothesis, "out")[[1]])  # All nodes out to currentStep
            n[currentStep + 1] <- length(nn)
            if (n[currentStep + 1] > n[currentStep]) {
                # For all of the nodes which are new on this step, set distance to currentStep
                distance[nn[n[currentStep] + 1:n[currentStep + 1]]] <- currentStep
            }
        }
    }
    
    # Now process the distances to work out the interactions.
    interactions <- array(NA, nNodes/2)
    interactions[distance[1:(nNodes/2)] < distance[(nNodes/2 + 1):nNodes]] <- 1  # NodeX+ is closer than NodeX-
    interactions[distance[1:(nNodes/2)] > distance[(nNodes/2 + 1):nNodes]] <- -1  # NodeX+ is further than NodeX-
    interactions[distance[1:(nNodes/2)] <= delta & distance[1:(nNodes/2)] == distance[(nNodes/2 + 1):nNodes]] <- 0  # NodeX+ and NodeX- are same distance
    # interactions[distance[1:(nNodes/2)] > delta & distance[(nNodes/2+1):nNodes] > delta] = NA # Neither NodeX+ and NodeX- are reached within delta
    # (don't actually need to do this as NA is the default value)
    
    if (is.null(nodesInExperimentalData)) {
        matrixOfCausalRelationships <- matrix(c((1:(nNodes/2))[!is.na(interactions)], interactions[!is.na(interactions)]), ncol = 2)
    } else {
        matrixOfCausalRelationships <- matrix(c(nodesInExperimentalData, interactions[nodesInExperimentalData]), ncol = 2)
        matrixOfCausalRelationships <- matrix(matrixOfCausalRelationships[!is.na(matrixOfCausalRelationships[, 2]), ], ncol = 2)
    }
    
    if (signOfHypothesis < 0) {
        matrixOfCausalRelationships[, 2] <- -matrixOfCausalRelationships[, 2]
    }
    
    if (!nrow(matrixOfCausalRelationships) == 0) {
        return(matrixOfCausalRelationships)
    } else {
        return(NULL)
    }
} 

#' @title compute final distribution
#' @description
#' Computes a final reference distribution of the score used to compute the final p-value. 

#' @param resultsMatrix a matrix containing the scores and weights from which the distribution is to be calculated
#' @return distributionMatrix  a matrix containing the reference distribution for the score 


# *** Summary *** Description: Takes the matrix which contains the scores and weights for the different n++, n+- etc.  values and combines those
# with the same score to get a matrix which represents the distribution of what scores the hypothesis can have under certain restrictions

# resultsTable contains the scores and weights. We are no longer interest in the individual values of n++, n+- etc., we just want to find the
# distribution of scores.

# When resultsMatrix is only one row (two values), the max and min functions below don't work because the array has been changed to a vector. To put
# it back into the form of a matrix the following if command is used

ComputeFinalDistribution <- function(resultsMatrix) {
    if (length(resultsMatrix) == 2) {
        resultsMatrix <- t(matrix(resultsMatrix, nrow = 2))
    }
    
    maxScore = max(resultsMatrix[, 1])
    minScore = min(resultsMatrix[, 1])
    
    # Pre-allocate the size of the storage array
    distributionMatrix <- matrix(0, (maxScore - minScore + 1), 2)
    numRows <- nrow(resultsMatrix)
    
    counter <- 1
    for (score in minScore:maxScore) {
        probability <- 0
        for (i in 1:numRows) {
            if (score == resultsMatrix[i, 1]) {
                probability <- probability + resultsMatrix[i, 2]
            }
        }
        if (probability > 0) {
            distributionMatrix[counter, ] <- c(score, probability)
            counter <- counter + 1
        }
    }
    
    # Remove the rows that were not populated in the for loops above
    distributionMatrix <- distributionMatrix[1:counter - 1, ]
    
    return(distributionMatrix)
} 

#' @title find maximum D value
#' @description
#' computes the maximum possible D-value 
#' for given values q+, q-, q0 and n+, n-, n0.
#' 
#' @param predictionListStats a vector containing the predicted values q+, q- and q0:
#' numbers of positive, negative and non-significant/contradictory predictions
#' @param experimentalDataStats A vector containing the observed values n+, n- and n0:
#' numbers of positive, negative and non-significant/contradictory observations
#' @param logOfFactorialOfPredictionListStats a vector containing the log of the factorial value for
#' each entry in predictionListStats
#' @param returnlog should the result be returned as a log; default FALSE
#' @return the maximum possible D value

FindMaximumDValue <- function(predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, returnlog = FALSE) {
    
    
    twoByTwoContingencyTable <- FindApproximateValuesThatWillMaximiseDValue(predictionListStats, experimentalDataStats)
    
    maximumDValue <- GetMaximumDValueFromTwoByTwoContingencyTable(twoByTwoContingencyTable, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
        returnlog)
    
    return(maximumDValue)
} 

#' @title  run ScanR
#' @description 
#' This function will return nodes regulated by the given hypothesisGene
#' @param network Computational Causal Graph, as an igraph.
#' @param experimentalData The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation.
#' @param numberOfDeltaToScan Iteratively scan for 1 to numberOfDeltaToScan delta values
#' @param topNumGenes A value to select top genes to report (typically top 100 genes)
#' @param correctPredictionsThreshold Minimal score for p-values calculation. Value is passed to RankTheHypothesis - scores below this value will get NAs for p-value and enrichment p-value. The default is Inf, so that no p-values are calculated.
#' @param writeResultFiles If set to TRUE the results of the scan will be written to two text files in the working directory. Default is TRUE.
#' @param writeNetworkFiles If set to "all" .sif files and corresponding _anno.txt files will be generated for the top correctly explained, incorrectly explained and ambiguously explained nodes. If set to "correct" they will only be calculated for correctly explained nodes. If set to "none", no networks will be generated. Default is "all".
#' @param doParallel A flag for running RankTheHypothesis in parallel mode. Default is FALSE.
#' @param numCores Number of cores to use if using parallel mode. If the default value of NULL is used, it will attempt to detect the number of cores available and use all of them bar one.
#' @param quiet a flag to suppress output to console. FALSE by default.
#' @param outputDir the directory to output the files to. Default is the working directory
#' @return returns list of genes from each delta scan run
#' @export
#' @concept CausalR
#' @examples
#' numberOfDeltaToScan <- 2
#' topNumGenes <- 4
#' #get path to example network file
#' networkFile <- system.file(package = 'CausalR', 'extdata', 'testNetwork.sif')
#' #create ccg
#' network <-  CreateCCG(networkFile)
#' #get path to example experimental data
#' experimentalDataFile <- system.file(package = 'CausalR', 'extdata', 'testData.txt')
#' #read in experimetal data
#' experimentalData <-  ReadExperimentalData(experimentalDataFile, network)
#' #run in single threaded mode
#' runSCANR(network, experimentalData, numberOfDeltaToScan, topNumGenes)
#' #run in parallel mode
#' runSCANR(network, experimentalData, numberOfDeltaToScan, topNumGenes,
#'          doParallel = TRUE, numCores = 2)


runSCANR <- function(network, experimentalData, numberOfDeltaToScan = 5, topNumGenes = 150, correctPredictionsThreshold = Inf, writeResultFiles = TRUE, writeNetworkFiles = "all", doParallel=FALSE, numCores=NULL, quiet = FALSE, outputDir = getwd()) {
    
    numberOfDeltaToScan <- as.integer(numberOfDeltaToScan)
    topNumGenes <- as.integer(topNumGenes)
    
    # If outputDir doesn't exist, create it
    if (!file.exists(file.path(outputDir))){
        dir.create(file.path(outputDir))
    }
    
    if (writeNetworkFiles == "all") {
        writeNetworkFiles <- TRUE
        correctlyExplainedOnly <- FALSE
    } else if (writeNetworkFiles == "correct") {
        writeNetworkFiles <- TRUE
        correctlyExplainedOnly <- TRUE
    } else if (writeNetworkFiles == "none"){
        writeNetworkFiles <- FALSE
    } else {
        stop("writeNetworkFiles should be set to either \"all\", \"correct\" or \"none\"")
    }
    
    resultsFileOutputName <- paste0(network$name, "-", colnames(experimentalData)[1], "-deltaScanned", numberOfDeltaToScan, "-top", topNumGenes)
    
    if (as.integer(numberOfDeltaToScan) > 0) {
        deltaToScan <- 1:numberOfDeltaToScan
    } else {
        stop("numberOfDeltaToScan must be an integer > 0")
    }
    
    # Create empty data frame and set column names to delta scan values
    testList <- data.frame(matrix(nrow = topNumGenes, ncol = numberOfDeltaToScan))
    colnames(testList) <- deltaToScan
    
    # Run RankTheHypotheses on each delta value and fill testList
    for (delta in deltaToScan) {
        rankedHypo <- RankTheHypotheses(network, experimentalData, delta, correctPredictionsThreshold, doParallel=doParallel, numCores=numCores, writeFile = FALSE, quiet = TRUE)
        testList[,delta] <- row.names(rankedHypo)[1:topNumGenes]
    }
    
    commonNodes <- Reduce(intersect, testList) # Nodes that appear in top 'topNumGenes' in every delta value
    uniqueNodes <- data.frame(names = unique(unlist(testList)), maxDelta = NA) # Nodes that appear in at least one list
    
    for (i in 1:nrow(uniqueNodes)) {
        # Find which columns contain this gene
        colsIn <- apply(testList, 2, function(r) any(r %in% uniqueNodes$names[i]))
        # Get the names of these columns as integers, and find the max
        uniqueNodes$maxDelta[i] <- max(as.integer(colnames(testList))[colsIn])
    }
    
    if (writeResultFiles) {
        if(!quiet) {
            cat("Writing TopNodes file to: ", paste0(outputDir, "/TopNodes-", resultsFileOutputName, ".txt"), "\n" , sep="")
            cat("Writing CommonNodes file to: ", paste0(outputDir, "/CommonNodes-", resultsFileOutputName, ".txt"), "\n", sep="")
        }
        utils::write.table(uniqueNodes, file = paste0(outputDir, "/TopNodes-", resultsFileOutputName, ".txt"), sep = "\t", row.names = FALSE, quote = FALSE)
        write(commonNodes, file = paste0(outputDir, "/CommonNodes-", resultsFileOutputName, ".txt"))
    }
    
    
    if (writeNetworkFiles) {
        for (node in commonNodes) {
            # get separate node name and regulation sign from the combined string
            negativeSign <- unlist(gregexpr(pattern ='-', text = node, fixed = TRUE))
            positiveSign <- unlist(gregexpr(pattern ='+', text = node, fixed = TRUE))
            
            if (negativeSign == nchar(node) & positiveSign < 0) {
                hypothesisnode <- substr(node, 1, nchar(node)-1)
                signOfHypothesis <- -1
            } else if (positiveSign == nchar(node) & negativeSign < 0) {
                hypothesisnode <- substr(node, 1, nchar(node)-1)
                signOfHypothesis <- +1
            } else {
                stop("Cannot deduce sign of hypothesis from node")
            }
            
            networkFilesOutputName <- paste0(network$name, "-", colnames(experimentalData)[1], "-delta", delta, "-top", topNumGenes, "-", node)
            #trimws(paste(format(Sys.time(), "%Y-%m-%d_%H.%M.%S"), "My files"))
            
            WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData, delta = numberOfDeltaToScan, outputFilesName = networkFilesOutputName, correctlyExplainedOnly = correctlyExplainedOnly, quiet = quiet, outputDir = outputDir)
        }
    }

    return(list(testList = testList, uniqueNodes = uniqueNodes, commonNodes = commonNodes))
} 

#' @title 
#' add weights to edges
#' @description
#' Adds weight information to the edges of given network (1 for activation and -1 for inhibition)

#' @param network an igraph constructed from the original .sif file
#' @param tableOfInteractions a column of the corresponding .sif file indicating the direction of activation/interaction
#' @return an augmented network

AddWeightsToEdges <- function(network, tableOfInteractions) {
    
    matrixOfInteractions <- as.matrix(tableOfInteractions)
    # matrixOfInteractions will be used to define an edge property called 'Weight' for each edge. If the value in matrixOfInteractions is
    # 'Activates/Activation' the value of the weight is +1.  Otherwise if the value is 'Inhibits/Inhibition', the value of weight will be -1.
    matrixOfWeights <- GetWeightsFromInteractionInformation(matrixOfInteractions)
    
    edgesOfNetwork <- igraph::E(network)
    network = igraph::set_edge_attr(network, "Weight", edgesOfNetwork, matrixOfWeights)
    
    return(network)
} 

#' @title Calculate Significance Using Cubic Algorithm 
#' @description
#' Calculate the p-value of a score given the hypothesis score and the distribution table (calculated using the cubic algorithm  1b in Assessing statistical significance in causal graphs - Chindelevitch et al)

#' @export
#' @concept CausalR
#' @param hypothesisScore The score whose p-value we want to find.
#' @param predictionListStats Number of predicted up-regulated, 
#'predicted down-regulated and ambiguous predictions.
#' @param experimentalDataStats Number of up-regulated, down-regulated and not 
#' significantly changed transcripts in the experimental data. 
#' @param epsilon  The threshold that is used when calculating the p-value 
#' using the cubic algorithm. (Defaults to 1e-5, only used for the cubic 
#' algorithm, ignored if useCubicAlgorithm is FALSE.)
#' @examples
#' CalculateSignificance(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingCubicAlgorithm1b(5, c(7,4,19), c(6,6,18), 1e-5)

#' @return  p value

CalculateSignificanceUsingCubicAlgorithm1b <- function(hypothesisScore, predictionListStats, experimentalDataStats, epsilon) {
    
    # Calculate the log of the factorial for all values in predictionListStats, this is to reduce the number of computations done later on.
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    
    logDMax <- FindMaximumDValue(predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, TRUE)
    
    # Number of positive predictions from the network. This will be an upper bound for r+ (r_p)
    q_p <- predictionListStats[1]
    # Number of negative predictions from the network. This will be an upper bound for r- (r_m)
    q_m <- predictionListStats[2]
    
    # Number of positive results. This will be an upper bound for c+ (c_p)
    n_p <- experimentalDataStats[1]
    # Number of negative results. This will be an upper bound for c- (c_m)
    n_m <- experimentalDataStats[2]
    
    # Array for the D values - first element is the total D values which have a score equal to or better than the cut-off, second element is all D
    # values (i.e. the ratio is the P value).
    logepsDMax <- log(epsilon) + logDMax
    weights <- c(0, 0)
    
    # In algorithm a superfamily is defined by the left 3x2 submatrix of the contingency table. Now we only need to interate over two quantities: r+ and
    # r-.
    for (r_p in 0:q_p) {
        
        # for 0 to the limit of negative predictions
        for (r_m in 0:q_m) {
            
            # for 0 to the limit of number of positive results Calculate r0
            r_z <- n_p + n_m - (r_p + r_m)
            
            # Need to check that n00 is postive, to ensure it is a valid superfamily
            if (r_z <= predictionListStats[3] && r_z >= 0) {
                
                # The five values in the array below define the superfamily
                weights <- GetWeightsAboveHypothesisScoreForAThreeByTwoTable(weights, r_p, r_m, r_z, n_p, n_m, predictionListStats, experimentalDataStats, 
                    logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax)
            }
        }
    }
    
    pValue <- weights[1]/weights[2]
    
    return(pValue)
} 

#' @title get score for numbers of correct and incorrect predictions
#' @description
#' Returns all possible rounding combinations of a 2x2 table. 
#' Given the values of n++, n+-, n-+ and n-- (stored in twoByTwoContingencyTable) this function will
#' compute all possibilities of rounding each value up or down. 
#' 
#' @param twoByTwoContingencyTable    Approximate values of n++, n+-, n-+ and n--, these values are
#' calculated to optimise the D-value (see page 6 of Assessing
#' statistical significance of causal graphs)
#' @return a matrix of rounding combinations

GetAllPossibleRoundingCombinations <- function(twoByTwoContingencyTable) {
    # Given the values of n++, n+-, n-+ and n-- (stored in twoByTwoContingencyTable) this function will compute all possibilities of rounding each value
    # up or down.  Inputs: twoByTwoContingencyTable Approximate values of n++, n+-, n-+ and n--, these values are calculated to optimise the D-value
    # (see page 6 of Assessing statistical significance of causal graphs)
    
    
    f_n_pp <- floor(twoByTwoContingencyTable[1])
    f_n_pm <- floor(twoByTwoContingencyTable[2])
    f_n_mp <- floor(twoByTwoContingencyTable[3])
    f_n_mm <- floor(twoByTwoContingencyTable[4])
    
    c_n_pp <- ceiling(twoByTwoContingencyTable[1])
    c_n_pm <- ceiling(twoByTwoContingencyTable[2])
    c_n_mp <- ceiling(twoByTwoContingencyTable[3])
    c_n_mm <- ceiling(twoByTwoContingencyTable[4])
    
    roundingCombinations <- matrix(c(f_n_pp, f_n_pm, f_n_mp, f_n_mm, f_n_pp, f_n_pm, f_n_mp, c_n_mm, f_n_pp, f_n_pm, c_n_mp, f_n_mm, f_n_pp, f_n_pm, 
        c_n_mp, c_n_mm, f_n_pp, c_n_pm, f_n_mp, f_n_mm, f_n_pp, c_n_pm, f_n_mp, c_n_mm, f_n_pp, c_n_pm, c_n_mp, f_n_mm, f_n_pp, c_n_pm, c_n_mp, c_n_mm, 
        c_n_pp, f_n_pm, f_n_mp, f_n_mm, c_n_pp, f_n_pm, f_n_mp, c_n_mm, c_n_pp, f_n_pm, c_n_mp, f_n_mm, c_n_pp, f_n_pm, c_n_mp, c_n_mm, c_n_pp, c_n_pm, 
        f_n_mp, f_n_mm, c_n_pp, c_n_pm, f_n_mp, c_n_mm, c_n_pp, c_n_pm, c_n_mp, f_n_mm, c_n_pp, c_n_pm, c_n_mp, c_n_mm), ncol = 4, byrow = TRUE)
    
    
    # If any of the input values are integers then there will be duplicates in the list Remove the duplicates
    keep = array(TRUE, 16)
    if (f_n_pp == c_n_pp) {
        keep[9:16] <- FALSE
    }
    if (f_n_pm == c_n_pm) {
        keep[c(5:8, 13:16)] <- FALSE
    }
    if (f_n_mp == c_n_mp) {
        keep[c(3:4, 7:8, 11:12, 15:16)] <- FALSE
    }
    if (f_n_mm == c_n_mm) {
        keep[c(2, 4, 6, 8, 10, 12, 14, 16)] <- FALSE
    }
    
    return(roundingCombinations[keep, ])
} 

#' @title computes an approximate maximum D value or weight
#' @description
#' Computes an approximate maximum D value (or weight).
#' The calculation is  approximate since only the first valid D value that is round. This has been
#' done to speed up the overall algorithm - to get the exact answer use GetMaximumDValueFromTwoByTwoContingencyTable.
#' @param n_pp the count n++ from the prediction-observation contingency matrix
#' @param n_pm the count n+- from the prediction-observation contingency matrix
#' @param n_mp the count n-+ from the prediction-observation contingency matrix
#' @param n_mm the count n-- from the prediction-observation contingency matrix
#' @param predictionListStats a vector containing the values q+, q- and q0: the number of positive, negative, non-significant/contradictory predictions
#' @param experimentalDataStats a vector containing the values n+, n- and n0: the number of positive, negative, non-significant/contradictory observations 
#' @param logOfFactorialOfPredictionListStats a vector containing the log of the factorial value for each entry in predictionListStats
#' @param returnlog return the result as a log, default is FALSE
#' @return the maximum D value or weight



GetApproximateMaximumDValueFromTwoByTwoContingencyTable <- function(n_pp, n_pm, n_mp, n_mm, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
    returnlog = FALSE) {
    # A function that will compute an approximate maximum D value (or weight).  The reason it is described as approiximate is that this function returns
    # the first valid D value that is found. This has been done to speed up the overall algorithm - to get the exact answer use
    # GetMaximumDValueFromTwoByTwoContingencyTable.
    
    # Inputs: twoByTwoContingencyTable Approximate values of n++, n+-, n-+ and n--, these values are calculated to optimise the D-value (see page 6 of
    # Assessing statistical significance of causal graphs) predictionListStats A vector containing the values q+, q- and q0 (the number of
    # positive/negative/non-significant (or contradictory) predictions) experimentalDataStats A vector containing the values n+, n- and n0 (the number
    # of positive/negative/non-significant (or contradictory) transcripts in the results) logOfFactorialOfPredictionListStats
    
    threeByThreeContingencyTable <- PopulateTheThreeByThreeContingencyTable(n_pp, n_pm, n_mp, n_mm, predictionListStats, experimentalDataStats)
    
    # Some of the combinationations produce an invalid contingency table - we can check this by seeing if any of the values in the table are negative.
    if (min(threeByThreeContingencyTable) >= 0) {
        # To speed up calculation, we will only calculate the D value for the first valid combination.
        weight <- CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable, logOfFactorialOfPredictionListStats, returnlog)
    } else {
        stop("Could not find a valid three by three contigency table given the approximate values in the two by two contingency table that will maximise the D-value.")
    }
    
    return(weight)
    
} 

#' @title make predictions
#' @description
#' Creates a matrix of predictions for a particular hypothesis.
#' The output is an array containing the relationship between each node and the hypothesis.
#' The hypothesis provided will be the vertex id of one of the nodes in the network
#' (as an integer node ID or name, including + or - for up/down regulation in the case of a CCG).
#' The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation.

#' @export
#' @concept CausalR
#' @param hypothesisnode the node in the causal graph from which predictions should be made.  Can be either a (numerical) node ID or a (string) node name.
#' @param signOfHypothesis whether the hypothesis node is up- or down-regulated. Should be +1 or -1.
#' @param network a (Computational) Causal Graph, as an igraph.
#' @param delta the distance to search within the causal graph.
#' @param nodesInExperimentalData optional. Nodes to include in the output. Should be a list of node IDs.
#' @return a matrix of predictions for the given particular hypothesis
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg <- CreateCCG(network)
#' predictions <- MakePredictions('NodeA', +1, ccg, 2)

MakePredictions <- function(hypothesisnode, signOfHypothesis, network, delta, nodesInExperimentalData = NULL) {
    # Create a matrix of predictions for a particular hypothesis.  The output is an array containing the relationship between each node and the
    # hypothesis.  The hypothesis provided will be the vertex id of one of the nodes in the network (as an integer node ID or name, including + or - for
    # up/down regulation in the case of a CCG).  The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation.
    
    if (network$isCCG) {
        predictions <- MakePredictionsFromCCG(hypothesisnode, signOfHypothesis, network, delta, nodesInExperimentalData)
    } else {
        predictions <- MakePredictionsFromCG(hypothesisnode, signOfHypothesis, network, delta, nodesInExperimentalData)
    }
    return(predictions)
} 

#' @title counts the number of positive and negative entries
#' @description
#' Counts the number of entries in the in the second column of an input table that are +1 or -1.
#' 
#' @param dataList an array or dataframe in which the second column is numeric
#' @return a vector of two components, the first of which giving the number of +1 entries, the second the number of -1's.
#' @export
#' @concept CausalR
#' @examples
#' expData<-read.table(system.file(package='CausalR', 'extdata', 'testData.txt'))
#' GetNumberOfPositiveAndNegativeEntries(expData)

GetNumberOfPositiveAndNegativeEntries <- function(dataList) {
    
    
    data <- as.numeric(dataList[, 2])
    dataStats <- c(sum(data == 1), sum(data == -1))
    
    return(dataStats)
} 

#' @title compute causal relationships matrix
#' @description 
#' Get a matrix of causal relationships from the network and the IDs of connected nodes
#' @param hypothesis a hypothesis node
#' @param network a CCG network
#' @param idsOfConnectedNodesFromSubgraph a list of connected nodes in the subgraph of interest
#' @return  causal relationships matrix

GetMatrixOfCausalRelationships <- function(hypothesis, network, idsOfConnectedNodesFromSubgraph) {
    
    nNodes <- length(idsOfConnectedNodesFromSubgraph)
    
    causalRelationships <- array(0, nNodes)
    
    for (iNode in 1:nNodes) {
        if (hypothesis == idsOfConnectedNodesFromSubgraph[iNode]) {
            # Hypothesis node's interaction with itself has to be 1.
            causalRelationships[iNode] <- 1
        } else {
            shortestPaths <- igraph::all_shortest_paths(network, hypothesis, idsOfConnectedNodesFromSubgraph[iNode], "out")
            numberOfShortestPaths <- length(shortestPaths[[1]])
            # Determine interaction type for first path
            interactionValue <- DetermineInteractionTypeOfPath(network, shortestPaths[[1]][1])
            
            if (numberOfShortestPaths > 1) {
                # if there is more than one shortest path, two things can happen: 1. they could all agree, in which case we can take any of the values in the vector
                # to populate the data frame 2. they could disagree, in this case the interaction given the value 0 (as opposed to 1/-1 for activation/inhibition.
                
                for (counter in 2:numberOfShortestPaths) {
                    interactionValueForAlternativePath <- DetermineInteractionTypeOfPath(network, shortestPaths[[1]][counter])
                    if (interactionValueForAlternativePath != interactionValue) {
                        interactionValue <- 0
                    (break)()
                    }
                }
            }
            causalRelationships[iNode] <- interactionValue
        }
    }
    
    return(causalRelationships)
} 

#' @title get weights from interaction information
#' @description
#' Returns a matrix of weights (-1,0,+1) indicating the direction of regulation from the interaction information.
#' @param interactionInfo a central column of the .sif file, giving the type of edge interaction
#' @return a matrix of weights corresponding the the direction of regulation

GetWeightsFromInteractionInformation <- function(interactionInfo) {
    
    
    numEdges <- length(interactionInfo)
    matrixOfWeight <- matrix(0, numEdges, 1)
    # If the interaction is activation, the weight assigned is 1, if the interaction is inhibition the weight assigned is -1.
    matrixOfWeight[interactionInfo == "Activates"] <- 1
    matrixOfWeight[interactionInfo == "Activation"] <- 1
    matrixOfWeight[interactionInfo == "Inhibits"] <- -1
    matrixOfWeight[interactionInfo == "Inhibition"] <- -1
    
    return(matrixOfWeight)
} 

#' @title read experimental data
#' @description 
#' Reads experimental data for the causal reasoning algorithm from a text file.
#' @usage ReadExperimentalData(fileName, network, removeDuplicates)



#' @export
#' @concept CausalR
#' @param fileName a file containing the experimental data (text file format)
#' @param network  a (Computational) Causal Graph, as an igraph.
#' @param removeDuplicates Optional, defaults to true. Remove duplicated nodes the experimental file (i.e. where the result for a node is repeated, use the first value given only; the alternative is to return a result which contains multiple rows for this node).
#' @return (n x 2) matrix of nodes and direction of regulation. The first column of the matrix contains the node IDs from the network, and the second contains the experimental values.
#' @examples
#' #get path to example network file
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ##create ccg
#' ccg <- CreateCCG(network)
#' #get path to example experimental data
#' fileName<- system.file(package='CausalR', 'extdata', 'testData.txt')

#' ReadExperimentalData(fileName, ccg)


ReadExperimentalData <- function(fileName, network, removeDuplicates = TRUE) {
    
    if (!(igraph::is_igraph(network) || is.null(network))) {
        stop("network input must be an igraph")
    }
    
    table <- utils::read.table(fileName, colClasses = "character")
    
    isValid <- ValidateFormatOfDataTable(table)
    
    if (!isValid) {
        stop()
    }
    
    # Check for / deal with genes which are listed more than once
    if (anyDuplicated(table[, 1])) {
        if (removeDuplicates) {
            warning("Some genes have more than one result in ", fileName, " - duplicates have been removed by keeping the first result in the file.")
            table <- table[!duplicated(table[, 1]), ]
        } else {
            warning("Some genes have more than one result in ", fileName, " - duplicates have not been removed, scoring functions will not work.")
        }
    }
    
    if (is.null(network)) {
        # No network info provided - return the raw table (Note that this option is provided purely for testing purposes, e.g. to make sure that the table
        # is being read in correctly, and is not intended to be used by an end user.)
        return(table)
    } else {
        processedData <- ProcessExperimentalData(table, network)
        # Name the first column from the fileName without the path or the extension
        colnames(processedData) <- c(tools::file_path_sans_ext(basename(fileName)), NA)
        return(processedData)
    }
} 

# Test suite for the WriteExplainedNodesToSifFile function

test_WriteExplainedNodesToSifFileProducesTheCorrectSifFileWithDefaultNameAndCorrectOnly <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")


    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, correctlyExplainedOnly = TRUE)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*.sif")), 1)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-testNetwork1-testData1-delta2-Node0+.sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 5)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
    checkEquals(c(outputData[2,1], outputData[2,2], outputData[2,3]), c("Node0", "Inhibits", "Node2"))
    checkEquals(c(outputData[3,1], outputData[3,2], outputData[3,3]), c("Node1", "Activates", "Node3"))
    checkEquals(c(outputData[4,1], outputData[4,2], outputData[4,3]), c("Node2", "Activates", "Node6"))
    checkEquals(c(outputData[5,1], outputData[5,2], outputData[5,3]), c("Node2", "Activates", "Node7"))
}


test_WriteExplainedNodesToSifFileProducesTheCorrectSifFileWithCustomNameAndCorrectOnly <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    outputFilesName <- basename(tempfile)
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, outputFilesName, correctlyExplainedOnly = TRUE)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*.sif")), 1)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-", basename(tempfile), ".sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 5)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
    checkEquals(c(outputData[2,1], outputData[2,2], outputData[2,3]), c("Node0", "Inhibits", "Node2"))
    checkEquals(c(outputData[3,1], outputData[3,2], outputData[3,3]), c("Node1", "Activates", "Node3"))
    checkEquals(c(outputData[4,1], outputData[4,2], outputData[4,3]), c("Node2", "Activates", "Node6"))
    checkEquals(c(outputData[5,1], outputData[5,2], outputData[5,3]), c("Node2", "Activates", "Node7"))
}


test_WriteExplainedNodesToSifFileProducesTheCorrectSifFilesWithDefaultNames <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*.sif")), 3)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-testNetwork1-testData1-delta2-Node0+.sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 5)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
    checkEquals(c(outputData[2,1], outputData[2,2], outputData[2,3]), c("Node0", "Inhibits", "Node2"))
    checkEquals(c(outputData[3,1], outputData[3,2], outputData[3,3]), c("Node1", "Activates", "Node3"))
    checkEquals(c(outputData[4,1], outputData[4,2], outputData[4,3]), c("Node2", "Activates", "Node6"))
    checkEquals(c(outputData[5,1], outputData[5,2], outputData[5,3]), c("Node2", "Activates", "Node7"))
    
    outputData <- read.table(paste0(outputDir, "/incorExplainedNodes-testNetwork1-testData1-delta2-Node0+.sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 1)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Inhibits", "Node2"))

    outputData <- read.table(paste0(outputDir, "/ambExplainedNodes-testNetwork1-testData1-delta2-Node0+.sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 4)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
    checkEquals(c(outputData[2,1], outputData[2,2], outputData[2,3]), c("Node0", "Inhibits", "Node2"))
    checkEquals(c(outputData[3,1], outputData[3,2], outputData[3,3]), c("Node1", "Inhibits", "Node5"))
    checkEquals(c(outputData[4,1], outputData[4,2], outputData[4,3]), c("Node2", "Inhibits", "Node5"))
}


test_WriteExplainedNodesToSifFileProducesTheCorrectAnnoFileWithDefaultNameAndCorrectOnly <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, correctlyExplainedOnly = TRUE)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*_anno.txt")), 1)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-testNetwork1-testData1-delta2-Node0+_anno.txt"), stringsAsFactors = FALSE, header = TRUE)
    checkEquals(nrow(outputData), 5)
    checkEquals(names(outputData), c("NodeID", "Regulation"))
    checkEquals(outputData[1,1], "Node0")
    checkEquals(outputData[1,2], 1)
    checkEquals(outputData[2,1], "Node1")
    checkEquals(outputData[2,2], 1)
    checkEquals(outputData[3,1], "Node3")
    checkEquals(outputData[3,2], 1)
    checkEquals(outputData[4,1], "Node6")
    checkEquals(outputData[4,2], -1)
    checkEquals(outputData[5,1], "Node7")
    checkEquals(outputData[5,2], -1)
}


test_WriteExplainedNodesToSifFileProducesTheCorrectAnnoFileWithCustomNameAndCorrectOnly <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    outputFilesName <- basename(tempfile)
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, outputFilesName, correctlyExplainedOnly = TRUE)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*_anno.txt")), 1)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-", basename(tempfile), "_anno.txt"), stringsAsFactors = FALSE, header = TRUE)
    checkEquals(nrow(outputData), 5)
    checkEquals(names(outputData), c("NodeID", "Regulation"))
    checkEquals(outputData[1,1], "Node0")
    checkEquals(outputData[1,2], 1)
    checkEquals(outputData[2,1], "Node1")
    checkEquals(outputData[2,2], 1)
    checkEquals(outputData[3,1], "Node3")
    checkEquals(outputData[3,2], 1)
    checkEquals(outputData[4,1], "Node6")
    checkEquals(outputData[4,2], -1)
    checkEquals(outputData[5,1], "Node7")
    checkEquals(outputData[5,2], -1)
}


test_WriteExplainedNodesToSifFileProducesTheCorrectAnnoFilesWithDefaultNames <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 2
    tempfile <- tempfile()
    outputDir <- paste(dirname(tempfile), as.integer(runif(1, 1, 1e9)), sep="-")
    
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir)
    
    # check number of files is correct
    checkEquals(length(list.files(outputDir, pattern = "*.sif")), 3)
    
    # Check data is correct
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-testNetwork1-testData1-delta2-Node0+_anno.txt"), stringsAsFactors = FALSE, header = TRUE)
    checkEquals(nrow(outputData), 5)
    checkEquals(names(outputData), c("NodeID", "Regulation"))
    checkEquals(outputData[1,1], "Node0")
    checkEquals(outputData[1,2], 1)
    checkEquals(outputData[2,1], "Node1")
    checkEquals(outputData[2,2], 1)
    checkEquals(outputData[3,1], "Node3")
    checkEquals(outputData[3,2], 1)
    checkEquals(outputData[4,1], "Node6")
    checkEquals(outputData[4,2], -1)
    checkEquals(outputData[5,1], "Node7")
    checkEquals(outputData[5,2], -1)
    
    outputData <- read.table(paste0(outputDir, "/incorExplainedNodes-testNetwork1-testData1-delta2-Node0+_anno.txt"), stringsAsFactors = FALSE, header = TRUE)
    checkEquals(nrow(outputData), 1)
    checkEquals(names(outputData), c("NodeID", "Regulation"))
    checkEquals(outputData[1,1], "Node2")
    checkEquals(outputData[1,2], 1)

    outputData <- read.table(paste0(outputDir, "/ambExplainedNodes-testNetwork1-testData1-delta2-Node0+_anno.txt"), stringsAsFactors = FALSE, header = TRUE)
    checkEquals(nrow(outputData), 1)
    checkEquals(names(outputData), c("NodeID", "Regulation"))
    checkEquals(outputData[1,1], "Node5")
    checkEquals(outputData[1,2], -1)
}


test_WriteExplainedNodesToSifFileProducesTheCorrectOutputFileWithHighDelta  <- function() {
hypothesisnode <- "Node0"
signOfHypothesis <- +1
networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
network <- CausalR::CreateCCG(networkFile)
experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
delta <- 10
tempfile <- tempfile()
outputDir <- dirname(tempfile)
outputFilesName <- basename(tempfile)

CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, outputFilesName, quiet=FALSE)

outputData <- read.table(paste0(outputDir, "/corExplainedNodes-", basename(tempfile), ".sif"), stringsAsFactors = FALSE)
checkEquals(nrow(outputData), 5)
checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
checkEquals(c(outputData[2,1], outputData[2,2], outputData[2,3]), c("Node0", "Inhibits", "Node2"))
checkEquals(c(outputData[3,1], outputData[3,2], outputData[3,3]), c("Node1", "Activates", "Node3"))
checkEquals(c(outputData[4,1], outputData[4,2], outputData[4,3]), c("Node2", "Activates", "Node6"))
checkEquals(c(outputData[5,1], outputData[5,2], outputData[5,3]), c("Node2", "Activates", "Node7"))
}


test_WriteExplainedNodesToSifFileProducesTheCorrectOutputFileWithDeltaOfOne  <- function() {
    hypothesisnode <- "Node0"
    signOfHypothesis <- +1
    networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
    network <- CausalR::CreateCCG(networkFile)
    experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
    experimentalData <- CausalR::ReadExperimentalData(experimentalDataFile, network)
    delta <- 1
    tempfile <- tempfile()
    outputDir <- dirname(tempfile)
    outputFilesName <- basename(tempfile)
    
    CausalR::WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData,  delta, outputDir, outputFilesName, quiet=FALSE)
    
    outputData <- read.table(paste0(outputDir, "/corExplainedNodes-", basename(tempfile), ".sif"), stringsAsFactors = FALSE)
    checkEquals(nrow(outputData), 1)
    checkEquals(c(outputData[1,1], outputData[1,2], outputData[1,3]), c("Node0", "Activates", "Node1"))
}

#' @title process experimental data
#' @description
#' Processes experimental data to get it into the correct form
#' for scoring. The node names that are read in as strings acquire 
#' an internal id when the network is created. This function will 
#' replace the node name with its id.

#' @param experimentalData input experimental data.
#' @param network an input interaction network.
#' @return processed experimental data formatted ready for scoring

ProcessExperimentalData <- function(experimentalData, network) {
    
    numNodesInExperimentalData <- nrow(experimentalData)
    
    # A matrix of zeros that will ultimately store the experimental data results as an array containing purely doubles
    processedResults <- matrix(0, numNodesInExperimentalData, 2)
    # Populate the second column (the expression values)
    processedResults[, 1] <- GetNodeID(network, experimentalData[, 1])
    processedResults[, 2] <- as.numeric(experimentalData[, 2])
    
    # If any of the node names aren't in the network, output a warning message and delete those rows from the table.
    if (any(is.na(processedResults[, 1]))) {
        for (i in 1:numNodesInExperimentalData) {
            if (is.na(processedResults[i, 1])) {
                warning("Node in experimental data not found in the network: ", experimentalData[i, 1])
            }
        }
        processedResults <- processedResults[!is.na(processedResults[, 1]), ]
    }
    
    return(processedResults)
} 

# Test suite for the ReadExperimentalData function

test_ValidateFormatOfDataTableReturnsFalseWhenTheInputFileOnlyHasOneColumn <- function() {
    matrix1 <- matrix(c("node0", "node1", "node2"), nrow = 3)
    dataTableWithOneColumn <- as.table(matrix1)
    checkTrue(!CausalR:::ValidateFormatOfDataTable(dataTableWithOneColumn))
}

test_ValidateFormatOfDataTableReturnsFalseWhenFormatOfTheSecondColumnIsNotCorrect <- function() {
    matrix1 <- matrix(c("node0", "node1", "node2", "+", "1", "0"), nrow = 3)
    dataTableWithIncorrectSecondColumn <- as.table(matrix1)
    checkTrue(!CausalR:::ValidateFormatOfDataTable(dataTableWithIncorrectSecondColumn))
}

test_ProcessExperimentalDataProducesTheCorrectOutputMatrixWithMatchedInputSize <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    network$isCCG <- FALSE
    experimentalResults <- matrix(c("node0", "node1", "node2", "+1", "-1", "1"), nrow = 3)
    processedData <- CausalR:::ProcessExperimentalData(experimentalResults, network)
    checkEquals(nrow(processedData), 3)
    checkEquals(ncol(processedData), 2)
    checkEquals(typeof(processedData), "double")
    checkEquals(processedData[1, 1], 1)
    checkEquals(processedData[2, 1], 2)
    checkEquals(processedData[3, 1], 3)
    checkEquals(processedData[1, 2], 1)
    checkEquals(processedData[2, 2], -1)
    checkEquals(processedData[3, 2], 1)
}

test_ProcessExperimentalDataProducesTheCorrectOutputMatrixWithSmallerExperimentalResults <- function() {
    network <- igraph::graph_from_literal("node0" - +"node1", "node0" - +"node2")
    network$isCCG <- FALSE
    experimentalResults <- matrix(c("node0", "node1", "+1", "-1"), nrow = 2)
    processedData <- CausalR:::ProcessExperimentalData(experimentalResults, network)
    checkEquals(nrow(processedData), 2)
    checkEquals(ncol(processedData), 2)
    checkEquals(typeof(processedData), "double")
    checkEquals(processedData[1, 1], 1)
    checkEquals(processedData[2, 1], 2)
    checkEquals(processedData[1, 2], 1)
    checkEquals(processedData[2, 2], -1)
    # Note that this test produces a warning due to the unequal sizes of the inputs
}

test_ProcessExperimentalDataProducesTheCorrectOutputMatrixForCCG <- function() {
    network <- igraph::graph_from_literal("node0+" - +"node1+", "node0+" - +"node2-", "node0-" - +"node1-", "node0-" - +"node2+")
    network$isCCG <- TRUE
    igraph::V(network)$unsignedName <- c("node0", "node1", "node2", "node0", "node1", "node2")
    experimentalResults <- matrix(c("node0", "node1", "+1", "-1"), nrow = 2)
    processedData <- CausalR:::ProcessExperimentalData(experimentalResults, network)
    checkEquals(nrow(processedData), 2)
    checkEquals(ncol(processedData), 2)
    checkEquals(typeof(processedData), "double")
    checkEquals(processedData[1, 1], 1)
    checkEquals(processedData[2, 1], 2)
    checkEquals(processedData[1, 2], 1)
    checkEquals(processedData[2, 2], -1)
}


test_ProcessExperimentalDataProducesTheCorrectOutputMatrixWithCCG <- function() {
    network <- igraph::graph_from_literal("node0+" - +"node1+", "node0+" - +"node2-", "node0-" - +"node1-", "node0-" - +"node2+")
    network$isCCG <- TRUE
    igraph::V(network)$unsignedName <- c("node0", "node1", "node2", "node0", "node1", "node2")
    experimentalResults <- matrix(c("node0", "node1", "node2", "+1", "-1", "1"), nrow = 3)
    processedData <- CausalR:::ProcessExperimentalData(experimentalResults, network)
    checkEquals(nrow(processedData), 3)
    checkEquals(ncol(processedData), 2)
    checkEquals(typeof(processedData), "double")
    checkEquals(processedData[1, 1], 1)
    checkEquals(processedData[2, 1], 2)
    checkEquals(processedData[3, 1], 3)
    checkEquals(processedData[1, 2], 1)
    checkEquals(processedData[2, 2], -1)
    checkEquals(processedData[3, 2], 1)
} 

#' @title run rank the hypothesis
#' @description 
#' A top level function that used to run CausalR
#' @param PPInet PPInet is the PPI interaction file
#' @param Expressiondata observed gene expression data
#' @param delta the number of links to follow from any hypothesis no. Dedepending on network size/topology, this value typically ranges between 1 and 5
#' @param correctPredictionsThreshold Minimal score for p-values calculation. Hypotheses with scores below this value will get NAs for p-value and enrichment p-value. The usual default is -inf within the RankTheHypotheses function, where it is employed.
#' @return rankedHypothesis table of results produced by the algorithm

runRankHypothesis <- function(PPInet, Expressiondata,delta, correctPredictionsThreshold){  

ccg = CreateCCG(PPInet)
experimentalData = ReadExperimentalData(Expressiondata, ccg)
rankedHypothesis=RankTheHypotheses(ccg, experimentalData, delta, correctPredictionsThreshold)

return(rankedHypothesis)

}

#' @title create a Computational Graph (CG)
#' @description
#' Creates a CG network from a .sif file.
#' Takes in a .sif file output from Cytoscape, and creates an 'igraph' representing the network.
#' The edges will be annotated with the type of interaction and a weight (1 for activation and -1 for inhibition)


#' @param sifFile the path of the .sif file that contains all 
#'       the information about the network Load in .sif file
#' @return a CG network
#' @export
#' @concept CausalR
#' @examples
#' # get path to example .sif file
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' #create cg
#' cg = CreateCG(network)

CreateCG <- function(sifFile) {
    
    tableOfInteractions <- ReadSifFileToTable(sifFile)
    print(paste("File read complete - read in", nrow(tableOfInteractions), "lines. Now constructing network", sep = " "))
    
    # Convert input data into a network
    network <- CreateNetworkFromTable(tableOfInteractions)
    print("Network has been created - now adding edge properties")
    
    # Add edge information
    interactionInformation <- GetInteractionInformation(tableOfInteractions)
    network <- AddWeightsToEdges(network, interactionInformation)
    print("Added weights to edges")
    
    network <- AddIDsToVertices(network)
    
    # give the network a name attribute, set to the filename minus the path and file extension
    network$name <- tools::file_path_sans_ext(basename(sifFile))
    
    network$isCCG <- FALSE
    
    return(network)
} 

#' @title validate format of table
#' @description 
#' Checks the format of the loaded in data. In particular exepects a table with threecolumns (in order) a initiating gene, an interaction ('Activates','Inhibits') and a responding gene and checks the number of rows and the values of the middle column.
#' @param dataTable the table to be tested
#' @return true if the test is satisfed.

ValidateFormatOfTable <- function(dataTable) {
    
    # The table should have three columns: the initiating gene, the type of interaction, and the responding gene
    if (!dim(dataTable)[2] == 3) {
        stop("The number of columns in the input file is incorrect")
    }
    
    # Test 2 The type of interaction column should either contain the entry 'Activates' or 'Inhibits'
    for (counter in 1:dim(dataTable)[1]) {
        if (!((dataTable[[2]][counter] == "Activates") || (dataTable[[2]][counter] == "Activation") || (dataTable[[2]][counter] == "Inhibits") || (dataTable[[2]][counter] == 
            "Inhibition"))) {
            stop("The middle column of the input file is incorrect. Each entry should be 'Activates/Activation' or 'Inhibits/Inhibition'")
        }
    }
} 

#' @title get regulated nodes
#' @description 
#' This function will compute the nodes regulated by the given hypothesis gene and write the results to a file
#' @param PPInet a protein-protein interaction network
#' @param Expressiondata a table of observed gene expression data
#' @param delta the number of edges to follow along the network. This should typically be between 1 and 5 dependent on network size/topology
#' @param hypothesisGene the name of the hypothesis gene
#' @param signOfHypothesis the sign of action expected from the hypothesis, +1 for up regulation, -1 for down 
#' @param outputfile the file to which the results should be written
#' @return Nodes regulated by hypothesis gene


GetRegulatedNodes <- function(PPInet, Expressiondata, delta, hypothesisGene = "", signOfHypothesis = 1, outputfile = "") {
    
    signOfHypothesis <- as.integer(signOfHypothesis)
    delta <- as.integer(delta)
    
    ccg = CreateCCG(PPInet)
    experimentalData = ReadExperimentalData(Expressiondata, ccg)
    # rankedHypothesis=RankTheHypotheses(ccg, experimentalData, delta)
    ndelta <- delta + 1
    predictions <- MakePredictionsFromCCG(hypothesisGene, signOfHypothesis, ccg, ndelta)
    x <- GetNodeName(ccg, CompareHypothesis(predictions, experimentalData))
    utils::write.table(x, file = outputfile, sep = "\t", col.names = TRUE, quote = FALSE)
    
} 

#' @title returns the score for a given number of correct and incorrect predictions
#' @description
#' Returns the score based on the values of n++, n+-, n-+ and n--
#' @param matrixRow a row af a matrix of correct and incorrect prediction scores
#' @return the corresponding score for the given row


GetScoreForNumbersOfCorrectandIncorrectPredictions <- function(matrixRow) {
    
    numCorrectPositivePredictions <- matrixRow[1]
    numIncorrectPositivePredictions <- matrixRow[2]
    numIncorrectNegativePredictions <- matrixRow[3]
    numCorrectNegativePredictions <- matrixRow[4]
    
    score <- (numCorrectPositivePredictions + numCorrectNegativePredictions) - (numIncorrectPositivePredictions + numIncorrectNegativePredictions)
    
    return(score)
} 

#' @title get set of differientially expressed genes
#' @description
#' Gets the set of differentially expressed genes in the results, G+ as defined by in Causal reasoning on biological networks: Interpreting transcriptional changes,  L Chindelevitch et al.
#' @param results a table of results
#' @return a matrix of differentially expressed genes

#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012.

GetSetOfDifferentiallyExpressedGenes <- function(results) {
    
    counter <- 1
    numResults <- nrow(results)
    differentiallyExpressedGenes <- matrix(0, numResults, 1)
    for (i in 1:numResults) {
        if ((as.numeric(results[i, 2]) != 0)) {
            differentiallyExpressedGenes[counter, 1] <- results[i, 1]
            counter <- counter + 1
        }
    }
    differentiallyExpressedGenes <- differentiallyExpressedGenes[1:counter - 1]
    
    return(differentiallyExpressedGenes)
} 

# A test suite for EnrichmentPValueCalculation

test_CalculateEnrichmentPValueProducesTheCorrectValueWhenResultsAndPredictionsContainTheSameIDs <- function() {
    predictions <- matrix(c(1, 2, 3, 4, 5, 1, 0, 0, 1, 0), ncol = 2)
    results <- matrix(c(1, 2, 3, 4, 5, 1, 0, 0, -1, 1), ncol = 2)
    checkEquals(CausalR:::CalculateEnrichmentPValue(predictions, results), 0.3)
}

test_CalculateEnrichmentPValueProducesTheCorrectValueWhenPredictionsAndResultsIDsAreMisMatched <- function() {
    predictions <- matrix(c(1, 2, 4, 5, 1, -1, -1, 1), ncol = 2)
    results <- matrix(c(1, 2, 3, 4, 1, -1, 0, 0), ncol = 2)
    checkEquals(CausalR:::CalculateEnrichmentPValue(predictions, results), 0.5)
}

test_GetSetOfDifferentiallyExpressedGenesIdentifiesTheCorrectGenes <- function() {
    results <- matrix(c(1, 2, 3, 4, 1, 1, 0, -1), ncol = 2)
    differentiallyExpressedGenes <- CausalR:::GetSetOfDifferentiallyExpressedGenes(results)
    checkEquals(length(differentiallyExpressedGenes), 3)
    checkEquals(differentiallyExpressedGenes[1], 1)
    checkEquals(differentiallyExpressedGenes[2], 2)
    checkEquals(differentiallyExpressedGenes[3], 4)
}

test_GetSetOfSignificantPredictionsIdentifiesTheCorrectValues <- function() {
    predictions <- matrix(c(1, 2, 3, 4, 1, 0, -1, -1), ncol = 2)
    significantPredictions <- CausalR:::GetSetOfSignificantPredictions(predictions)
    checkEquals(length(significantPredictions), 3)
    checkEquals(significantPredictions[1], 1)
    checkEquals(significantPredictions[2], 3)
    checkEquals(significantPredictions[3], 4)
} 

#' @title add IDs to vertices

#' @description 
#' Adds the IDs as a vertex property to 
#' the vertices in the network. Used when creating
#' sub-networks where the new nodes will retain the IDs from their original
#' network

#' @param network the network to which the IDs are to be added
#' @return  network with IDs added

AddIDsToVertices <- function(network) {
    
    
    numNodes <- igraph::gorder(network)
    network <- igraph::set_vertex_attr(network, "ID", 1:numNodes, 1:numNodes)
} 

#' @title   get scores weights matrix by the cubic algorithm
#' @description
#' Implements the cubic algorithm as described on pages 6 and 7 of Assessing statistical significance in causal graphs,
#' Chindelevitch et al. 2012
#' @param predictionListStats a vector containing the values q+, q- and q0 
#' @param experimentalDataStats a vector containing the values n+, n- and n0
#' @param epsilon the algorithms tolerance epsilon
#' @return a matrix containing the ternary dot product distribution


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.


GetScoresWeightsMatrixByCubicAlg <- function(predictionListStats, experimentalDataStats, epsilon) {
    
    
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    
    DMax <- FindMaximumDValue(predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats)
    
    # Not all values of r_, r- c+, c- are possible for the contigency table. This step finds all that combinations that are valid. Each row of this
    # matrix will represent a family of contingency tables.
    matrixOfAllPossibleRowAndColumnSumValues <- GetRowAndColumnSumValues(predictionListStats, experimentalDataStats)
    
    # Ensure matrixOfAllPossibleRowAndColumnSumValues has 4 columns
    matrixOfAllPossibleRowAndColumnSumValues <- matrix(matrixOfAllPossibleRowAndColumnSumValues, ncol = 4)
    numOfCombinations <- nrow(matrixOfAllPossibleRowAndColumnSumValues)
    
    # For each combination of row and column sum values, the value of n++ is bounded by zero and q+ (predictionListStats[1]).  Hence the maximum number
    # of entries in distributionMatrix will be:
    maxNumberOfDistributionValues <- numOfCombinations * (predictionListStats[1] + 1)
    distributionMatrix <- matrix(0, maxNumberOfDistributionValues, 2)
    
    # A counter that will be used to keep track how many entries have been put into distributionMatrix
    i <- 1
    
    for (counter in 1:numOfCombinations) {
        rowAndColumnSumValues <- matrixOfAllPossibleRowAndColumnSumValues[counter, ]
        # To reduce the overall runtime of the algorithm, the approximate maxDValue for the family is calculated, rather than the actual.
        approxDFamilyMax <- GetMaxDValueForAFamily(rowAndColumnSumValues, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats)
        if (approxDFamilyMax > epsilon * DMax) {
            # Calculate the value of n++ that is used to calculate approxDFamilyMax r+
            r_p <- rowAndColumnSumValues[1]
            # c+
            c_p <- rowAndColumnSumValues[3]
            # t
            total <- sum(rowAndColumnSumValues[1:2])
            if (total > 0) {
                n_pp <- round(r_p * c_p/total)
            } else {
                n_pp <- 0
            }
            
            # Iterate over possible values of n++, starting from the one calculated above.  Since the values decrease monotonically either side of this value,
            # we start at this value and iterate in both directions until the D value is less than a given threshold. This will give us the non-neglible parts
            # of the distribution.
            
            # An upper bound for possible values of n++ is the minimum of r+ and c+
            for (u in n_pp:min(r_p, c_p)) {
                # The output to this function is a 1x4 vector containing the values of n++, n+-, n-+, n--
                contingencyTableValues <- PopulateTwoByTwoContingencyTable(rowAndColumnSumValues, u)
                # None of these values can be less than zero
                if (min(contingencyTableValues) >= 0) {
                    DValue <- GetWeightForNumbersOfCorrectandIncorrectPredictions(contingencyTableValues, predictionListStats, experimentalDataStats, 
                                                                                  logOfFactorialOfPredictionListStats)
                    if (DValue > epsilon * DMax) {
                        # Only need to compute the score if the D-value is non-neglible
                        score <- GetScoreForNumbersOfCorrectandIncorrectPredictions(contingencyTableValues)
                        distributionMatrix[i, ] <- c(score, DValue)
                        i <- i + 1
                    } else {
                        break
                    }
                }
            }
            
            # The loop below does exactly the same thing as the one above; it covers the cases that are not done in the loop above. A lower bound for possible
            # values of n++ is 0 To avoid n_pp become negative or double counting when n_pp = 0, we add the following if statement
            if (n_pp > 0) {
                
                for (u in (n_pp - 1):0) {
                    contingencyTableValues <- PopulateTwoByTwoContingencyTable(rowAndColumnSumValues, u)
                    if (min(contingencyTableValues) >= 0) {
                        DValue <- GetWeightForNumbersOfCorrectandIncorrectPredictions(contingencyTableValues, predictionListStats, experimentalDataStats, 
                                                                                      logOfFactorialOfPredictionListStats)
                        if (DValue > epsilon * DMax) {
                            score <- GetScoreForNumbersOfCorrectandIncorrectPredictions(contingencyTableValues)
                            distributionMatrix[i, ] <- c(score, DValue)
                            i <- i + 1
                        } else {
                            break
                        }
                    }
                }
            }
        }
    }
    
    # Trim this matrix so that rows are not filled in during the algorithm are removed, and ensure it has two columns
    distributionMatrix <- matrix(distributionMatrix[1:i - 1, ], ncol = 2)
    
    return(distributionMatrix)
} 

#' @title updates weights for contingency table and produce values for p-value calculation
#' @description
#' Finds the D-Values (weights) from any 3x2 contingency tables 
#' that have a score above and including the hypothesis score. It also calculates the
#' total weight, and returns a 2x1 vector of the two values. The ratio of these values is the p-value.

#' @param weights Weights
#' @param r_p the row sum r+
#' @param r_m the row sum r-
#' @param r_z the row sum r0
#' @param n_p the column sum n+
#' @param n_m the column sum n-
#' @param predictionListStats a list of prediction statistics
#' @param experimentalDataStats the observed experimental data
#' @param logOfFactorialOfPredictionListStats log factorial's of prediction list stats
#' @param hypothesisScore the hypothesis score to be considered
#' @param logepsDMax log of epsilon logD Maximum value
#' @param logDMax a logD Maximum value
#' @return a vector containing the hypothesis score and the total weight

GetWeightsAboveHypothesisScoreForAThreeByTwoTable <- function(weights, r_p, r_m, r_z, n_p, n_m, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
    hypothesisScore, logepsDMax, logDMax) {
    
    # To reduce the overall runtime of the algorithm, the approximate maxDValue for the family is calculated, rather than the actual.
    log_approxDFamilyMax <- GetMaxDValueForAThreeByTwoFamily(r_p, r_m, r_z, n_p, n_m, predictionListStats, logOfFactorialOfPredictionListStats, TRUE)
    if (log_approxDFamilyMax > logepsDMax) {
        # Calculate the values of n++ and n-+ that were used to calculate approxDFamilyMax
        
        # t = n_p + n_m ( the sum of all the values in the 3x2 submatrix)
        total <- n_p + n_m
        
        if (total > 0) {
            n_pp <- ceiling(r_p * n_p/total)
        } else {
            n_pp <- 0
        }
        
        # Iterate over possible values of n++, starting from the one calculated above.  Since the values decrease monotonically either side of this value,
        # we start at this value and iterate in both directions until the D value is less than a given threshold. This will give us the non-neglible parts
        # of the distribution.
        
        # An upper bound for possible values of n++ is the minimum of r+ and n+
        for (m_pp in n_pp:min(r_p, n_p)) {
            
            # We still need to define one more value in the 3x2 matrix to determine all the rest n+- is bounded by the minumum of r- and n+
            for (m_mp in 0:min(r_m, n_p)) {
                
                # n+-
                m_pm <- r_p - m_pp
                # n--
                m_mm <- r_m - m_mp
                # n0+
                m_zp <- n_p - (m_pp + m_mp)
                # n0-
                m_zm <- n_m - (m_pm + m_mm)
                contingencyTableValues <- c(m_pp, m_pm, m_mp, m_mm, m_zp, m_zm)
                
                m_pz <- predictionListStats[1] - r_p
                m_mz <- predictionListStats[2] - r_m
                m_zz <- predictionListStats[3] - r_z
                
                # None of these values can be less than zero
                if (min(contingencyTableValues) >= 0) {
                    
                    threeByThreeContingencyTable <- c(m_pp, m_pm, m_pz, m_mp, m_mm, m_mz, m_zp, m_zm, m_zz)
                    logDValue <- sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable))
                    # Only need to compute the score if the D-value is non-neglible
                    if (logDValue > logepsDMax) {
                        weight_contrib <- exp(logDValue - logDMax)
                        weights[2] <- weights[2] + weight_contrib
                        
                        # Rather than just calculate the DValue by taking the exponential of logDvalue, we first subtract a constant.  This means when we take the
                        # exponential we get the DValue divided by a different constant, but this cancels out when we take ratios later when computing the p-value
                        if (m_pp + m_mm - (m_pm + m_mp) >= hypothesisScore) {
                            weights[1] <- weights[1] + weight_contrib
                        }
                    }
                }
            }
        }
        
        # The loop below does exactly the same thing as the one above; it covers the cases that are not done in the loop above. A lower bound for possible
        # values of n++ is 0 To avoid n_pp become negative or double counting when n_pp = 0, we add the following if statement
        if (n_pp > 0) {
            for (m_pp in (n_pp - 1):0) {
                
                # Iterate over all values of m_mp
                for (m_mp in 0:min(r_m, n_p)) {
                    
                    # n+-
                    m_pm <- r_p - m_pp
                    # n--
                    m_mm <- r_m - m_mp
                    # n0+
                    m_zp <- n_p - (m_pp + m_mp)
                    # n0-
                    m_zm <- n_m - (m_pm + m_mm)
                    contingencyTableValues <- c(m_pp, m_pm, m_mp, m_mm, m_zp, m_zm)
                    
                    m_pz <- predictionListStats[1] - r_p
                    m_mz <- predictionListStats[2] - r_m
                    m_zz <- predictionListStats[3] - r_z
                    
                    # None of these values can be less than zero
                    if (min(contingencyTableValues) >= 0) {
                        
                        threeByThreeContingencyTable <- c(m_pp, m_pm, m_pz, m_mp, m_mm, m_mz, m_zp, m_zm, m_zz)
                        logDValue <- sum(logOfFactorialOfPredictionListStats) - sum(lfactorial(threeByThreeContingencyTable))
                        
                        # Only need to compute the score if the D-value is non-neglible
                        if (logDValue > logepsDMax) {
                            # Rather than just calculate the DValue by taking the exponential of logDvalue, we first subtract a constant.  This means when we take the
                            # exponential we get the DValue divided by a different constant, but this cancels out when we take ratios later when computing the p-value
                            weight_contrib <- exp(logDValue - logDMax)
                            weights[2] <- weights[2] + weight_contrib
                            if (m_pp + m_mm - (m_pm + m_mp) >= hypothesisScore) {
                                weights[1] <- weights[1] + weight_contrib
                            }
                        }
                    }
                }
            }
        }
    }
    return(weights)
} 

#' calculates an enrichment p-value
#' @description
#' Calculate a enrichment p-value for a given hypothesis by comparing the corresponding predicted and observed gene changes

#' @export
#' @concept CausalR
#' @param  predictions  predictions of changes from the CCG for a particular hypothesis
#' @param  results  gene changes observed in the experimental data
#' @return  an enrichment p-value
#' @examples
#' predictions <- matrix(c(1,2,3,1,1,-1), ncol = 2)
#' results<- matrix(c(1,2,3,4,1,1,-1,1), ncol = 2)
#' CalculateEnrichmentPValue(predictions, results)

CalculateEnrichmentPValue <- function(predictions, results) {
    
    setOfSignificantPredictions <- GetSetOfSignificantPredictions(predictions)
    setOfDifferentiallyExpressedGenes <- GetSetOfDifferentiallyExpressedGenes(results)
    setOfNonDifferentiallyExpressedGenes <- setdiff(results[, 1], setOfDifferentiallyExpressedGenes)
    
    # n_pp + n_pm + n_mp + n_mm
    numSignificantPredictionsThatAreResponsive <- length(intersect(setOfSignificantPredictions, setOfDifferentiallyExpressedGenes))
    # n+0 + n-0
    numSignificantPredictionsThatAreUnresponsive <- length(intersect(setOfSignificantPredictions, setOfNonDifferentiallyExpressedGenes))
    # n0+ + n0-
    numZeroPredictionsThatAreResponsive <- length(setOfDifferentiallyExpressedGenes) - numSignificantPredictionsThatAreResponsive
    # n00
    numZeroPredictionsThatAreUnreponsive <- length(setOfNonDifferentiallyExpressedGenes) - numSignificantPredictionsThatAreUnresponsive
    
    contingencyTable <- matrix(c(numSignificantPredictionsThatAreResponsive, numSignificantPredictionsThatAreUnresponsive, numZeroPredictionsThatAreResponsive, 
        numZeroPredictionsThatAreUnreponsive), nrow = 2)
    enrichmentPValue <- stats::fisher.test(contingencyTable, alternative = "greater")[[1]]
    
    return(enrichmentPValue)
} 

#' @title check row and column sum values are valid
#' @description
#' Checkes to see if the values of r+, r-, c+ and c- which are stored in rowAndColumnSumValues define a valid contingency
#' table
#' @param rowAndColumnSumValues  a 4x1 vector containing the row and column sum values (r+, r-, c+, c-) for a 2x2 contingency table
#' @param predictionListStats a vector containing the values q+, q- and q0 
#' @param experimentalResultStats A vector containing the values n+, n- and n0
#' @return TRUE if the table is valid; otherwise FALSE


# Check if the values of r+, r-, c+ and c- which are stored in rowAndColumnSumValues make up a valid contingency table in relation to the whole 3x3
# table

# Inputs: rowAndColumnSumValues A 4x1 vector containing the row and column sum values (r+, r-, c+, c-) for a 2x2 contingency table
# predictionListStats A vector containing the values q+, q- and q0 (the number of positive/negative/non-significant (or contradictory) predictions)
# experimentalResultStats A vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or contradictory)
# transcripts in the results)



CheckRowAndColumnSumValuesAreValid <- function(rowAndColumnSumValues, predictionListStats, experimentalResultStats) {
    
    # r+ and r-
    r_p <- rowAndColumnSumValues[1]
    r_m <- rowAndColumnSumValues[2]
    # c+ and c-
    c_p <- rowAndColumnSumValues[3]
    c_m <- rowAndColumnSumValues[4]
    
    if (c_m < 0) {
        return(FALSE)
    }
    
    # q+, q- and q0
    q_p <- predictionListStats[1]
    q_m <- predictionListStats[2]
    q_z <- predictionListStats[3]
    
    # n+, n- and n0
    n_p <- experimentalResultStats[1]
    n_m <- experimentalResultStats[2]
    
    # Compute the other values in the 3x3 table
    
    n_pz <- q_p - r_p
    if (n_pz < 0) {
        return(FALSE)
    }
    
    n_mz <- q_m - r_m
    if (n_mz < 0) {
        return(FALSE)
    }
    
    n_zp <- n_p - c_p
    if (n_zp < 0) {
        return(FALSE)
    }
    
    n_zm <- n_m - c_m
    if (n_zm < 0) {
        return(FALSE)
    }
    
    n_zz <- q_z - (n_zp + n_zm)
    
    if (n_zz < 0) {
        return(FALSE)
    }
    
    return(TRUE)
} 

#' @title get node name
#' @description
#' Returns the node name from one or more node IDs, or substitute node names for node IDs,
#' given in first column of a matrix typically of  predictions or experimental data
#' @param network Built from igraph
#' @param nodeID a node ID or a matrix containing node IDs in its first column
#' @param signed whether or not the node name should be signed. Setting this value to TRUE gives a signed name indicating whether the gene is up or down regulated in the network
#' @return a node name or a vector of node names depending if the input is an matrix.
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg = CreateCCG(network)
#' nodeID <- 10
#' GetNodeName(ccg, nodeID)


GetNodeName <- function(network, nodeID, signed = FALSE) {
    
    if (is.matrix(nodeID) || is.data.frame(nodeID)) {
        if (network$isCCG) {
            name <- nodeID
            if (signed) {
                name[, 1] <- igraph::V(network)$name[nodeID[, 1]]
            } else {
                name[, 1] <- igraph::V(network)$unsignedName[nodeID[, 1]]
            }
            
        } else {
            name <- nodeID
            name[, 1] <- igraph::V(network)$name[nodeID[, 1]]
        }
    } else {
        if (network$isCCG) {
            if (signed) {
                name <- igraph::V(network)$name[nodeID]
            } else {
                name <- igraph::V(network)$unsignedName[nodeID]
            }
        } else {
            name <- igraph::V(network)$name[nodeID]
        }
    }
    
    return(name)
} 

#' @title 
#' analyse predictions list
#' @description
#' Taking the list of predictions from a particular hypothesis, counts the number of
#' positive and negative predictions in the list and the number of 0's (from numPredictions).


#' @param predictionsList list of predictions
#' @param numPredictions  number of predictions
#' @return prediction statistics
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' ccg <- CreateCCG(network)
#' predictions <- MakePredictions('NodeA', +1, ccg, 2)
#' AnalysePredictionsList(predictions,8)

AnalysePredictionsList <- function(predictionsList, numPredictions) {
    
    predictionsListStats <- GetNumberOfPositiveAndNegativeEntries(predictionsList)
    predictionsListStats[3] <- numPredictions - (predictionsListStats[1] + predictionsListStats[2])
    
    return(predictionsListStats)
} 

#' @title get set of significant predictions
#' @description 
#' Gets the set of positive and negative predictions, the combination of the sets Sh+ and Sh- in Causal reasoning on biological networks: Interpreting transcriptional changes,  L Chindelevitch et al.
#' @param predictions a table of predictions
#' @return a matrix of positive and negative predictions

#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012.

GetSetOfSignificantPredictions <- function(predictions) {
    
    counter <- 1
    numPredictions <- nrow(predictions)
    significantPredictions <- matrix(0, numPredictions, 1)
    for (i in 1:numPredictions) {
        if ((as.numeric(predictions[i, 2]) != 0)) {
            significantPredictions[counter, 1] <- predictions[i, 1]
            counter <- counter + 1
        }
    }
    significantPredictions <- significantPredictions[1:counter - 1]
    
    return(significantPredictions)
} 

#' @title returns interaction information from input data
#' @description
#' Gets the interaction information from the input data
#' @param dataTable a data table containing the information read in from the .sif file representing the network.
#' @return a vector of interaction information

GetInteractionInformation <- function(dataTable) {
    
    informationMatrix <- as.matrix(dataTable)
    interactionInfo <- informationMatrix[, 2]
    return(interactionInfo)
} 

#' @title order hypotheses
#' @description
#' Ranks the hypotheses.  Takes a matrix containing the scores for each node of the network, and ranks them placing the hypothesis with the 
#' most correct predictions is at the top

#' @param scoresMatrix a matrix containing the scores for each node of the network
#' @return a ranked table of hypotheses

OrderHypotheses <- function(scoresMatrix) {
    
    tableOfScores <- as.table(scoresMatrix)
    # Order in descending order
    orderedTable <- tableOfScores[order(-1 * (as.numeric(tableOfScores[, 3]))), ]
    
    return(orderedTable)
} 

#' @title 
#' analyse experimental data
#' @description
#' Returns the number of up- and down-regulated genes in the experimental data

#' @param  experimentalData  a dataframe containing a list of genes with corresponding direction of change (1 or -1)
#' @return up and down regulation statistics for the experimental data


AnalyseExperimentalData <- function(experimentalData) {
    
    
    experimentalDataStats <- GetNumberOfPositiveAndNegativeEntries(experimentalData)
    experimentalDataStats[3] <- nrow(experimentalData) - sum(experimentalDataStats[1:2])
    
    return(experimentalDataStats)
} 

#' @title calculate total weight for all contingency tables
#' @description
#' Calculates the total weights or D-values for all possible  contingency tables. This value can be  used to calculate the p-value

#' @param experimentalDataStats a vector containing the values n+, n- and n0, the number of positive/negative/non-significant (or contradictory) transcripts in the results
#' @param returnlog whether the result should be returned as a log. Default is FALSE.

#' @return a D-value or weight

CalculateTotalWeightForAllContingencyTables <- function(experimentalDataStats, returnlog = FALSE) {
    
    # Total weight is (total number of measured transcripts)!/((number of positive results)!*(number of negative results)!*(number of non-sig.
    # results)!)
    logOfTotalWeight <- lfactorial(sum(experimentalDataStats)) - (lfactorial(experimentalDataStats[1]) + lfactorial(experimentalDataStats[2]) + lfactorial(experimentalDataStats[3]))
    
    if (returnlog) {
        return(logOfTotalWeight)
    } else {
        return(exp(logOfTotalWeight))
    }
} 

#' @title get maximun D value for a family
#' @description
#' Computes the maximum D value for a particular family - denoted as D_fam on page 6 of Assessing Statistical Signifcance of Causal Graphs
#' @param r_p row sum r+
#' @param r_m row sum r-
#' @param c_p column sum c+
#' @param predictionListStats approximate values of n++, n+-, n-+ and n--
#' @param experimentalDataStats a vector containing the values q+, q- and q0:  number of positive, negative, non-significant/contradictory predictions
#' @param logOfFactorialOfPredictionListStats a vector containing the values n+, n- and n0:  number of positive, negative, non-significant/contradictory observations
#' @param returnlog return result as log, default value is FALSE
#' @return the maximum DFam Value


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.



GetMaxDValueForAFamily <- function(r_p, r_m, c_p, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, returnlog = FALSE) {
    # t
    total <- r_p + r_m
    
    # Compute the values of n++, n+-, n-+ and n-- that maximise the D-value; these correspond to the following formula: n_ab is approximately equal to
    # q_a*n_b/T, where T = q+ + q- + q0 = n+ + n- + n0, and a,b are either + or -.  See Assessing statistical significance in causal graphs, page 6.  We
    # need integer values and we need to respect the row and column totals; achieve this by rounding n_pp up (arbitrarily) and calculate the other
    # values to get the right totals.
    if (total > 0) {
        n_pp <- ceiling(r_p * c_p/total)
        n_pm <- r_p - n_pp
        n_mp <- c_p - n_pp
        n_mm <- r_m - n_mp
        maximumDFamValue <- GetApproximateMaximumDValueFromTwoByTwoContingencyTable(n_pp, n_pm, n_mp, n_mm, predictionListStats, experimentalDataStats, 
            logOfFactorialOfPredictionListStats, returnlog)
    } else {
        maximumDFamValue <- GetApproximateMaximumDValueFromTwoByTwoContingencyTable(0, 0, 0, 0, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
            returnlog)
    }
    
    return(maximumDFamValue)
} 

#' @title find approximate values that will maximise D value
#' @description 
#' Finds an approximate table values to maximise D.
#' Given the values of q+, q-, q0, n+, n- and n0 this function will produce the approximate values
#' of n++, n+-, n-+ and n-- that will maximise the D value. See Assessing statistical significance of 
#' casual graphs, page 6. The values are approximate since they need to be rounded, although the direction
#' of rounding is not clear at this stage.
#' 

#' @param predictionListStats a vector containing the values q+, q- and q0: 
#' numbers of positive, negative and non-significant/contradictory predictions
#' @param experimentalDataStats a vector containing the values n+, n- and n0:
#' numbers of positive, negative and non-significant/contradictory predictions
#' @return a 2x2 contingency table which approximately maximises D


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.

FindApproximateValuesThatWillMaximiseDValue <- function(predictionListStats, experimentalDataStats) {
    
    
    # Inputs: predictionListStats A vector containing the values q+, q- and q0 (the number of positive/negative/non-significant (or contradictory)
    # predictions) experimentalResultStats A vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or
    # contradictory) transcripts in the results)
    
    q_p <- predictionListStats[1]
    q_m <- predictionListStats[2]
    
    n_p <- experimentalDataStats[1]
    n_m <- experimentalDataStats[2]
    
    Tval <- sum(predictionListStats)
    # The values of n++, n+-, n-+ and n-- that give the maximum D-value are given by the formula within the paper - Assessing statistical significance
    # in causal graphs, page 6.  The formula is n_ab is approximately equal to q_a*n_b/T, where T = q+ + q- + q0 = n+ + n- + n0, and a,b are either + or
    # -.
    n_pp <- q_p * n_p/Tval
    n_pm <- q_p * n_m/Tval
    n_mp <- q_m * n_p/Tval
    n_mm <- q_m * n_m/Tval
    
    twoByTwoContingencyTable <- c(n_pp, n_pm, n_mp, n_mm)
    
    return(twoByTwoContingencyTable)
} 

#' @title Write explained nodes to Sif file
#' @description
#' Outputs networks of explained nodes in .sif file format for producing visualisations using Cytoscape. Output will be to a
#' directory beginning with a timestamp, 

#' @param hypothesisnode a hypothesis node
#' @param signOfHypothesis the direction of change of hypothesis node
#' @param network a computational causal graph
#' @param experimentalData The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.
#' @param delta the number of edges across which the hypothesis should be followed
#' @param outputDir the directory to output the files to. Default is the working directory
#' @param outputFilesName a character string to use for the name of the output files. Default value is "", which results in files using the default naming convention of "network file name-data file name-delta value-node name". Set to NA if not writing to file.
#' @param correctlyExplainedOnly if TRUE network files will only be produced for correctly explained nodes. If FALSE network files will be produced for each of correctly explained, incorrectly explained and ambiguously explained nodes.
#' @param quiet a flag to suppress output to console. FALSE by default.
#' @return files containing paths from hypothesis node to explained nodes in sif format and corresponding annotation (_anno.txt) files
#' @export
#' @concept CausalR
#' @examples
#' hypothesisnode <- "Node0"
#' signOfHypothesis <- +1
#' networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
#' network <- CreateCCG(networkFile)
#' experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
#' experimentalData <- ReadExperimentalData(experimentalDataFile, network)
#' delta <- 2
#' WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData, delta,
#'                              outputFilesName=NA)


WriteExplainedNodesToSifFile <- function(hypothesisnode, signOfHypothesis, network, experimentalData, delta, outputDir = getwd(), outputFilesName = "", correctlyExplainedOnly = FALSE, quiet = FALSE) {
    
    signOfHypothesis <- as.integer(signOfHypothesis)
    delta <- as.integer(delta)
    
    if(correctlyExplainedOnly) {
        nodeSetsToProcess <- c("corExplainedNodes")
    } else {
        nodeSetsToProcess <- c("corExplainedNodes", "incorExplainedNodes", "ambExplainedNodes")
    }
    
    # If outputDir doesn't exist, create it
    if (!file.exists(file.path(outputDir))){
        dir.create(file.path(outputDir))
    }
    
    for (thisNodeSet in nodeSetsToProcess) {
        
        ## Initialise empty lists. Each will have one element for each delta
        ## In explainedNodes each element will be a table of explained nodes (I.e. nodes that appear in both network and experimentalData with the same sign).
        ## The table will contain the node name and the value (1 or -1)
        ## In uniqueExplainedNodes each element will be a table that is a subset of that in explainedNodes, where entries that appear in the previous
        ## (lower delta) tables have been removed
        ## In pathsToUniqueExplainedNodes each element will be a table that contains the same nodes as described in uniqueExplainedNodes but each row will contain
        ## the path to that node from hypothesis node.
        explainedNodes <- list()
        uniqueExplainedNodes <- list()
        pathsToUniqueExplainedNodes <- list()
        
        for (thisDelta in 1:delta) {
            
            ## Get a table of explained nodes and their values within thisDelta 
            explainedNodes[[thisDelta]] <- GetExplainedNodesOfCCG(hypothesisnode, signOfHypothesis, network, experimentalData, thisDelta)[[thisNodeSet]]
            
            ## Get explained nodes from this delta only (not lower delta). I.e. remove nodes that are listed in previous delta
            if (thisDelta == 1) {
                uniqueExplainedNodes[[thisDelta]] <- explainedNodes[[thisDelta]]
            } else {
                uniqueExplainedNodes[[thisDelta]] <- explainedNodes[[thisDelta]][!(explainedNodes[[thisDelta]][,1] %in% explainedNodes[[thisDelta-1]][,1]),]
            }
            
            ## For each unique explained node for thisDelta, excluding the hypothesis node itself, get the shortest paths to it from the hypothesisnode
            if(nrow(uniqueExplainedNodes[[thisDelta]]) > 0) {
                targetNodes <- uniqueExplainedNodes[[thisDelta]]$name[uniqueExplainedNodes[[thisDelta]]$name != hypothesisnode]
                if (length(targetNodes) > 0) {
                pathsToUniqueExplainedNodes[[thisDelta]] <- do.call("rbind",
                    unlist(lapply(targetNodes, function(targetNode) GetShortestPathsFromCCG(network, hypothesisnode, targetNode, quiet = TRUE)), recursive = FALSE)
                )
                } else {
                    pathsToUniqueExplainedNodes[[thisDelta]] <- NULL
                }
            }
        }
        
        ## Convert the paths into sif format 
        if (length(pathsToUniqueExplainedNodes) > 0) {
            output <- do.call("rbind", lapply(pathsToUniqueExplainedNodes, GetPathsInSifFormat))
            output <- sort(unique(output))
        } else {
            output <- NULL
        }
        
        ## Combine list elements
        output2 <- do.call("rbind", uniqueExplainedNodes)
        output2 <- unique(output2)
        output2 <- output2[ order(-output2[,2], output2[,1]), c("name", "exp")]
        colnames(output2) <- c("NodeID", "Regulation")
        
        if (signOfHypothesis < 0) {
            node <- paste0(hypothesisnode, "-")
        } else {
            node <- paste0(hypothesisnode, "+")
        }
        
        if (!is.na(outputFilesName)) {
            if(outputFilesName == "") {
                outputFilesName <- paste0(network$name, "-", colnames(experimentalData)[1], "-delta", delta, "-", node)
            }
            if(!quiet) {
                cat("Writing sif file to: ", paste0(outputDir, "/", thisNodeSet, "-", outputFilesName, ".sif"), "\n" , sep="")
                cat("Writing annotation file to: ", paste0(outputDir, "/", thisNodeSet, "-", outputFilesName, "_anno.txt"), "\n", sep="")
            }
            write(output, paste0(outputDir, "/", thisNodeSet, "-", outputFilesName, ".sif"), sep = "\n")
            utils::write.table(output2, paste0(outputDir, "/", thisNodeSet, "-", outputFilesName, "_anno.txt"), row.names = FALSE, quote = FALSE, sep = "\t")
        }
        
        if (!quiet) {
            cat(thisNodeSet , "\n")
            cat(output, sep = "\n")
        }
    }
}

#' @title score hypothesis
#' @description 
#' Score a single hypothesis, using the predictions from the network and the experimental data returning a vector of scoring statistics
#' @param matrixOfPredictions a matrix of predictions
#' @param matrixOfExperimentalData a matrix of experimentaldata
#' @return scoreBreakdown a vector giving, in order, the overall score, and the numbers of correct, incorrect and ambigous predictions
#' @export
#' @concept CausalR
#' @examples
#' predictions <- matrix(c(1,2,3,+1,0,-1),ncol=2)
#' experimentalData <- matrix(c(1,2,4,+1,+1,-1),ncol=2)
#' ScoreHypothesis(predictions,experimentalData)
#' CompareHypothesis(predictions,experimentalData)

ScoreHypothesis <- function(matrixOfPredictions, matrixOfExperimentalData) {
    
    idx <- match(matrixOfExperimentalData[, 1], matrixOfPredictions[, 1])
    
    if (all(is.na(idx))) {
        return(c(0, 0, 0, 0))
    } else {
        # Use these to get the predicted values for the nodes in common
        predictedValues <- matrixOfPredictions[idx[!is.na(idx)], 2]
        
        # The equivalent experimental values are the ones where idx is not NA
        experimentalValues <- matrixOfExperimentalData[!is.na(idx), 2]
        
        individualScores <- predictedValues * experimentalValues
        
        overallScore <- sum(individualScores)
        correctPredictions <- sum(individualScores == 1)
        incorrectPredictions <- sum(individualScores == -1)
        ambiguousPredictions <- sum(predictedValues == 0 & experimentalValues != 0)
        
        scoreBreakdown <- c(overallScore, correctPredictions, incorrectPredictions, ambiguousPredictions)
        
        return(scoreBreakdown)
    }
} 

#' @title calculate overall significance p-value
#' @description
#' Calculates the p-value of a score given the hypothesis score and the 
#' distribution table, using either the quartic or the (faster) cubic algorithm

#' @export
#' @concept CausalR
#' @param hypothesisScore score for a particular hypothesis 
#' @param predictionListStats numbers of predicted up-regulated, 
#' predicted down-regulated and ambiguous predictions predicted by the algorithm 
#' @param experimentalResultStats numbers of up-regulated, 
#' down-regulated and not significantly changed transcripts in the 
#' experimental data 
#' @param epsilon threshold that is used when calculating the p-value 
#' using the cubic algorithm 
#' @param useCubicAlgorithm  use the cubic algorithm, defaults to TRUE
#' @param use1bAlgorithm  use the 1b version of the algorithm, defaults to TRUE
#' used to calculate the p-value
#' @return the resulting  p-value
#' @examples
#' CalculateSignificance(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)



CalculateSignificance <- function(hypothesisScore, predictionListStats, experimentalResultStats, epsilon = 1e-05, useCubicAlgorithm = TRUE, use1bAlgorithm = TRUE) {
    
    if (useCubicAlgorithm) {
        if (use1bAlgorithm) {
            pValue <- CalculateSignificanceUsingCubicAlgorithm1b(hypothesisScore, predictionListStats, experimentalResultStats, epsilon)
        } else {
            pValue <- CalculateSignificanceUsingCubicAlgorithm(hypothesisScore, predictionListStats, experimentalResultStats, epsilon)
        }
    } else {
        pValue <- CalculateSignificanceUsingQuarticAlgorithm(hypothesisScore, predictionListStats, experimentalResultStats)
    }
    return(pValue)
} 

#' @title  validate format of the experimental data table
#' @description
#' Checks the format of the experimental data. This is expected to be two columns, the first containing the gene name and the second the direction of regulation, -1, 0 or 1. The function checks the number of columns and the values of the second column,
#' @param dataTable the data table to be tested 
#' @return true if the data table is valid

ValidateFormatOfDataTable <- function(dataTable) {
    
    isValid = TRUE
    
    # Test 1 Check the table has two columns
    if (ncol(dataTable) != 2) {
        print("The experimental data read in didn't have two columns")
        isValid = FALSE
    }
    # Test 2 Check all the entries in the second column are 1, 0 or -1.
    if (isValid) {
        for (i in 1:nrow(dataTable)) {
            value <- (dataTable[i, 2])
            if (!((value == "1") | (value == "0") | (value == "-1") | (value == "+1"))) {
                print("The values in the second column do not match what is expected. They should be (+)1, 0 or -1")
                isValid = FALSE
            }
        }
    }
    
    return(isValid)
} 

#' @title Get paths in Sif format
#' @description
#' Converts network paths into Simple interaction file (.sif) format for importing into Cytoscape

#' @param arrayOfPaths an array of paths (in the format outputted by GetShortestPathsFromCCG) to be converted to .sif format
#' @return network visualisation

GetPathsInSifFormat <- function(arrayOfPaths)  {
    
    listofstuff <- NULL
    
    if (!is.null(arrayOfPaths)) {
        if (length(arrayOfPaths) > 0) {
            for (i in 1:nrow(arrayOfPaths)) {
                for (j in 2:ncol(arrayOfPaths)) {
                    
                    from <- arrayOfPaths[i,j-1]
                    to <- arrayOfPaths[i,j]
                    
                    finalChars <- paste(substr(from, nchar(from), nchar(from)), substr(to, nchar(to), nchar(to)), sep="")
                    
                    if (finalChars=="++" || finalChars=="--") {
                        word = "Activates"
                    } else if (finalChars=="+-" || finalChars=="-+"){
                        word = "Inhibits"
                    } else {
                        stop(paste("Error: Nodes should end in either + or -"))
                    }
                    
                    listofstuff <- rbind(listofstuff, paste(substr(arrayOfPaths[i,j-1], 1, nchar(arrayOfPaths[i,j-1])-1), word, substr(arrayOfPaths[i,j], 1, nchar(arrayOfPaths[i,j])-1), sep="\t"))
                }
            }
        }
    }
    
    return(unique(listofstuff))
}

#' @title get maximum D value from two-by-two contingency table
#' @description
#' Computes the maximum D value (or weight) given approximate values of n++, n+-, n-+ and n--.
#' These values are approximate and in general are non-integer values; they are found by using an 
#' approximation detailed in the paper Assessing statistical significance in causal graphs on page 6 i.e. n_ab is approximately
#' equal to q_a*n_b/t where a and b are either +, - or 0. The value is an approximation since the direction in which the 
#' number should be rounded is not clear and hence this function runs through all possible combinations of rounding before
#' concluding the maximum D-value.


#' @param twoByTwoContingencyTable approximate values of n++, n+-, n-+ and n--, these values arecalculated to optimise the D-value
#' @param predictionListStats a vector containing the values q+, q- and q0 the number of positive/negative/non-significant (or contradictory) predictions)
#' @param experimentalDataStats a vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or contradictory) transcripts in the results)
#' @param logOfFactorialOfPredictionListStats a vector containing the log of the factorial value for each entry in predictionListStats
#' @param returnlog whether or not he value should be returned as a log (TRUE) or not (FALSE)
#' @return the maximal D-value


#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.


GetMaximumDValueFromTwoByTwoContingencyTable <- function(twoByTwoContingencyTable, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
    returnlog = FALSE) {
    # A function that will compute the maximum D value (or weight) given approximate values of n++, n+-, n-+ and n--.  The reason these values are
    # described as approximate is that they are non-integer values; they are found by using an approximation detailed in the paper Assessing statistical
    # significance in causal graphs on page 6 i.e. n_ab is approximately equal to q_a*n_b/t where a and b are either +, - or 0. The value is an
    # approximation since the direction in which the number should be rounded is not clear and hence this function runs through all possible
    # combinations of rounding before concluding the maximum D-value.
    
    
    # We need to find all the combinations (maximum of 16) when applying the rounding to the values in twoByTwoContingencyTable.  The exception is if
    # all four values are integers.  combinationsOfRounding will be a nx4 matrix containing the possible values of n_pp, n_pm, n_mp, n_mm
    if (mean(twoByTwoContingencyTable == round(twoByTwoContingencyTable)) == 1) {
        # All four values are integers.
        combinationsOfRounding <- matrix(twoByTwoContingencyTable, ncol = 4)
    } else {
        combinationsOfRounding <- GetAllPossibleRoundingCombinations(twoByTwoContingencyTable)
    }
    
    maximumDValue <- 0
    
    for (i in 1:nrow(combinationsOfRounding)) {
        numbersOfCorrectandIncorrectPredictions <- combinationsOfRounding[i, ]
        threeByThreeContingencyTable <- PopulateTheThreeByThreeContingencyTable(numbersOfCorrectandIncorrectPredictions[1], numbersOfCorrectandIncorrectPredictions[2], 
            numbersOfCorrectandIncorrectPredictions[3], numbersOfCorrectandIncorrectPredictions[4], predictionListStats, experimentalDataStats)
        # Some of the combinationations produce an invalid contingency table - this we can check by seeing if any of the values in the table are negative.
        if ((min(threeByThreeContingencyTable) >= 0)) {
            weight <- CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable, logOfFactorialOfPredictionListStats, returnlog)
            maximumDValue <- max(maximumDValue, weight)
        }
    }
    
    return(maximumDValue)
} 

#' @title make predictions from CG
#' @description
#' Create a matrix of predictions for a particular hypothesis - the output
#' is a matrix containing the relationship between each node and the hypothesis.
#' The hypothesis provided will be the vertex id of one of the nodes in the network (as an integer). 
#' The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation
#' @param hypothesisnode a hypothesis node
#' @param signOfHypothesis the direction of change of hypothesis node
#' @param network a computational causal graph
#' @param delta the number of edges across which the hypothesis should be followed
#' @param nodesInExperimentalData the number of nodes in experimental data
#' @return an matrix containing the relationship between each node and the hypothesis
#' @export
#' @concept CausalR
#' @examples
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' cg <- CreateCG(network)
#' MakePredictionsFromCG('NodeA', +1, cg, 2)

MakePredictionsFromCG <- function(hypothesisnode, signOfHypothesis, network, delta, nodesInExperimentalData = NULL) {
    
    signOfHypothesis <- as.integer(signOfHypothesis)
    delta <- as.integer(delta)
    
    if (is.numeric(hypothesisnode)) {
        hypothesis <- hypothesisnode
    } else {
        hypothesis <- which(igraph::V(network)$name == hypothesisnode)
    }
    
    # Check that the input hypothesis is a possible id. Ids are numbered from one to the number of vertices, so it is enough to check the hypothesis is
    # not less than one or greater than the number of vertices.
    if (is.na(hypothesis) || length(hypothesis) != 1 || hypothesis < 1 || hypothesis > igraph::gorder(network)) {
        stop("Couldn't create predictions - hypothesis ", hypothesisnode, " is not present in the network")
    }
    
    # Create subgraph containing the nodes connected to the hypothesis node
    subgraphOfConnectedNodes <- igraph::make_ego_graph(network, delta, hypothesis, "out")[[1]]
    
    hypothesisInSubgraph <- which(igraph::V(subgraphOfConnectedNodes)$name == igraph::V(network)[hypothesis]$name)
    
    # Get ids of connected nodes (these are will be the ids from the full network)
    idsOfConnectedNodes <- igraph::vertex_attr(subgraphOfConnectedNodes, "ID")
    
    if (!is.null(nodesInExperimentalData)) {
        # Get ids of nodes which are both in the subgraph and in the experimental data
        idsOfNodesInSubgraphAndData_main <- RemoveIDsNotInExperimentalData(idsOfConnectedNodes, nodesInExperimentalData)
        nNodes <- length(idsOfNodesInSubgraphAndData_main)
        
        # If there are no such nodes, the CCG will be empty, so return
        if (nNodes == 0) {
            return(NULL)
        }
        
        # Get the IDs of these nodes in the subgraph's numbering
        idsOfNodesInSubgraphAndData_sub <- FindIdsOfConnectedNodesInSubgraph(idsOfNodesInSubgraphAndData_main, subgraphOfConnectedNodes)
        
        # Create the matrix of causal relationships
        matrixOfCausalRelationships = matrix(0, nNodes, 2)
        matrixOfCausalRelationships[, 1] <- idsOfNodesInSubgraphAndData_main
        matrixOfCausalRelationships[, 2] <- GetMatrixOfCausalRelationships(hypothesisInSubgraph, subgraphOfConnectedNodes, idsOfNodesInSubgraphAndData_sub)
    } else {
        # Create the matrix of causal relationships
        nNodes <- length(idsOfConnectedNodes)
        matrixOfCausalRelationships = matrix(0, nNodes, 2)
        matrixOfCausalRelationships[, 1] <- idsOfConnectedNodes
        matrixOfCausalRelationships[, 2] <- GetMatrixOfCausalRelationships(hypothesisInSubgraph, subgraphOfConnectedNodes, 1:nNodes)
    }
    
    if (signOfHypothesis < 0) {
        matrixOfCausalRelationships[, 2] <- -matrixOfCausalRelationships[, 2]
    }
    
    return(matrixOfCausalRelationships)
} 

#' @title calculate significance using the cubic algorithm 
#' @description
#' Calculates the p-value of a score given the hypothesis score and the distribution table (calculated using the cubic algorithm)
#' @export
#' @concept CausalR
#' @param  hypothesisScore the score whose p-value we want to find.
#' @param  predictionListStats numbers of predicted up-regulated, predicted down-regulated and ambiguous predictions.
#' @param  experimentalDataStats numbers of up-regulated, down-regulated and not significantly changed transcripts in the experimental data. 
#' @param  epsilon an epsilon threshold that is used when calculating the p-value 
#' using the cubic algorithm. Defaults to 1e-5.
#' @return  p-value
#' @examples
#' CalculateSignificance(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
#' CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
#' CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)

#' @references
#' L Chindelevitch et al.
#' Assessing statistical significance in causal graphs.
#' BMC Bioinformatics, 13(35), 2012.




CalculateSignificanceUsingCubicAlgorithm <- function(hypothesisScore, predictionListStats, experimentalDataStats, epsilon) {
    
    # Calculate the log of the factorial for all values in predictionListStats, this is to reduce the number of computations done later on.
    logOfFactorialOfPredictionListStats <- lfactorial(predictionListStats)
    
    logDMax <- FindMaximumDValue(predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, TRUE)
    
    # Not all values of r_, r- c+, c- are possible for the contigency table. This step finds all that combinations that are valid. Each row of this
    # matrix will represent a family of contingency tables.  Number of positive predictions from the network. This will be an upper bound for r+ (r_p)
    q_p <- predictionListStats[1]
    # Number of negative predictions from the network. This will be an upper bound for r- (r_m)
    q_m <- predictionListStats[2]
    # Number of positive results. This will be an upper bound for c+ (c_p)
    n_p <- experimentalDataStats[1]
    # Number of negative results. This will be an upper bound for c- (c_m)
    n_m <- experimentalDataStats[2]
    
    r_pm_limit <- n_p + n_m - predictionListStats[3]
    
    # Array for the D values - first element is the total D values which have a score equal to or better than the cut-off, second element is all D
    # values (i.e. the ratio is the P value).
    weights <- c(0, 0)
    logepsDMax <- log(epsilon) + logDMax
    
    # Iterate all possible values of r+, r-, c+ and c- and check if they produce a valid contingency table If so, calculate how much these values
    # contribute to the weights.
    for (r_p in 0:q_p) {
        r_m_min <- max(0, r_pm_limit - r_p)
        if (r_m_min <= q_m) {
            for (r_m in r_m_min:q_m) {
                c_p_min <- max(0, r_p + r_m - n_m)
                c_p_max <- min(n_p, r_p + r_m)
                if (c_p_min <= c_p_max) {
                    for (c_p in c_p_min:c_p_max) {
                        # calculate the column sum of m
                        weights <- weights + GetWeightsAboveHypothesisScoreAndTotalWeights(r_p, r_m, c_p, predictionListStats, experimentalDataStats, logOfFactorialOfPredictionListStats, 
                        hypothesisScore, logepsDMax, logDMax)
                    }
                }
            }
        }
    }
    
    pValue <- weights[1]/weights[2]
    
    return(pValue)
} 

#' @title create a Computational Causal Graph (CCG)
#' @description
#' Creates a computational causal graph from a network file.
#' @export
#' @concept CausalR
#' @param filename file name of the network file (in .sif file format)
#' @param nodeInclusionFile optional path to a text file listing nodes to exclude in the CCG (or include - see argument excludeNodesInFile). 
#' @param excludeNodesInFile flag to determine if nodes in inclusion file should be taken as nodes to include or nodes to exclude. Default is TRUE to exclude.
#' @return an igraph object containing the CCG.
#' @examples
#' # get path to example .sif file
#' network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#' #create ccg
#' ccg = CreateCCG(network)

#' 
#' 
#' @usage CreateCCG(filename, nodeInclusionFile = NULL, excludeNodesInFile = TRUE)
#' 
#' @note CreateCG and CreateCCG create causal and computational causal graphs respectively.
#' 
#' @references
#' L Chindelevitch et al.
#' Causal reasoning on biological networks: Interpreting transcriptional changes.
#' Bioinformatics, 28(8):1114-21, 2012.



CreateCCG <- function(filename, nodeInclusionFile = NULL, excludeNodesInFile = TRUE) {
    # Create a Computational Causal Graph containing separate +ve and -ve nodes from a SIF file
    
    # Note that this is a CCG in the sense that the Chindelevitch et al paper uses.  Generally in this code we use a CCG to mean the matrix of
    # interactions.
    
    
    tableOfInteractions <- ReadSifFileToTable(filename)

    if (!is.null(nodeInclusionFile)) {
        # If a node 'inclusion file' has been specified get a vector of TRUE/FALSE, where TRUE corresponds
        # to rows that contain at least one of the nodes listed in the file
        inclusionNodes <- scan(nodeInclusionFile, what = "", quiet = TRUE)
        inclusionNodesFound <- tableOfInteractions[,1] %in% inclusionNodes | tableOfInteractions[,3] %in% inclusionNodes 
        
        if(excludeNodesInFile) {
            tableOfInteractions <- tableOfInteractions[!inclusionNodesFound,]
        } else {
            tableOfInteractions <- tableOfInteractions[inclusionNodesFound,]
        }
    }
    
    if(nrow(tableOfInteractions) < 1) {
        stop("No interactions found in network file (or all interactions are excluded by nodeInclusionFile)")
    }
    
    nEdges <- dim(tableOfInteractions)[1]
    
    nodeNames <- unique(c(tableOfInteractions[, 1], tableOfInteractions[, 3]))
    
    # Alternative approach to getting the node names which puts them in the same order that creating an igraph directly from tableOfInteractions would:
    # temp <- matrix(c(tableOfInteractions[,1],tableOfInteractions[,3])) dim(temp) <- c(nEdges,2) temp <- t(temp) dim(temp) <- c(1,nEdges*2) nodeNames
    # <- unique(matrix(temp))
    
    nNodes <- length(nodeNames)
    
    # Create a list of signed node names, i.e. 'node0+', 'node1+', ..., 'node0-', 'node1-', ...
    nodeNamesSigned <- c(nodeNames, nodeNames)
    for (i in 1:nNodes) {
        nodeNamesSigned[i] <- paste(nodeNamesSigned[i], "+", sep = "")
    }
    for (i in (nNodes + 1):(nNodes * 2)) {
        nodeNamesSigned[i] <- paste(nodeNamesSigned[i], "-", sep = "")
    }
    
    # Create a matrix of the edges of the CCG Two entries per line in the SIF file - one from nodeX+ and the other from nodeX-
    edgeList <- matrix(0, nEdges * 2, 2)
    for (i in 1:nEdges) {
        startNode <- which(tableOfInteractions[i, 1] == nodeNames)
        endNode <- which(tableOfInteractions[i, 3] == nodeNames)
        edgeList[i, 1] <- startNode
        edgeList[i + nEdges, 1] <- startNode + nNodes
        if (tableOfInteractions[i, 2] == "Activation" || tableOfInteractions[i, 2] == "Activates") {
            edgeList[i, 2] <- endNode
            edgeList[i + nEdges, 2] <- endNode + nNodes
        } else if (tableOfInteractions[i, 2] == "Inhibition" || tableOfInteractions[i, 2] == "Inhibits") {
            edgeList[i, 2] <- endNode + nNodes
            edgeList[i + nEdges, 2] <- endNode
        }
    }
    
    # Now create the igraph, and add the node names as a property to each vertex
    network <- igraph::graph_from_edgelist(edgeList, TRUE)
    igraph::V(network)$name <- nodeNamesSigned
    
    # give the network a name attribute, set to the filename minus the path and file extension
    network$name <- tools::file_path_sans_ext(basename(filename))
    
    network$isCCG <- TRUE
    igraph::V(network)$unsignedName <- c(nodeNames, nodeNames)
    
    return(network)
} 

# Copyright: Copyright 2012 GSK. All rights reserved
#
# *** SVN ***
# LastChanged: 	$Date: 2012-11-12 14:12:28 +0000 (Mon, 12 Nov 2012) $
# Changed By:  	$Author: rild $
# Version:		$Revision: 105 $
# Source:		$HeadURL: file:///L:/7000-7499/7412/SVNrepository/trunk/NetworkFileReader/CheckEdgeForTest_SifFileReader.r $
# 
# *** Summary ***
# Description:  A helper file for the function Test_SifFileReader

CheckEdgeForTest_SifFileReader <- function(edge, vertex1, vertex2){
checkEquals(edge[1],vertex1)&&checkEquals(edge[2], vertex2)
}

%\VignetteEngine{knitr::knitr}
%\ !Rnw weave = knitr
<<style-knitr,eval=TRUE,echo=FALSE,results="asis">>=BiocStyle::latex()
@

%\vignette>
%\VignetteIndexEntry{CausalR : an R Package for causal reasoning on networks}
%\vignettePackage{CausalR}
%\VignetteDepends{igraph,CausalR}
%\VignetteKeyword{GraphAndNetwork}
%\VignetteKeyword{Network}
%\vignetteEncoding{utf8}
\let\nofiles\relax 


\documentclass[11pt]{article}

%\usepackage[utf8]{inputenc}
\usepackage{verbatim} 
\usepackage[toc,page]{appendix}
\usepackage[backend=bibtex8]{biblatex}


\parskip=0.8mm
\parindent=0pt



\title{Overview of Causal Reasoning with \textit{CausalR} : \\
Hypothesis Scoring and P-value Calculation}

\author{Steven J Barrett, Glyn Bradley}

%\bibliographystyle{alpha}


\begin{document}

\maketitle

\tableofcontents

%\clearpage
\newpage

\section{Introduction}

\textit{CausalR} provides causal reasoning (causal network analysis) methods for the Bioconductor project, enabling the prediction of regulators of sets of observed endpoints (e.g. genome scale expression measurements) by integration with prior knowledge in the form of a causal interaction network. Predicted regulators (hypotheses) are assigned a score reflecting their fit with the input differiential experimental data. Two levels of signmificance are computed for each hypothesis. 

\vspace{5mm}

Table 1 is an example of the ranked hypotheses output of \textit{CausalR}, produced by executing functions {\tt CreateCCG()}, {\tt ReadExperimentalData()} then {\tt RankTheHypotheses()} on the example causal network and experimental data used in the following text.


\begin{table}[h!]
\hskip-1.0cm\begin{tabular}{ |c|p{1.7cm}|p{0.8cm}|p{1.15cm}|p{1.45cm}|p{1.8cm}|p{1.25cm}|p{1.8cm}|  }
\hline
Node &Regulation &Score &Correct &Incorrect &Ambiguous &p-value &Enrichment p-value \\
[0.3ex]
\hline
Node0 &1 &4 &5 &1 &1 &0.114 &1 \\
\hline
Node1 &1 &3 &3 &0 &0 &0.121 &1 \\
\hline
Node3 &1 &1 &1 &0 &0 &0.500 &0.875 \\
\hline
Node5 &-1 &1 &1 &0 &0 &0.375 &0.875 \\
\hline
Node6 &-1 &1 &1 &0 &0 &0.375 &0.875 \\
\hline
Node7 &-1 &1 &1 &0 &0 &0.375 &0.875 \\
\hline
Node2 &1 &0 &2 &2 &0 &0.629 &0.5 \\
\hline
Node4 &1 &0 &0 &0 &0 &0.625 &1 \\
\hline
Node2 &-1 &0 &2 &2 &0 &0.543 &0.5 \\
\hline
Node4 &-1 &0 &0 &0 &0 &0.500 &1 \\
\hline
Node5 &1 &-1 &0 &1 &0 &1 &0.875 \\
\hline
Node6 & 1 &-1 &0 &1 &0 &1 &0.875 \\
\hline
Node7 &1 &-1 &0 &1 &0 &1 &0.875 \\
\hline
Node3 &-1 &-1 &0 &1 &0 &1 &0.875 \\
\hline
Node1 &-1 &-3 &0 &3 &0 &0.957 &1 \\
\hline
Node0 &-1 &-4 &1 &5 &1 &0.971 &1 \\ [1ex]
\hline
\end{tabular}
\caption{\textit{CausalR} {\tt delta=2} output for example inputs.}
\label{tab:t1}
\end{table}

Reconstruction of regulatory networks from either newly predicted of previous known regulators of the input experimental data is facilitated, outputting .sif network and annotation files for visualisation in Cytposcape. The \textit{SCAN} (Sequential Causal Analysis of Netwroks) methodology of limited false positives in the prediction of regulators is also provided. For further background on methodological details, suggested analysis approaches and an actual biological causal network, see Bradley and Barrett \cite{submitted}.

\vspace{5mm}
The following tutorial text provides a step-by-step series of examples on using \textit{CausalR}, together with further descriptive explanation of what is happening at each stage. For convenience, the complete set of commands, with details of further functionality, are listed in Appendix A in the form of an R script.



\section{Loading Input Network and Experimental Data}

After starting the R environment (either the standard R console or via an IDE such as \textit{Rstudio, http://www.rstudio.com}), and setting the working directory to an appropriate location for accessing input files, proceed to loading the \textit{CausalR} library, 

<<>>=
library(CausalR)
@

The loading of the main package also performs 

<<eval=FALSE>>=
library(igraph)
@

since \textit{igraph} contains network analysis functionality required by CausalR.
%Then load the required \textit{igraph} package, which contains network analysis tools used by CausalR,


\vspace{5mm}
%Set the working directory to the location of inputs.

%setwd("E:/CR/GlynsFinal/CausalR/inst/extdata")

These triples represent the network to be used in the example processing,

%\vspace{5mm}

\begin{verbatim}
Node0 Activates Node1
Node0    Inhibits    Node2
Node1    Activates    Node3
Node1    Activates    Node4
Node1    Inhibits    Node5
Node2    Inhibits    Node5
Node2    Activates    Node6
Node2    Activates    Node7
\end{verbatim}

%\vspace{5mm}

Each line triple represents a directed edge in the input network of Figure 1.

\begin{figure}[ht]
    \includegraphics{Fig1.png}
    \caption{ Example input network. }
    \label{fig:f1}
\end{figure}

This is read from tab-separated (Cytoscape {\tt .sif} format) file by one of two user functions, {\tt CreateCG()} or {\tt CreateCCG()} which respectively create and store either a  causal or a computational causal graph. To understand what these are we first show how the function {\tt CreateCG()} is used to create an internal representation of the network as a Causal Graph (CG).

<<>>=
cg <- CreateCG(system.file( "extdata", "testNetwork1.sif", 
    package="CausalR"))
@

The CG is stored in the workspace as an \textit{igraph} object,  which is a particular type of object that can be used to store and process graphs in R (see \textit{http://igraph.org/r/doc/igraph.pdf} ). The Causal Graph can then be visualised via,
 
<<>>=
PlotGraphWithNodeNames(cg)  # producing the following graph.
@

Note that although only directional connectivity is shown, edge sign attribution of the input network is retained. Note also that \textit{igraph} will assign internal numeric nodeIDs (for simplicity, the example network here mirrors these) that are used internally by \textit{CausalR} functions. Most \textit{CausalR} functions will automatically convert \textit{igraph} nodeIDs back to node names recognisable to the user, but there are exceptions that return output with these numbers. For those outputs, the user will need to apply the {\tt GetNodeName()}.

\vspace{5mm}

The {\tt CreateCCG()} function creates in the workspace an \textit{igraph} object (here assigned to "ccg") which contains a computational causal network representation of the contents of the input network.sif file.

<<>>=
ccg <- CreateCCG(system.file( "extdata", "testNetwork1.sif", 
    package="CausalR"))
@

The latter is employed internally instead of a CG for processing efficiency. 

\vspace{5mm}

A CCG contains double the nodes and vertices, to separately represent up and down regulation of each node, and these are connected according to the original edge relations from the network.sif file. The Computational Causal Graph (CCG) for the input network input can be visualised via,


<<>>=
PlotGraphWithNodeNames(ccg) # producing the following graph.
@

The input differential experimental results (used in subsequent worked examples) is as follows,

\begin{verbatim}
Node0    1
Node1    1
Node2    1
Node3    1
Node4    0
Node5    -1
Node6    -1
Node7    -1
\end{verbatim}

In the second column, 1 represents differential activation, -1 differential inhibition and 0 indicates no differential regulation was observed for the entity represented by the node. 

\vspace{5mm}

The inclusion of all non-differential results is critical to accurate significance calculations (in their absence \textit{CausalR} will otherwise see them as not measured and process them accordingly). 

\vspace{5mm}

A tab-separated file containing this content is read into the workspace using {\tt ReadExperimentalData()}, as follows,


<<>>=
experimentalData <- ReadExperimentalData(system.file( "extdata", 
                        "testData1.txt", package="CausalR"),ccg)
@

At this stage checks are performed to identify entities (nodes) not present within the CG/CCG.

\vspace{5mm}

These are flagged to the user and excluded from the two column experimental data matrix [\textit{igraph} nodeID, regulation] that is stored in the workspace for subsequent processing.

\newpage

\section{Computing Score: Multiple Causal Hypotheses}


To compute a list of score-ranked hypotheses within an interactive session, the user would employ the {\tt RankTheHypotheses()} function as follows, 

{\scriptsize
<<>>=
options(width=120)
RankTheHypotheses(ccg, experimentalData, delta=2) 
@
}

{\tt RankTheHypotheses()} incorporates usability features (see Section 9) and parallelisation is supported (see Appendix B for instructions).

\vspace{5mm}

Alternatively, the following is used to output similar (results for both + and - hypotheses will again be generated) for a subset of nodes,

{\scriptsize
<<>>=
options(width=120)
testlist<-c('Node0','Node2','Node3')
RankTheHypotheses(ccg, experimentalData, 
                                delta=2, listOfNodes=testlist)
@
}

{\tt RankTheHypotheses()} takes as arguments the workspace CCG and experimental matrix object (both from the processing described above), together with a user supplied numeric value for the path length, termed delta, $delta$. The latter represents the desired number of edges in the network the user wishes the algorithm to look forward, from the hypothesis, to observed result nodes.

\vspace{5mm}

With each further level searched into the network, the number of connected nodes increases roughly exponentially until the $delta$ approximates the network average degree. With deltas close to or above the network average degree the majority of hypothesis nodes will be connected to most other nodes in the network, and consequently the ability of the scoring algorithm to differentiate good hypotheses from bad ones will be degraded.

\vspace{5mm}

{\tt RankTheHypotheses()} calls the hidden function {\tt Scorehypothesis()}, which computes  scores reflecting the overall level of agreement between predictions based upon an individual hypothesis and the experimentally observed results for nodes within $delta$ edges (i.e. to which it can be causally-linked).


\section{Computing Score: Specific Causal Hypothesis}

To get scores and significance summary for both up- and down-regulation hypotheses concerning an individual node, the {\tt listOfNodes} input can be assigned to a sole node name (note that node names are case sensitive in \textit{CausalR}),

{\scriptsize
<<>>=
options(width=120)
RankTheHypotheses(ccg, experimentalData, 2, listOfNodes='Node0')
@
}

It may first be useful to check on node content and length of the shortest path from the proposed hypothesis (root) node to any specific (outcome) target node(s) of interest. For this the function {\tt GetShortestPathsFromCCG()} is employed,

<<results='hide'>>=
GetShortestPathsFromCCG(ccg, 'Node0', 'Node3')
@

{\tt node0+ node1+ node3+}
\vspace{3mm}

An alternative to {\tt RankTheHypotheses()} enables acquisition of further detail on the breakdown of scoring contributions. The {\tt MakePredictionsFromCCG()} function can be used directly to get scores for a particular signed hypothesis of a node, 

<<>>=
predictions <- MakePredictionsFromCCG('Node0',+1,ccg,2)
predictions
@

The input {\tt node0} is the root (or hypothesis) node, +1 is the direction of regulation, ccg is the network, and the last value, 2, is the {\tt delta} value (these inputs are explained in more detail in the following section).  The predictions will be returned as above, with the \textit{igraph} assigned node ID in the first column and the direction of regulation in the second.

\vspace{5mm}

The {\tt ScoreHypothesis()} function can then be used to provide a summary of the comparisons between the predictions from the given hypothesis against the observed outcomes in the experimental data,

<<>>=
ScoreHypothesis(predictions, experimentalData)
@

This returns 4 integer values, which (in order) are the score (the number of correct predictions minus the number of incorrect predictions), number of correct predictions, number of incorrect predictions and number of ambiguous predictions (these will be explained later within the Scoring Section).

\vspace{5mm}

To discover how each predicted node outcome contributes to the score, the {\tt CompareHypothesis()} function is used to provide a comparison between experiment and prediction for each node separately,


<<>>=
GetNodeName(ccg,CompareHypothesis(predictions, experimentalData))
@

The use of function {\tt GetNodeName()} is required here to convert \textit{igraph} nodeIDs, output by {\tt CompareHypothesis()}, back to user recognisable node names, as was supplied in the inputs.

\vspace{5mm}

Functions {\tt CalculateSignificance()} and {\tt CalculateEnrichmentPValue()} can be used to calculate the statistical significance of a given signed hypothesis, as detailed in the R code of Appendix A.

\subsection{Hypothesis-specific CCGs}

During scoring, hypothesis-specific CCGs are sequentially created and scored. Each hypothesis will be represented as the unique root node of its corresponding CCG, whilst the maximum depth for any hypothesisCCG is equal to the path length, {\tt delta}.

\vspace{5mm}

Since each hypothesis observes a either upward or downward regulation, the number of individual hypothesisCCGs computed and scored will be equal to twice the number of nodes in the input network, if all possible hypotheses are tested, as this is the default behaviour for function {\tt RankTheHypotheses()}.  

\vspace{5mm}

Prior to the construction of each hypothesisCCG, nodes beyond {\tt delta} edges from the hypothesis are removed from the processed network and untested nodes (those without experimental results, apart from the hypothesis node itself) are also excluded. To construct the final hypothesisCCG, regulatory relations are then applied between the hypothesis along all paths toward the remaining nodes, observing the original network edge values.

\section{Scoring}

To aid scoring the CCG object represents a partitioning of the set of nodes in the experimental data, represented by $T|G|$,  into three sets: the positively regulated, negatively regulated and those that are unaffected. These sets are denoted as $S_h+ , S_h-$ and $S_h0$ respectively, as defined in \cite{2012a}. 

\vspace{5mm}

Thus in the terminology of the Methods: Scoring hypotheses section of \cite{2012a}, the observed experimental input (as above) would be represented as :

\begin{eqnarray*}
G_+ & = & \left\{ 
\mbox{Node0},
\mbox{Node1},
\mbox{Node2},
\mbox{Node3}
\right\} \\
G_- & = & \left\{ 
\mbox{Node5},
\mbox{Node6},
\mbox{Node7}
\right\} \\
G_0 & = & \left\{ 
\mbox{Node4}
\right\}
\end{eqnarray*}

Any node greater than {\tt delta} edges from the hypothesis node is automatically added to $S_h0$, since the path length to the hypothesis is deemed too distant to include in scoring, according to user parameterisation. 

\begin{figure}[ht]
\includegraphics{Fig2.png}
\caption{ Network consequences of {\tt Node0+} }
\label{fig:f2}
\end{figure}

\subsection{Example Causal hypothesis to be Scored}

In the following example calculations, the causal hypothesis to be scored is up-regulation of Node0, denoted as  {\tt Node0+}. Figure 2 above shows the downstream consequences of up-regulating Node0, as derived by traversing its hypothesisCCG, observing how the original network edge values translate the regulatory sign of the (root) hypothesis along the paths to predicted outcomes. Note that Node 5 is both positively and negatively regulated by {\tt Node0+}.

\subsection{Network Predictions based upon Hypothesis {\tt Node0+}}

The predictions $|TG_c|$ from the network (represented as per \cite{2012a}, Methods: Scoring hypotheses) would be:

\begin{eqnarray*}
S_h+ & = & \left\{ 
\mbox{Node0},
\mbox{Node1},
\mbox{Node3},
\mbox{Node4}
\right\} \\
S_h- & = & \left\{ 
\mbox{Node2},
\mbox{Node6},
\mbox{Node7}
\right\} \\
S_h0 & = & \left\{ 
\mbox{Node5}
\right\}
\end{eqnarray*}


\subsection{Scoring Calculation}

[Example for {\tt delta=2}] Comparing $T|G|$ to $T|G_c|$ yields the correct predictions from this hypothesis as: Node 0, Node 1, Node 3, Node 6 and Node 7. The only incorrect prediction from this hypothesis is:  Node 2. 

\vspace{5mm}

The \textit{CausalR} scoring algorithm traces the shortest path from hypothesis to outcome node and then evaluates its sign to achieve a predicted regulation for that outcome node. The hypothesis here yields a so called "ambiguous" prediction-outcome match for Node 5, since there are two shortest paths to it and their predictions disagree, hence it doesn't contribute to the scoring.

\vspace{5mm}

Node 4 is also excluded from the score calculation because the observed experimental outcome for it was non-differentially regulated (unchanged) and this is a result that cannot be predicted by CausalR, as only an up or down regulation can be inferred from a signed graph.

\vspace{5mm}

Subtracting number of incorrect predictions (1) from the number that was correct (5) yields a score of 4 for hypothesis {\tt Node0+}.


\section{Hypothesis Score Significance Calculation}

To account for the scenario where the more heavily connected hypothesis nodes have greater potential to score more highly than less connected ones, the significance of each hypothesis score needs to be individually calculated.

\vspace{5mm}

A lower significance (higher p-value) cut-off can then be applied as a filter to the overall scores ranking, as decided via the biological expertise of the user.

\vspace{5mm}

\textit{CausalR} provides both p-values and enrichment p-values for this purpose. There are two alternatives for p-value calculation: a computationally expensive exact "quartic" algorithm (see section 2.4 in \cite{2012a}) and (for larger inputs) a very much more efficient (minutes instead of hours, hence set as default)  approximate "cubic" algorithm (see pages 6-7 in \cite{2012b}). 

\subsection{Calculation of p-value}

[Continuation of previous example for {\tt delta=2}] Within the context of network causal reasoning (see section 2.4, \cite{2012a}), we are looking for the probability of obtaining a hypothesis (node) score at least as extreme as that which was observed, assuming the null hypothesis, that the hypothesis node was not responsible for producing the experimental outcomes, is true (i.e.\ result is not significantly different from random).  

\vspace{5 mm}

To calculate the p-value significance of this score for hypothesis {\tt Node0+}, these predictions must be scored against all possible predictions with $|G+| = 4$, and $|G-| = 3$.

\vspace{5 mm}

Given that there are 8 nodes to pick each combination from, the total number of combinations is:  ${}^8C_4 \times {}^4C_3 = 280$

\vspace{5 mm}

There is only one way to get the highest possible score of 7.

\vspace{5 mm}

There are two approaches to achieve a score of 6. The first is to choose all 4 nodes in $ G+$ from the nodes in $ S_h+$, and two of the nodes in $ G-$ from $ S_h-$: there are ${}^4C_4$ x ${}^3C_2 = 3$ ways of doing this. The other possibility is to choose 3 nodes in $ S_h+$ and 1 in $ S_h0$ as the nodes in $ G+$. Then choose all three of the nodes in $ G-$ from $ S_h-$ there are ${}^4C_3$ x ${}^3C_3 = 4$ ways of doing this. Hence there are 7 ways of scoring 6 in total.

\vspace{5 mm}

A score of 5 cannot be achieved.

\vspace{5 mm}

There are two ways to score 4. 
(The first is to choose 3 nodes from $S_h+$ and 1 from $S_h-$ as the nodes in $ G+$, then choose 2 nodes from $S_h-$ and the 1 in $S_h0$ as the nodes in $G-$. There are ${}^4C_3$ x ${}^3C_1 = 12$ ways of doing this. The second way is to choose 3 nodes from $ S_h+$ and the 1 from $ S_h0$ as the nodes in $ G+$, then choose 2 nodes from $ S_h-$ and 1 in $ S_h+$ as the nodes in $ G-$. There are ${}^4C_3$ x ${}^3C_2 = 12$ ways of doing this. This means in total there are 24 ways of scoring 4.

\vspace{5 mm}

We now have all the information we need to calculate the p-value of the hypothesis above. There are $24+7+1 = 32$ ways to score 4 or higher, hence the p-value should be $32/280 = 0.114$, as can be seen for the up-regulated node 0 hypothesis in Table 1. 

\subsection{Calculation of Enrichment p-value}

Enrichment $p-$values, $p_E$, are calculated using standard approaches used for gene ontology enrichment (see \cite{GOEAST})

\vspace{5 mm}

Where $n_{++} + n_{+-} + n_{-+} + n_{--} = m$ ( i.e. the number of differentially expressed genes, $m$ in Table 2), and $T = T|G|$ is the total number of experimental transcripts; then $p_E$ is the probability of getting m or more differential  nodes from a set of $T$ nodes given the fraction of nodes predicted to be differential.

\vspace{5 mm}

This computation is done using the Fisher exact test (see \cite{Fisher}), as implemented in the fisher.test function in base R. 

\vspace{5mm}

The goal is to compare the actual number of differential nodes to the number of predicted ones, given the values of $q_+, q_- , q_0 , n_+, n_-, n_0, n_{++}, n_{+-}, n_{-+}$ and $n_{--}$ in Table 2 below.

\vspace{10 mm}

\begin{table}[h!]
\begin{tabular}{ |p{2.8cm}|p{5cm}|p{2cm}|p{1.5cm}|}
\hline
\textit{Predicted} \newline Causal Network &\textit{Measured} \newline Differential &\textit{Measured} Non-differential &\textit{Measured} Total\\
\hline
Differentially Expressed &$n_{++} + n_{+-}  + n_{-+} + n_{--}$ $ = m$ & $ n_{+_0} + n_{-0}$ & $ q_+ + q_-$ \\
\hline
Non-differentially Expressed &$ n_{0+} + n_{0-} = n_+ + n_- - m$ & $ n_{00}$ & $ q_0$ \\
\hline
Total & $n_+ + n_-$ &$ n_0$ &$ T|G|$ \\
\hline
\end{tabular}
\caption{ Result outcome categorisations used in significance calculations.}
\label{tab:t2}
\end{table}


\vspace{10 mm}


The probability of getting this arrangement of values is given by the hypergeometric distribution,

\begin{displaymath}
p=\frac{{q_+ + q_- \choose m}{q_0 \choose n_{0+}+n_{0-}}}{{|T(G)| \choose n_++n_-}}
\end{displaymath}

To obtain $p_E$, the probability of getting m or more elements in the intersection between the sets of predicted versus experimentally-observed differential nodes, needs to be calculated. The formula employed to calculate the enrichment p-value is therefore,

\begin{displaymath}
p_E= \sum^{q_+ + q_-}_{i=m} \frac{{q_+ + q_- \choose i}{q_0 \choose n_{+}+n_{-}-i}}{{|T(G)| \choose n_++n_-}}
\end{displaymath}

\vspace{5mm}

\subsection{Efficiency and Accuracy of Significance Calculation}

As stated earlier, it is crucial to include non-differential outcomes in the experimental input for accurate significance calculations. The problem with this is that for large NGS study data compute times increase greatly and much of the added effort will be on significance calculations for poor hypotheses. 

\vspace{5mm}

This can be avoided by resetting the {\tt correctPredictionsThreshold} value used by the {\tt RankTheHypotheses()} function.  This is normally set to minus infinity, which means that $p$ and $p_E$ values will be calculated regardless of hypothesis score. When reset to a positive value it restricts significance calculations to a subset of hypotheses with a minimum number of correct predictions (all other hypotheses will automatically get {\tt NA} values).


{\scriptsize
<<>>=
options(width=120)
Rankfor4<-RankTheHypotheses(ccg, experimentalData, 2, 
                            correctPredictionsThreshold=4)
Rankfor4   # For example output only
subset(Rankfor4,Correct>=4)
@
}

This can also be useful where the user wishes to employ the more computationally expensive exact quartic (1a) algorithm on a subset of interesting hypotheses, instead of using the default cubic (1b) approximation algorithm.

\newpage

\section{Sequential Causal Analysis of Networks (SCAN)}

Given that true causal regulators are likely to regulate nodes at various distances along the pathways in which they feature, the use of a range of different deltas can be employed to improve enrichment for them. This was the thinking behind Sequential Causal Analysis of Networks ($SCAN$) approach introduced in Bradley and Barrett \cite{submitted}.

\vspace{5 mm}

By invoking the {\tt runSCANR()} function, the {\tt RankThehypotheses()} function is iteratively run for a range of path lengths ({\tt deltas}), starting  from {\tt delta=1} up to the value of the user-supplied variable {\tt NumberOfDeltaToScan} (default is 5), to discover nodes that consistently score in the highest  {\tt topNumGenes} (default is 150) across all SCAN deltas run.

\vspace{8 mm}

So for example given the following experimental data,

\begin{verbatim}
NodeA    1
NodeB    -1
NodeC    1
NodeD    -1
NodeE    1
NodeF    -1
NodeG    1
NodeH    -1
NodeI    1
NodeJ    -1
\end{verbatim}

\vspace{5 mm}

and a more complex network, as in the {\tt Cytoscape} (\textit{http:www.cytoscape.org}) depiction of {\tt testnet.sif} in Figure 3 below).

\begin{figure}[ht]
    \includegraphics[width=120mm,scale=1.0]{Fig3.png}
    \caption{ Network input for SCAN Example. }
    \label{fig:f3}
\end{figure}

\vspace{5mm}

The {\tt runSCANR()} function will first write the number of nodes to analyse to the R console window (this will be repeated for each delta run). Then the next {\tt NumberOfDeltaToScan} lines output will list the top hypotheses from each delta that was run, whilst the final line gives the SCAN results for the top hypotheses in common between these deltas. 

\vspace{1mm}

<<>>=
runSCANR(ccg, experimentalData, numberOfDeltaToScan=2, topNumGenes=4, 
         correctPredictionsThreshold=1,writeResultFiles = TRUE, 
         writeNetworkFiles = "none",quiet=TRUE)
@
{\tt testList} contains the hypotheses found at each delta, as restricted by topNumGenes.

\vspace{5mm}
{\tt uniqueNodes} contains all hypotheses, with the maximum delta that returned them.

\vspace{5mm}
{\tt commonNodes} contains only the hypotheses subset ranked in the first topNumGenes of all hypotheses across ALL deltas run, and these are ranked in order found at the highest delta.

\vspace{5mm}

Additionally, under the above settings, three files are created in the current R workspace:

\vspace{5mm}

1. A common nodes file: CommonNodes-testNetwork1-testData1-deltaScanned2-top4.txt

Containing:
\begin{verbatim}
Node1+
Node0+
Node3+
Node5-
\end{verbatim}

This content is equivalent to that in {\tt commonNodes} above.

\vspace{5mm}

2. A top nodes file: TopNodes-testNetwork1-testData1-deltaScanned2-top4.txt

Containing:
\begin{verbatim}
names  maxDelta
Node1+	2
Node0+	2
Node3+	2
Node5-	2
\end{verbatim}

This content is equivalent to that in uniqueNodes above.

\vspace{5mm}

3. A ResultsTable: ResultsTable-NetworkFileName-ExptlDataFileName-deltaused.txt containing RankTheHypotheses()output for the highest delta run.

\vspace{5mm}

Note that the above files are auto-named according to the input files and content-defining parameters (more details in Section 9).

\vspace{5mm}

Hypothesis-specific regulatory networks can be generated directly by {\tt runSCANR()} , from the commonNode list, however the returned number of common hypotheses won't apriori be known (see Section 8 for details of direct and indirect hypothesis regulatory network file creation).

\vspace{5mm}

From experiments with \textit{CausalR} on signals for known pathways (see Bradley and Barrett \cite{submitted}), setting of the {\tt topNumGenes} parameter to represent 1 percent of the number of nodes present in the input network provides a reliable starting heuristic for identifying the true regulators.

\vspace{5mm}

A SCAN run covering 1-5 deltas can typically take overnight with large inputs, however parallelisation is supported via setting {\tt doParallel=TRUE}). Various additional steps can also be taken to improve run times:

\begin{enumerate}
    \item Use of the R compiler (see Appendix B)
    \item Excluding scoring of hypotheses for non-differential nodes,


<<eval=FALSE>>=
AllData<-read.table(file="testData1.txt", sep = "\t")
DifferentialData<-AllData[AllData[,2]!=0,]
write.table(DifferentialData, file="DifferentialData.txt", 
    sep="\t", row.names=FALSE, col.names=FALSE, quote=FALSE)

runSCANR(ccg, ReadExperimentalData("DifferentialData.txt", ccg), 
        NumberOfDeltaToScan=2,topNumGenes=100,
        correctPredictionsThreshold=2)
@

\item Suppression of significance calculations. Note that these will be inaccurate and unused anyway if non-differential results are excluded, as with \textit{SCAN} we are relying more upon biological logic rather statistics. As this is the usual scenario, the {\tt correctPredictionsThreshold} parameter is set with a default value of Inf (infinity, so as to not compute p-values, substituting NAs for them. Scoring and rankings will still be computed for all hypotheses). {\tt correctPredictionsThreshold} can be set to a lower value if accurate p-values are required for a partial or full set of hypotheses scored.
\end{enumerate}

Accurate p-values and pE-values can also later be computed separately for the subset of interesting hypotheses coming from \textit{SCAN}  using,

<<results='hide'>>=
testlist<-c('Node0','Node3','Node2')
RankTheHypotheses(ccg, experimentalData,2,listOfNodes=testlist)
@

which gives up- and down-regulated hypothesis results for each named input node in the testlist.

\vspace{5 mm}

Alternatively, for an individual signed node hypothesis, the user can start from {\tt MakePredictionsFromCCG() } as described earlier under  section "Calculating Score for a Specific Causal Hypothesis".

\newpage

\section{Visualising the Network of Nodes explained by a Specific Hypothesis}

Hypothesis-specific regulatory sub-networks can be generated indirectly (i.e. after running SCAN) for individual hypotheses or all hypotheses, as well as directly via {\tt runSCANR()} itself.

\subsection{Indirect Individual Hypothesis Network Generation} 

The function {\tt WriteExplainedNodesToSifFile()} allows production of a network of explained nodes, for a particular hypothesis, in .sif file format for visualising in Cytoscape. The function is called as follows: \\ 

<<eval=FALSE>>=
WriteExplainedNodesToSifFile("Node1", +1,ccg,experimentalData,delta=2)
@

Unless directed elsewhere, this creates six new files in the working directory:

\begin{verbatim}
corExplainedNodes-testNetwork1-testData1-delta2-Node1+.sif
corExplainedNodes-testNetwork1-testData1-delta2-Node1+_anno.txt

incorExplainedNodes-testNetwork1-testData1-delta2-Node1+.sif
incorExplainedNodes-testNetwork1-testData1-delta2-Node1+_anno.txt

ambExplainedNodes-testNetwork1-testData1-delta2-Node1+.sif
ambExplainedNodes-testNetwork1-testData1-delta2-Node1+_anno.txt
\end{verbatim}


The correctly explained nodes (corExplainedNodes) .sif file contains interactions representing all of the explained nodes (i.e. nodes that have the same direction of regulation in both the network and the experimental data, as predicted under the hypothesis of up-regulated Node1+, with content structured as follows,

%\vspace{5mm}

\begin{verbatim}
Node1  Activates	Node1
Node1	Activates	Node3
Node1	Inhibits	Node5
\end{verbatim}

%\vspace{5mm}

The corresponding (corExplainedNodes) annotation file contains the respective explained node regulation values,

%\vspace{5mm}

\begin{verbatim}
NodeID  Regulation
Node1	1
Node3	1
Node5	-1
\end{verbatim}

%\vspace{5mm}


These files can be directly loaded into Cytoscape to visualise the regulatory network of hypothesis-explained nodes, such as in figure 4.


\begin{figure}[ht]
    \includegraphics{Fig4.png}
    \caption{ Example hypothesis-specific regulatory network (Cytoscape view). }
    \label{fig:f4}
\end{figure}




\subsection{Indirect Network Generation for All Hypotheses found Common across SCAN Deltas} 

An additional function, {\tt WriteAllExplainedNodesToSifFile()}, accepts the {\tt commonNodes} component of the {\tt scanResults} workspace object (assigned to store the {\tt runSCANR()} output) to generate networks for all the hypotheses found in common across all SCAN delta settings:

<<>>=
scanResults <- runSCANR(ccg, experimentalData, numberOfDeltaToScan=2, 
  topNumGenes=4,correctPredictionsThreshold=1,
  writeResultFiles = FALSE, writeNetworkFiles = "none",quiet=FALSE)
WriteAllExplainedNodesToSifFile(scanResults, ccg, experimentalData, 
  delta=2, correctlyExplainedOnly = TRUE, quiet = TRUE)
@

This produces pairs of {\tt corExplainedNodes} .sif and annotation network files for all hypotheses (Node0+, Node1+, Node3+ and Node5-), in each case including only those experimental data nodes that are correctly explained by that hypothesis (i.e. as defined by using writeNetworkFiles = "correct" instead of "all").

\subsection{Direct Network Generation for All Hypotheses found Common across SCAN Deltas }

The following command shows how to generate the same networks as in the latter example above, only directly from {\tt runSCANR()}


<<eval=FALSE>>=
runSCANR(ccg, experimentalData, numberOfDeltaToScan=2, 
  topNumGenes=4,correctPredictionsThreshold=1,quiet=TRUE,
  writeResultFiles = TRUE, writeNetworkFiles = "correct")
@

Files generated in this way get the full input auto-naming, i.e. corExplainedNodes-testNetwork1-testData1-delta2-top4-Node0+.sif, but care needs to be taken to sensibly try to restrict the set of common nodes produced by {\tt runSCANR()} otherwise there may be very many!

\newpage


\section{Additional Usability Features}


\subsection{Automated File Naming}

The {\tt RankTheHypotheses()} and {\tt runSCANR()} functions will auto-name their primary output files with names that identify all relevant inputs and parameterisations employed.

%\vspace{5mm}

These filenames will therefore contain the input network filename, input experimental signal filename and the input {\tt delta} parameter value. 

%\vspace{5mm}

The results table produced by {\tt runSCANR()} will also feature the {\tt topNumGenes} parameter value.

%\vspace{5mm}

All network files output by functions {\tt WriteExplainedNodesToSifFile()} and {\tt WriteAllExplainedNodesToSifFile()} will contain in their filename the specified hypothesis node name with the +/- direction of regulation.


\subsection{Output Suppression, Re-direction and Quiet Mode}

All the functions mentioned in the above section will by default output to the current R working directory, however they can to configured to send outputs to a named directory specified via the {\tt outputDir} parameter (this can accept a path with directory name, if the folder is not to be in working directory). If the specified directory doesn't exist it will be created.

{\tt RankTheHypotheses()} and {\tt runSCANR()} functions have a quiet flag that is set to FALSE by default. Changing this to TRUE will suppress output of some or all results to the screen (depending upon other settings;useful for batch scripted runs). 

Similarly, there are flags to suppress results files creation:

{\tt RankTheHypotheses()} has a writeFile flag set to TRUE so as to write a file by default. When set to {\tt FALSE} no output files are created (useful when outputs are to be stored a workspace object, i.e. via assignment using the {\tt <-} assignment operator.

{\tt runSCANR()} has a {\tt writeResultsFiles} flag which works in a similar way to control writing of the usual SCAN text file outputs. The default {\tt TRUE} setting will result in three text files. {\tt FALSE} will switch this off, i.e. for when results are directed into the R workspace via assignment to an object.

{\tt RunSCANR()} also accepts a further {\tt writeNetworkFiles} flag for control of automated regulator network files generation. This is set to {\tt all} by default.


\subsection{Operating Systems}

{\tt CausalR} as a Bioconductor package has been built for Linux, PC and Mac operating systems. Operation and functionality is identical regardless of the OS used.

\subsection{Parallelisation}

The {\tt RankTheHypotheses()} and {\tt RunSCANR()} functions are fully parallelised. Parallelisation can be switched-on by including {\tt doParallel=TRUE} in their parameters. The user can optionally specify the number of cores by setting the {\tt numCores} flag to an integer value (default is {\tt NULL}, whereby the number of available cores will be automatically detected) . See {\tt Appendix B} for further advice on speeding-up {\tt CausalR} processing.

\subsection{Network Filtering}

The {\tt CreateCCG()} function can read an optional node inclusion text file containing a list of node names to filter the network interactions against. \\
The direction of filtering is controlled by an optional {\tt excludeNodesInFile} parameter to specifiy including ({\tt excludeNodesInFile=FALSE}) only those text file nodes (and their interactions) in the {\tt CCG} network object, or to (default setting) exclude those nodes from all interactions in the {\tt CCG}.

<<eval=FALSE>>=
CreateCCG(filename, nodeInclusionFile = 'NodesList.txt', 
                                    excludeNodesInFile = TRUE)
@

Note that both the influencing and influenced nodes will be searched against across all interactions within the input network. Individual interactions will be included or excluded if one (or both) node name(s) match against any specified within the node inclusion file.



\newpage

\begin{thebibliography}{9}

\bibitem{submitted} Bradley, G. and Barrett, S.J. \textit{(submitted)}
\textit{CausalR}: extracting mechanistic sense from genome scale data. Bioinformatics, application note. 

\bibitem{2012a} Chindelevitch, L. \textit{et al.} (2012a)
Causal reasoning on biological networks: interpreting transcriptional changes. Bioinformatics, 28(8):1114-21.

\bibitem{2012b} Chindelevitch, L. \textit{et al.} (2012b)
Assessing statistical significance in causal graphs (Methodology article). BMC Bioinformatics, 13:35- 48.

\bibitem{Fisher} Fischer, R. A. (1970)
Statistical Methods for Research Workers. Oliver \& Boyd.

\bibitem{GOEAST} Zheng, Q. and Wang, X-J. (2008)
GOEAST: a web-based software toolkit for Gene Ontology enrichment analysis. Nucleic Acids Research, 36, W358--W363. 

\end{thebibliography}


\newpage
\begin{appendices}
%\appendix
%\section*{Appendices}
%\addcontentsline{toc}{section}{Appendices}
\renewcommand{\thesubsection}{\Alph{subsection}}

\subsection{Command Sequence Listing for Examples}
 
<<eval=FALSE>>=
# Set-up
library(CausalR)
library(igraph)

# Load network, create CG and plot
cg <- CreateCG('testNetwork1.sif')

PlotGraphWithNodeNames(cg)
@

<<results='hide'>>=
# Load network, create CCG and plot
ccg <- CreateCCG(system.file( "extdata", "testNetwork1.sif", 
    package="CausalR"))
@
<<eval=FALSE>>=
PlotGraphWithNodeNames(ccg)
@

<<results='hide'>>=
# Load experimental data
experimentalData <- ReadExperimentalData(system.file( "extdata",
                        "testData1.txt", package="CausalR"),ccg)
@

<<results='hide'>>=
# Make predictions for all hypotheses, with pathlength set to 2.
RankTheHypotheses(ccg, experimentalData, 2)
@


<<eval=FALSE>>=
# Make predictions for all hypotheses, running in parallel
# NOTE: this requires further set-up as detailed in Appendix B.
RankTheHypotheses(ccg,experimentalData,delta,doParallel=TRUE)
@

<<results='hide'>>=
# Make predictions for a single node (results for + and - 
# hypotheses for the node will be generated),
RankTheHypotheses(ccg, experimentalData,2,listOfNodes='Node0')
@

<<results='hide'>>=
# Make predictions for an arbitrary list of nodes (gives results
# for up- and down-regulated hypotheses for each named node),
testlist <- c('Node0','Node3','Node2')
RankTheHypotheses(ccg, experimentalData,2,listOfNodes=testlist)
@

<<results='hide'>>=
# An example of making predictions for a particular signed hypo-
# -thesis at delta=2, for up-regulated node0, i.e.node0+.
# (shown to help understanding of hidden functionality)
predictions<-MakePredictionsFromCCG('Node0',+1,ccg,2)
GetNodeName(ccg,CompareHypothesis(predictions,experimentalData))
@

<<results='hide'>>=
# Scoring the hypothesis predictions
ScoreHypothesis(predictions,experimentalData)
@

<<eval=FALSE>>=
# Compute statistics required for Calculating Significance 
# p-value
Score<-ScoreHypothesis(predictions,experimentalData)
CalculateSignificance(Score, predictions, experimentalData)
PreexperimentalDataStats <-
    GetNumberOfPositiveAndNegativeEntries(experimentalData)

    #this gives integer values for n_+ and n_- for the 
    #experimental data,as shown in Table 2.

    PreexperimentalDataStats

    # add required value for n_0, number of non-differential
    # experimental results,
    experimentalDataStats<-c(PreexperimentalDataStats,1)
    # then use,
    AnalysePredictionsList(predictions,8)
    # ...to output integer values q_+, q_- and q_0 for
    #    significance calculations (see Table 2)
    # then store this in the workspace for later use,
    predictionListStats<-AnalysePredictionsList(predictions,8)
@

<<eval=FALSE>>=
# Compute Significance p-value using default cubic algorithm
CalculateSignificance(Score,predictionListStats, 
    experimentalDataStats, useCubicAlgorithm=TRUE)
# or simply,
CalculateSignificance(Score,predictionListStats, 
    experimentalDataStats)
# as use cubic algorithm is the default setting.
@

<<eval=FALSE>>=
# Compute Significance p-value using default quartic algorithm
CalculateSignificance(Score,predictionListStats, 
                experimentalDataStats,useCubicAlgorithm=FALSE)
@

<<eval=FALSE>>=
# Compute enrichment p-value
CalculateEnrichmentPvalue(predictions, experimentalData)
@

<<eval=FALSE>>=
# Running SCAN whilst excluding scoring of hypotheses for non-
# -differential nodes
AllData<-read.table(file="testData1.txt", sep="\t")
DifferentialData<-AllData[AllData[,2]!=0,]
write.table(DifferentialData, file="DifferentialData.txt", 
    sep="\t",row.names=FALSE, col.names=FALSE, quote=FALSE )

runSCANR(ccg, ReadExperimentalData("DifferentialData.txt", ccg), 
        NumberOfDeltaToScan=3, topNumGenes=100, 
        correctPredictionsThreshold=3)
@

<<eval=FALSE>>=
# Indirect Individual Hypothesis Network Generation (after running SCAN)
WriteExplainedNodesToSifFile("Node1", +1,ccg,experimentalData,delta=2)
@

<<eval=FALSE>>=
# Indirect Network Generation for All Hypotheses (after running SCAN)
scanResults <- runSCANR(ccg, experimentalData, numberOfDeltaToScan=2, 
  topNumGenes=4,correctPredictionsThreshold=1,
  writeResultFiles = FALSE, writeNetworkFiles = "none",quiet=FALSE)
WriteAllExplainedNodesToSifFile(scanResults, ccg, experimentalData, 
  delta=2, correctlyExplainedOnly = TRUE, quiet = TRUE)
@

<<eval=FALSE>>=
# Direct Network Generation for All Hypotheses (whilst running SCAN)
runSCANR(ccg, experimentalData, numberOfDeltaToScan=2, 
  topNumGenes=4,correctPredictionsThreshold=1,quiet=TRUE,
  writeResultFiles = TRUE, writeNetworkFiles = "correct")
@

\newpage

\subsection{Speeding-up CausalR Processing}

The runtimes for {\tt RankTheHypotheses()} increase with the size of network and experimental input. This can be offset via the use of parallelisation or the R (JIT) byte code compiler (no additional package installations are needed to use these as they are integral to the latest versions of base R). Note that run times will not be improved for very small examples where the cost of setting-up paralleisation or JIT outweighs its benefit. \\

Parallelisation is recommended for architectures featuring a multicore processor(s), whilst JIT is advised for older single or dual core machines.\\

Running {\tt RankTheHypotheses()} in parallel : \\

Parallel processing can be activated via the {\tt doParallel} flag, 

<<eval=FALSE>>=
RankTheHypotheses(ccg,experimentalData,delta,doParallel=TRUE)
@

By default the function will attempt to detect the number of available processer cores and use all of them bar one. \\

Alternatively, the number of cores to use can be set directly via the {\tt numCores} flag,

<<eval=FALSE>>=
RankTheHypotheses(ccg,experimentalData,delta,
                                  doParallel=TRUE, numCores=3)
@

To get the number of processor cores on their system:\\

\textit{Linux} users can use the {\tt lscpu} or {\tt nproc -all} commands. In the absence of user privileges for these, try {\tt grep processor /proc/cpuinfo}. The Linux top command can be used to check existing CPU utilisation.\\

\textit{Windows} users can open a DOS window and type,\\

{\tt WMIC CPU Get DeviceID,NumberOfCores,NumberOfLogicalProcessors} \\ 

or {\tt WMIC CPU Get /Format:List} can be used. \\

Alternatively, they can look in the 'CPU Usage History' under the Windows Task manager 'Performance' tab. \\


Running {\tt RankTheHypotheses()} with JIT compilation : \\

In order to activate the R byte code compiler functionality, R must be started with an additional flag command, R\textunderscore COMPILE\textunderscore PACKAGES=1. This can be done from the \textit{Linux} or \textit{Windows} command line when invoking R. \\

Alternatively, prior to starting R from the desktop icon on a \textit{Windows} machine, place the R\textunderscore COMPILE\textunderscore PACKAGES=1 after the path to Rgui.exe in the \textit{Target} field, under the \textit{Shortcut} tab of the R icon \textit{Properties} (accessed by right mouse click on the R icon). \\


Upon starting R, from double clicking the icon, the commands to use prior to loading \textit{CausalR} are:

<<>>=
library(compiler)
enableJIT=3
@

When set to greater than zero, {\tt enableJIT} invokes Just-In-Time (JIT) compilation, whilst {\tt $enableJIT=0$} disables it. JIT can also be enabled by starting R with the environment variable {\tt R\_ENABLE\_JIT} set to 1, 2 or 3, depending upon the level required. \textit{CausalR} works with the level set to 3. \\

For further details see, \textit{http://www.inside-r.org/r-doc/compiler/compile} \\ 
or, alternatively, type $??compiler::compile$ at the R command prompt to pull up the relevant R help page.\\

\noindent The JIT compiler compiles the package the first time it is run. Therefore, in order to achieve speed-up, the user needs to run a small \textit{CausalR} example prior to running a very large one. JIT can also be used for medium sized inputs where the user wishes to run the computationally-expensive exact p-value calculation algorithm. \\

Note: when running very large networks/input the memory available to R may need reconfiguration (this is explained in the R documentation, but one easy way is to start-up R with the command line flag, -max-mem-size=????M, where ???? is a value in KB such that 4000M = 4MB (this can also be inserted in the \textit{Windows} icon \textit{Target} field, see above). Use of 64bit R is also recommended in order to be able to address sufficient memory. 

\end{appendices}
\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CheckPossibleValuesAreValid.r
\name{CheckPossibleValuesAreValid}
\alias{CheckPossibleValuesAreValid}
\title{check possible values are valid}
\usage{
CheckPossibleValuesAreValid(predictionDataStats, experimentalDataStats,
  possibleValues)
}
\arguments{
\item{predictionDataStats}{a vector of predicted results}

\item{experimentalDataStats}{a vector of observed experimental results}

\item{possibleValues}{a vector of possible values  n++, n+-, n-+ and n--}
}
\value{
TRUE if and only if the given vector of possible values is valid
}
\description{
Checks if the a given set of possible values for n++, n+-, n-+ and n-- are agree with the predicted and experimental
data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateWeightGivenValuesInThreeByThreeContingencyTable.r
\name{CalculateWeightGivenValuesInThreeByThreeContingencyTable}
\alias{CalculateWeightGivenValuesInThreeByThreeContingencyTable}
\title{calculate weight given values in three-by-three contingency table}
\usage{

  CalculateWeightGivenValuesInThreeByThreeContingencyTable(threeByThreeContingencyTable,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{threeByThreeContingencyTable}{a 3x3 contingency table}

\item{logOfFactorialOfPredictionListStats}{log of Factorial of prediction statistics}

\item{returnlog}{should the result be returned as a log value. Default is FALSE.}
}
\value{
a D-value (or weight)
}
\description{
Given the values in the three by three contingency table and the values of the number of positive/negative/non-significant predictions (q+, q-, q0) this function calculates the D-value (or weight).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetApproximateMaximumDValueFromTwoByTwoContingencyTable.r
\name{GetApproximateMaximumDValueFromTwoByTwoContingencyTable}
\alias{GetApproximateMaximumDValueFromTwoByTwoContingencyTable}
\title{computes an approximate maximum D value or weight}
\usage{
GetApproximateMaximumDValueFromTwoByTwoContingencyTable(n_pp, n_pm, n_mp, n_mm,
  predictionListStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{n_pp}{the count n++ from the prediction-observation contingency matrix}

\item{n_pm}{the count n+- from the prediction-observation contingency matrix}

\item{n_mp}{the count n-+ from the prediction-observation contingency matrix}

\item{n_mm}{the count n-- from the prediction-observation contingency matrix}

\item{predictionListStats}{a vector containing the values q+, q- and q0: the number of positive, negative, non-significant/contradictory predictions}

\item{experimentalDataStats}{a vector containing the values n+, n- and n0: the number of positive, negative, non-significant/contradictory observations}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial value for each entry in predictionListStats}

\item{returnlog}{return the result as a log, default is FALSE}
}
\value{
the maximum D value or weight
}
\description{
Computes an approximate maximum D value (or weight).
The calculation is  approximate since only the first valid D value that is round. This has been
done to speed up the overall algorithm - to get the exact answer use GetMaximumDValueFromTwoByTwoContingencyTable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReadExperimentalData.r
\name{ReadExperimentalData}
\alias{ReadExperimentalData}
\title{read experimental data}
\usage{
ReadExperimentalData(fileName, network, removeDuplicates)
}
\arguments{
\item{fileName}{a file containing the experimental data (text file format)}

\item{network}{a (Computational) Causal Graph, as an igraph.}

\item{removeDuplicates}{Optional, defaults to true. Remove duplicated nodes the experimental file (i.e. where the result for a node is repeated, use the first value given only; the alternative is to return a result which contains multiple rows for this node).}
}
\value{
(n x 2) matrix of nodes and direction of regulation. The first column of the matrix contains the node IDs from the network, and the second contains the experimental values.
}
\description{
Reads experimental data for the causal reasoning algorithm from a text file.
}
\examples{
#get path to example network file
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
##create ccg
ccg <- CreateCCG(network)
#get path to example experimental data
fileName<- system.file(package='CausalR', 'extdata', 'testData.txt')
ReadExperimentalData(fileName, ccg)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetNumberOfPositiveAndNegativeEntries.r
\name{GetNumberOfPositiveAndNegativeEntries}
\alias{GetNumberOfPositiveAndNegativeEntries}
\title{counts the number of positive and negative entries}
\usage{
GetNumberOfPositiveAndNegativeEntries(dataList)
}
\arguments{
\item{dataList}{an array or dataframe in which the second column is numeric}
}
\value{
a vector of two components, the first of which giving the number of +1 entries, the second the number of -1's.
}
\description{
Counts the number of entries in the in the second column of an input table that are +1 or -1.
}
\examples{
expData<-read.table(system.file(package='CausalR', 'extdata', 'testData.txt'))
GetNumberOfPositiveAndNegativeEntries(expData)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateTotalWeightForAllContingencyTables.r
\name{CalculateTotalWeightForAllContingencyTables}
\alias{CalculateTotalWeightForAllContingencyTables}
\title{calculate total weight for all contingency tables}
\usage{
CalculateTotalWeightForAllContingencyTables(experimentalDataStats,
  returnlog = FALSE)
}
\arguments{
\item{experimentalDataStats}{a vector containing the values n+, n- and n0, the number of positive/negative/non-significant (or contradictory) transcripts in the results}

\item{returnlog}{whether the result should be returned as a log. Default is FALSE.}
}
\value{
a D-value or weight
}
\description{
Calculates the total weights or D-values for all possible  contingency tables. This value can be  used to calculate the p-value
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateSignificanceUsingCubicAlgorithm1b.r
\name{CalculateSignificanceUsingCubicAlgorithm1b}
\alias{CalculateSignificanceUsingCubicAlgorithm1b}
\title{Calculate Significance Using Cubic Algorithm}
\usage{
CalculateSignificanceUsingCubicAlgorithm1b(hypothesisScore, predictionListStats,
  experimentalDataStats, epsilon)
}
\arguments{
\item{hypothesisScore}{The score whose p-value we want to find.}

\item{predictionListStats}{Number of predicted up-regulated, 
predicted down-regulated and ambiguous predictions.}

\item{experimentalDataStats}{Number of up-regulated, down-regulated and not 
significantly changed transcripts in the experimental data.}

\item{epsilon}{The threshold that is used when calculating the p-value 
using the cubic algorithm. (Defaults to 1e-5, only used for the cubic 
algorithm, ignored if useCubicAlgorithm is FALSE.)}
}
\value{
p value
}
\description{
Calculate the p-value of a score given the hypothesis score and the distribution table (calculated using the cubic algorithm  1b in Assessing statistical significance in causal graphs - Chindelevitch et al)
}
\examples{
CalculateSignificance(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
CalculateSignificanceUsingCubicAlgorithm1b(5, c(7,4,19), c(6,6,18), 1e-5)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputeFinalDistribution.r
\name{ComputeFinalDistribution}
\alias{ComputeFinalDistribution}
\title{compute final distribution}
\usage{
ComputeFinalDistribution(resultsMatrix)
}
\arguments{
\item{resultsMatrix}{a matrix containing the scores and weights from which the distribution is to be calculated}
}
\value{
distributionMatrix  a matrix containing the reference distribution for the score
}
\description{
Computes a final reference distribution of the score used to compute the final p-value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetMaxDValueForAThreeByTwoFamily.r
\name{GetMaxDValueForAThreeByTwoFamily}
\alias{GetMaxDValueForAThreeByTwoFamily}
\title{get maximum D value for three-by-two a family}
\usage{
GetMaxDValueForAThreeByTwoFamily(r_p, r_m, r_z, n_p, n_m, predictionListStats,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{r_p}{a r+ row sum from the prediction-observation matrix}

\item{r_m}{a  r- row sum from the prediction-observation matrix}

\item{r_z}{a  r0 row sum from the prediction-observation matrix}

\item{n_p}{a number of predicted increases from the prediction-observation matrix}

\item{n_m}{a number of predicted decreases  from the prediction-observation matrix}

\item{predictionListStats}{a vector contain the number of postive, negative and non-significant/contradictory predictions: q+, q- and q0.}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial for each element in the predictionListStats object}

\item{returnlog}{whether or not the maximum D value should be returned as a log (TRUE). Otherwise a non-logged value is returned.}
}
\value{
Maximum D_fam Value
}
\description{
Returns the maximum D value for a particular family as described as D_fam 
 on pages 6 and 7 of  Assessing Statistical Significance of Causal Graphs in Assessing Statistical Signifcance of Causal Graphs
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetScoresForSingleNode.r
\name{GetScoresForSingleNode}
\alias{GetScoresForSingleNode}
\title{Get scores for single node}
\usage{
GetScoresForSingleNode(iNode, timeToRunSoFar, nodesToBeTested, network, delta,
  processedExperimentalData, numPredictions, epsilon, useCubicAlgorithm,
  use1bAlgorithm, symmetricCCG, correctPredictionsThreshold,
  experimentalDataStats, quiet)
}
\arguments{
\item{iNode}{this node}

\item{timeToRunSoFar}{the time to run so far}

\item{nodesToBeTested}{List of all nodes to be tested}

\item{network}{Computational Causal Graph, as an igraph.}

\item{delta}{Distance to search within the causal graph.}

\item{processedExperimentalData}{The processed experimental data}

\item{numPredictions}{The number of predictions}

\item{epsilon}{The threshold that is used when calculating the p-value using the cubic algorithm (see 'Assessing statistical significance in causal graphs').}

\item{useCubicAlgorithm}{An indicator specifying which algorithm will be used to calculate the p-value. The default is set as useCubicAlgorithm = TRUE which uses the cubic algorithm. If this value is set as FALSE, the algorithm will use the much slower quartic algorithm which does compute the exact answer, as opposed to using approximations like the cubic algorithm.}

\item{use1bAlgorithm}{An indicator specifying whether the 1a or 1b (default, faster) variant of the cubic algorithm described in Chindelevitch's paper will be used to calculate the p-value.}

\item{symmetricCCG}{This flag specifies whether the CCG is assumed to be symmetric. The value is set as TRUE as a default. If this is the case the running time of the algorithm is reduced since the negative node values can be calculated using symmetry and the results of calculations performed for the positive node}

\item{correctPredictionsThreshold}{A threshold on the number of correct predictions for a given hypothesis. If a hypothesis produces fewer correct predictions than predictionsThreshold then the algorithm will not calculate the two p-values. Instead 'NA' will be displayed in the final two columns of the corresponding row of the results table. As a default correctPredictionsThreshold is set as -Inf, so that the p-values are calculated for all specified hypotheses. Note: Set to Inf to turn off p-value calculations entirely.}

\item{experimentalDataStats}{Stats from the experimental data}

\item{quiet}{a flag to supress progress output}
}
\value{
If symmetricCCG is false, this returns a single line of the scoreMatrix for the 'iNode'th node in nodesToBeTested. If symmetricCCG is true this returns two lines. The first of which corresponds to the positive node and the second the negative node.
}
\description{
A helper function for RankTheHypotheses to calculate a line of the scoresMatrix table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ValidateFormatOfDataTable.r
\name{ValidateFormatOfDataTable}
\alias{ValidateFormatOfDataTable}
\title{validate format of the experimental data table}
\usage{
ValidateFormatOfDataTable(dataTable)
}
\arguments{
\item{dataTable}{the data table to be tested}
}
\value{
true if the data table is valid
}
\description{
Checks the format of the experimental data. This is expected to be two columns, the first containing the gene name and the second the direction of regulation, -1, 0 or 1. The function checks the number of columns and the values of the second column,
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetScoresWeightsMatrix.r
\name{GetScoresWeightsMatrix}
\alias{GetScoresWeightsMatrix}
\title{get scores weight matrix}
\usage{
GetScoresWeightsMatrix(matrixOfPossibleValues, predictionDataStats,
  experimentalDataStats, logOfFactorialOfPredictionListStats)
}
\arguments{
\item{matrixOfPossibleValues}{values of n++, n+-, n-+ and n-- that need to be assessed}

\item{predictionDataStats}{a table of predicions}

\item{experimentalDataStats}{a table of observed experimental data}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial value for
each entry in predictionListStats}
}
\value{
a matrix containing scores and logs of the weights
}
\description{
Computes the score and weight for a network/set of experimental data based on the table containing possible values of n++, n+-, n-+ and n--.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetSetOfDifferentiallyExpressedGenes.r
\name{GetSetOfDifferentiallyExpressedGenes}
\alias{GetSetOfDifferentiallyExpressedGenes}
\title{get set of differientially expressed genes}
\usage{
GetSetOfDifferentiallyExpressedGenes(results)
}
\arguments{
\item{results}{a table of results}
}
\value{
a matrix of differentially expressed genes
}
\description{
Gets the set of differentially expressed genes in the results, G+ as defined by in Causal reasoning on biological networks: Interpreting transcriptional changes,  L Chindelevitch et al.
}
\references{
L Chindelevitch et al.
Causal reasoning on biological networks: Interpreting transcriptional changes.
Bioinformatics, 28(8):1114-21, 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetWeightsAboveHypothesisScoreForAThreeByTwoTable.r
\name{GetWeightsAboveHypothesisScoreForAThreeByTwoTable}
\alias{GetWeightsAboveHypothesisScoreForAThreeByTwoTable}
\title{updates weights for contingency table and produce values for p-value calculation}
\usage{
GetWeightsAboveHypothesisScoreForAThreeByTwoTable(weights, r_p, r_m, r_z, n_p,
  n_m, predictionListStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax)
}
\arguments{
\item{weights}{Weights}

\item{r_p}{the row sum r+}

\item{r_m}{the row sum r-}

\item{r_z}{the row sum r0}

\item{n_p}{the column sum n+}

\item{n_m}{the column sum n-}

\item{predictionListStats}{a list of prediction statistics}

\item{experimentalDataStats}{the observed experimental data}

\item{logOfFactorialOfPredictionListStats}{log factorial's of prediction list stats}

\item{hypothesisScore}{the hypothesis score to be considered}

\item{logepsDMax}{log of epsilon logD Maximum value}

\item{logDMax}{a logD Maximum value}
}
\value{
a vector containing the hypothesis score and the total weight
}
\description{
Finds the D-Values (weights) from any 3x2 contingency tables 
that have a score above and including the hypothesis score. It also calculates the
total weight, and returns a 2x1 vector of the two values. The ratio of these values is the p-value.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetAllPossibleRoundingCombinations.r
\name{GetAllPossibleRoundingCombinations}
\alias{GetAllPossibleRoundingCombinations}
\title{get score for numbers of correct and incorrect predictions}
\usage{
GetAllPossibleRoundingCombinations(twoByTwoContingencyTable)
}
\arguments{
\item{twoByTwoContingencyTable}{Approximate values of n++, n+-, n-+ and n--, these values are
calculated to optimise the D-value (see page 6 of Assessing
statistical significance of causal graphs)}
}
\value{
a matrix of rounding combinations
}
\description{
Returns all possible rounding combinations of a 2x2 table. 
Given the values of n++, n+-, n-+ and n-- (stored in twoByTwoContingencyTable) this function will
compute all possibilities of rounding each value up or down.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PopulateTheThreeByThreeContingencyTable.r
\name{PopulateTheThreeByThreeContingencyTable}
\alias{PopulateTheThreeByThreeContingencyTable}
\title{populate the three-by-three contingency table}
\usage{
PopulateTheThreeByThreeContingencyTable(n_pp, n_pm, n_mp, n_mm,
  predictionDataStats, experimentalDataStats)
}
\arguments{
\item{n_pp}{n++ contingency table entry}

\item{n_pm}{n+- contingency table entry}

\item{n_mp}{n-+ contingency table entry}

\item{n_mm}{n-- contingency table entry}

\item{predictionDataStats}{a prediction data table.}

\item{experimentalDataStats}{an experimental data table}
}
\value{
Vector of calculated values for n0+, n0-, n+0, n-0 and n00 - See: Chindelevitch et al.Bioinformatics (2012).
}
\description{
Populates 3x3 signed contingency table of expected versus observed changes.
Given the values of n++, n+-, n-+ and n--, calculates n0+, n0-, n+0, n-0 and n00. 
Notation from Chindelevitch et al. Causal reasoning on biological networks 
Bioinformatics (2012) paper.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ProcessExperimentalData.r
\name{ProcessExperimentalData}
\alias{ProcessExperimentalData}
\title{process experimental data}
\usage{
ProcessExperimentalData(experimentalData, network)
}
\arguments{
\item{experimentalData}{input experimental data.}

\item{network}{an input interaction network.}
}
\value{
processed experimental data formatted ready for scoring
}
\description{
Processes experimental data to get it into the correct form
for scoring. The node names that are read in as strings acquire 
an internal id when the network is created. This function will 
replace the node name with its id.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RemoveIDsNotInExperimentalData.r
\name{RemoveIDsNotInExperimentalData}
\alias{RemoveIDsNotInExperimentalData}
\title{remove IDs not in experimental data}
\usage{
RemoveIDsNotInExperimentalData(connectedNodes, nodesInExperimentalData)
}
\arguments{
\item{connectedNodes}{a list of connected nodes}

\item{nodesInExperimentalData}{a list of nodes in the experimental data}
}
\value{
connectedNodesInExperimentalData a list of connected nodes with the redundant nodes removed
}
\description{
Takes in a list of connected nodes and removes those not in the experimental data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetScoresWeightsMatrixByCubicAlg.r
\name{GetScoresWeightsMatrixByCubicAlg}
\alias{GetScoresWeightsMatrixByCubicAlg}
\title{get scores weights matrix by the cubic algorithm}
\usage{
GetScoresWeightsMatrixByCubicAlg(predictionListStats, experimentalDataStats,
  epsilon)
}
\arguments{
\item{predictionListStats}{a vector containing the values q+, q- and q0}

\item{experimentalDataStats}{a vector containing the values n+, n- and n0}

\item{epsilon}{the algorithms tolerance epsilon}
}
\value{
a matrix containing the ternary dot product distribution
}
\description{
Implements the cubic algorithm as described on pages 6 and 7 of Assessing statistical significance in causal graphs,
Chindelevitch et al. 2012
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DetermineInteractionTypeOfPath.r
\name{DetermineInteractionTypeOfPath}
\alias{DetermineInteractionTypeOfPath}
\title{determine interaction type of path}
\usage{
DetermineInteractionTypeOfPath(network, nodesInPath)
}
\arguments{
\item{network}{an igraph representing the network}

\item{nodesInPath}{an ordered list of the nodes visited on the path - note that these contain numbers which use 
R's internal reference to the edges}
}
\value{
a signed integer representing the paths sign
}
\description{
Determines the sign of a given path.
Given a path and through the network, this function will determine if the path results in activation or inhibition.
Activation is indicated by 1, inhibition by -1
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetRowAndColumnSumValues.r
\name{GetRowAndColumnSumValues}
\alias{GetRowAndColumnSumValues}
\title{get row and column sum values}
\usage{
GetRowAndColumnSumValues(predictionListStats, experimentalResultStats)
}
\arguments{
\item{predictionListStats}{a vector containing the number of postive, negative, or non-signficant/contradictory predictions (q+, q- and q0)}

\item{experimentalResultStats}{a vector containing the number of postive, negative, or non-signficant/contradictory observations (n+, n- and n0)}
}
\value{
a matrix of row and sum values r+, r-, c+ and c-
}
\description{
Returns the possible values of r+, r-, c+ and c- (the column and row sum values) following page 6 of Assessing
statistical  significance in causal graphs (Chindelevitch et. al)
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetMaximumDValueFromTwoByTwoContingencyTable.r
\name{GetMaximumDValueFromTwoByTwoContingencyTable}
\alias{GetMaximumDValueFromTwoByTwoContingencyTable}
\title{get maximum D value from two-by-two contingency table}
\usage{
GetMaximumDValueFromTwoByTwoContingencyTable(twoByTwoContingencyTable,
  predictionListStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{twoByTwoContingencyTable}{approximate values of n++, n+-, n-+ and n--, these values arecalculated to optimise the D-value}

\item{predictionListStats}{a vector containing the values q+, q- and q0 the number of positive/negative/non-significant (or contradictory) predictions)}

\item{experimentalDataStats}{a vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or contradictory) transcripts in the results)}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial value for each entry in predictionListStats}

\item{returnlog}{whether or not he value should be returned as a log (TRUE) or not (FALSE)}
}
\value{
the maximal D-value
}
\description{
Computes the maximum D value (or weight) given approximate values of n++, n+-, n-+ and n--.
These values are approximate and in general are non-integer values; they are found by using an 
approximation detailed in the paper Assessing statistical significance in causal graphs on page 6 i.e. n_ab is approximately
equal to q_a*n_b/t where a and b are either +, - or 0. The value is an approximation since the direction in which the 
number should be rounded is not clear and hence this function runs through all possible combinations of rounding before
concluding the maximum D-value.
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CausalR-package.r
\docType{package}
\name{CausalR-package}
\alias{CausalR}
\alias{CausalR-package}
\title{The CausalR package}
\description{
Causal network analysis methods for regulator prediction and network reconstruction from genome scale data.
}
\details{
The most important functions are:
\itemize{
    \item{\code{\link{CreateCCG}}: read a computational causal graph from a .sif file}
    \item{\code{\link{ReadExperimentalData}}: read a experimental data from a .txt file}
    \item{\code{\link{MakePredictions}}: make causal reasoning predictions from a CCG}
    \item{\code{\link{ScoreHypothesis}}: score causal reasoning predictions}
    \item{\code{\link{CalculateSignificance}}: calculate statisitical significance of a result}
    \item{\code{\link{RankTheHypotheses}}: compare different possible regulatory hypotheses on a single CCG}
    \item{\code{\link{runSCANR}}: reduce false positives by selecting common hypotheses across pathlengths}
    \item{\code{\link{WriteExplainedNodesToSifFile}}: reconstruct hypothesis specific regulatory network}
}
}
\author{
{
    Glyn Bradley, Steven J. Barrett, Chirag Mistry, Mark Pipe, David Riley, David Wille, Bhushan Bonde, Peter Woollard
}
}
\references{
\itemize{
    \item{"CausalR - extracting mechanistic sense from genome scale data", Bradley, G. and Barrett, S.J., Application note, Bioinformatics (\emph{submitted})}
    \item{"Causal reasoning on biological networks: interpreting transcriptional changes", Chindelevitch \emph{et al.}, Bioinformatics \bold{28} 1114 (2012). doi:\href{http://dx.doi.org/10.1093/bioinformatics/bts090}{10.1093/bioinformatics/bts090}}
    \item{"Assessing statistical significance in causal graphs", Chindelevitch \emph{et al.}, BMC Bioinformatics \bold{13} 35 (2012). doi:\href{http://dx.doi.org/10.1186/1471-2105-13-35}{10.1186/1471-2105-13-35}}
}
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetSetOfSignificantPredictions.r
\name{GetSetOfSignificantPredictions}
\alias{GetSetOfSignificantPredictions}
\title{get set of significant predictions}
\usage{
GetSetOfSignificantPredictions(predictions)
}
\arguments{
\item{predictions}{a table of predictions}
}
\value{
a matrix of positive and negative predictions
}
\description{
Gets the set of positive and negative predictions, the combination of the sets Sh+ and Sh- in Causal reasoning on biological networks: Interpreting transcriptional changes,  L Chindelevitch et al.
}
\references{
L Chindelevitch et al.
Causal reasoning on biological networks: Interpreting transcriptional changes.
Bioinformatics, 28(8):1114-21, 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetShortestPathsFromCCG.r
\name{GetShortestPathsFromCCG}
\alias{GetShortestPathsFromCCG}
\title{get shortest paths from CCG}
\usage{
GetShortestPathsFromCCG(network, hypothesisnode, targetnode,
  showbothdirs = FALSE, quiet = FALSE)
}
\arguments{
\item{network}{built from iGraph}

\item{hypothesisnode}{hypothesis node ID}

\item{targetnode}{target node ID}

\item{showbothdirs}{where multiple paths from a positive and negative node, FALSE returns only the shortest. Otherwise both are returned.}

\item{quiet}{a flag to suppress output to console. FALSE by default.}
}
\value{
a list of vectors containing the nodes of individual paths
}
\description{
Gets the node names in the shortest path from one node in a CCG to another
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg = CreateCCG(network)
hypothesisnode = 1
targetnode = 10
GetShortestPathsFromCCG (ccg, hypothesisnode, targetnode)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetMatrixOfCausalRelationships.r
\name{GetMatrixOfCausalRelationships}
\alias{GetMatrixOfCausalRelationships}
\title{compute causal relationships matrix}
\usage{
GetMatrixOfCausalRelationships(hypothesis, network,
  idsOfConnectedNodesFromSubgraph)
}
\arguments{
\item{hypothesis}{a hypothesis node}

\item{network}{a CCG network}

\item{idsOfConnectedNodesFromSubgraph}{a list of connected nodes in the subgraph of interest}
}
\value{
causal relationships matrix
}
\description{
Get a matrix of causal relationships from the network and the IDs of connected nodes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MakePredictions.r
\name{MakePredictions}
\alias{MakePredictions}
\title{make predictions}
\usage{
MakePredictions(hypothesisnode, signOfHypothesis, network, delta,
  nodesInExperimentalData = NULL)
}
\arguments{
\item{hypothesisnode}{the node in the causal graph from which predictions should be made.  Can be either a (numerical) node ID or a (string) node name.}

\item{signOfHypothesis}{whether the hypothesis node is up- or down-regulated. Should be +1 or -1.}

\item{network}{a (Computational) Causal Graph, as an igraph.}

\item{delta}{the distance to search within the causal graph.}

\item{nodesInExperimentalData}{optional. Nodes to include in the output. Should be a list of node IDs.}
}
\value{
a matrix of predictions for the given particular hypothesis
}
\description{
Creates a matrix of predictions for a particular hypothesis.
The output is an array containing the relationship between each node and the hypothesis.
The hypothesis provided will be the vertex id of one of the nodes in the network
(as an integer node ID or name, including + or - for up/down regulation in the case of a CCG).
The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation.
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg <- CreateCCG(network)
predictions <- MakePredictions('NodeA', +1, ccg, 2)
}
\concept{
CausalR
}

% Generated by roxygen2 (4.1.0): do not edit by hand
% Please edit documentation in R/runRankHypothesis.r
\name{runRankHypothesis}
\alias{runRankHypothesis}
\title{run rank the hypothesis}
\usage{
runRankHypothesis(PPInet, Expressiondata, delta, correctPredictionsThreshold)
}
\arguments{
\item{PPInet}{PPInet is the PPI interaction file}

\item{Expressiondata}{observed gene expression data}

\item{delta}{the number of links to follow from any hypothesis no. Dedepending on network size/topology, this value typically ranges between 1 and 5}

\item{correctPredictionsThreshold}{Minimal score for p-values calculation. Hypotheses with scores below this value will get NAs for p-value and enrichment p-value. The usual default is -inf within the RankTheHypotheses function, where it is employed.}
}
\value{
rankedHypothesis table of results produced by the algorithm
}
\description{
A top level function that used to run CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetPathsInSifFormat.r
\name{GetPathsInSifFormat}
\alias{GetPathsInSifFormat}
\title{Get paths in Sif format}
\usage{
GetPathsInSifFormat(arrayOfPaths)
}
\arguments{
\item{arrayOfPaths}{an array of paths (in the format outputted by GetShortestPathsFromCCG) to be converted to .sif format}
}
\value{
network visualisation
}
\description{
Converts network paths into Simple interaction file (.sif) format for importing into Cytoscape
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnalysePredictionsList.r
\name{AnalysePredictionsList}
\alias{AnalysePredictionsList}
\title{analyse predictions list}
\usage{
AnalysePredictionsList(predictionsList, numPredictions)
}
\arguments{
\item{predictionsList}{list of predictions}

\item{numPredictions}{number of predictions}
}
\value{
prediction statistics
}
\description{
Taking the list of predictions from a particular hypothesis, counts the number of
positive and negative predictions in the list and the number of 0's (from numPredictions).
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg <- CreateCCG(network)
predictions <- MakePredictions('NodeA', +1, ccg, 2)
AnalysePredictionsList(predictions,8)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetNodeName.r
\name{GetNodeName}
\alias{GetNodeName}
\title{get node name}
\usage{
GetNodeName(network, nodeID, signed = FALSE)
}
\arguments{
\item{network}{Built from igraph}

\item{nodeID}{a node ID or a matrix containing node IDs in its first column}

\item{signed}{whether or not the node name should be signed. Setting this value to TRUE gives a signed name indicating whether the gene is up or down regulated in the network}
}
\value{
a node name or a vector of node names depending if the input is an matrix.
}
\description{
Returns the node name from one or more node IDs, or substitute node names for node IDs,
given in first column of a matrix typically of  predictions or experimental data
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg = CreateCCG(network)
nodeID <- 10
GetNodeName(ccg, nodeID)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AddWeightsToEdges.r
\name{AddWeightsToEdges}
\alias{AddWeightsToEdges}
\title{add weights to edges}
\usage{
AddWeightsToEdges(network, tableOfInteractions)
}
\arguments{
\item{network}{an igraph constructed from the original .sif file}

\item{tableOfInteractions}{a column of the corresponding .sif file indicating the direction of activation/interaction}
}
\value{
an augmented network
}
\description{
Adds weight information to the edges of given network (1 for activation and -1 for inhibition)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetNodeID.r
\name{GetNodeID}
\alias{GetNodeID}
\title{get CCG node ID}
\usage{
GetNodeID(network, nodename, direction = 1)
}
\arguments{
\item{network}{a CCG object}

\item{nodename}{the node name, or names, for which the ID is required}

\item{direction}{the direction of regulation of the required node or nodes. Maybe +1 (default) or -1.}
}
\value{
a scalar or vector containing the node ID or IDs requested
}
\description{
Returns the CCG node ID from a node name or a vector of node names and a given direction of regulation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetWeightForNumbersOfCorrectandIncorrectPredictions.r
\name{GetWeightForNumbersOfCorrectandIncorrectPredictions}
\alias{GetWeightForNumbersOfCorrectandIncorrectPredictions}
\title{get weight for numbers of correct and incorrect predictions}
\usage{
GetWeightForNumbersOfCorrectandIncorrectPredictions(n_pp, n_pm, n_mp, n_mm,
  predictionDataStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{n_pp}{the contingency table entry n++}

\item{n_pm}{the contingency table entry n+-}

\item{n_mp}{the contingency table entry n-+}

\item{n_mm}{the contingency table entry n--}

\item{predictionDataStats}{prediction data statistics}

\item{experimentalDataStats}{experimental data statistics}

\item{logOfFactorialOfPredictionListStats}{log of factorial of prediction list stats}

\item{returnlog}{true if the result should be returned as a log}
}
\value{
none
}
\description{
Gets the weight based on the values of n++, n+-, n-+ and n--.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ValidateFormatOfTable.r
\name{ValidateFormatOfTable}
\alias{ValidateFormatOfTable}
\title{validate format of table}
\usage{
ValidateFormatOfTable(dataTable)
}
\arguments{
\item{dataTable}{the table to be tested}
}
\value{
true if the test is satisfed.
}
\description{
Checks the format of the loaded in data. In particular exepects a table with threecolumns (in order) a initiating gene, an interaction ('Activates','Inhibits') and a responding gene and checks the number of rows and the values of the middle column.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FindIdsOfConnectedNodesInSubgraph.r
\name{FindIdsOfConnectedNodesInSubgraph}
\alias{FindIdsOfConnectedNodesInSubgraph}
\title{find Ids of connected nodes in subgraph}
\usage{
FindIdsOfConnectedNodesInSubgraph(idsOfConnectedNodes, subgraphOfConnectedNodes)
}
\arguments{
\item{idsOfConnectedNodes}{a list of connected nodes in the full graph}

\item{subgraphOfConnectedNodes}{a subgraph}
}
\value{
a list of connected nodes in the subgraph
}
\description{
Adds the IDs of the connected nodes in a subgraph to an existing list. 
Given the IDs of connected nodes in the full network, this function will find
the corresponding IDs in the subgraph
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetInteractionInformation.r
\name{GetInteractionInformation}
\alias{GetInteractionInformation}
\title{returns interaction information from input data}
\usage{
GetInteractionInformation(dataTable)
}
\arguments{
\item{dataTable}{a data table containing the information read in from the .sif file representing the network.}
}
\value{
a vector of interaction information
}
\description{
Gets the interaction information from the input data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RankTheHypotheses.r
\name{RankTheHypotheses}
\alias{RankTheHypotheses}
\title{rank the hypotheses}
\usage{
RankTheHypotheses(network, experimentalData, delta, epsilon = 1e-05,
  useCubicAlgorithm = TRUE, use1bAlgorithm = TRUE, symmetricCCG = TRUE,
  listOfNodes = NULL, correctPredictionsThreshold = -Inf, quiet = FALSE,
  doParallel = FALSE, numCores = NULL, writeFile = TRUE,
  outputDir = getwd())
}
\arguments{
\item{network}{Computational Causal Graph, as an igraph.}

\item{experimentalData}{The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.}

\item{delta}{Distance to search within the causal graph.}

\item{epsilon}{The threshold that is used when calculating the p-value using the cubic algorithm (see 'Assessing statistical significance in causal graphs').}

\item{useCubicAlgorithm}{An indicator specifying which algorithm will be used to calculate the p-value. The default is set as useCubicAlgorithm = TRUE which uses the cubic algorithm. If this value is set as FALSE, the algorithm will use the much slower quartic algorithm which does compute the exact answer, as opposed to using approximations like the cubic algorithm.}

\item{use1bAlgorithm}{An indicator specifying whether the 1a or 1b (default, faster) variant of the cubic algorithm described in Chindelevitch's paper will be used to calculate the p-value.}

\item{symmetricCCG}{This flag specifies whether the CCG is assumed to be symmetric. The value is set as TRUE as a default. If this is the case the running time of the algorithm is reduced since the bottom half of the table can be filled in using the results of calculations performed earlier.}

\item{listOfNodes}{A list of nodes specified by the user. The algorithm will only calculate and store the results for the nodes in the specified list. The default value is NULL; here the algorithm will calculate and store results for all the nodes in the network.}

\item{correctPredictionsThreshold}{A threshold on the number of correct predictions for a given hypothesis. If a hypothesis produces fewer correct predictions than predictionsThreshold then the algorithm will not calculate the two p-values. Instead 'NA' will be displayed in the final two columns of the corresponding row of the results table. As a default correctPredictionsThreshold is set as -Inf, so that the p-values are calculated for all specified hypotheses.}

\item{quiet}{a flag to suppress output to console. FALSE by default.}

\item{doParallel}{A flag for running RankTheHypothesis in parallel mode.}

\item{numCores}{Number of cores to use if using parallel mode. If the default value of NULL is used, it will attempt to detect the number of cores available and use all of them bar one.}

\item{writeFile}{A flag for determining if the output should be written to a file in the working directory. Default is TRUE.}

\item{outputDir}{the directory to output the files to. Default is the working directory}
}
\value{
A data frame containing the results of the algorithm.
}
\description{
Rank the hypotheses in the causal network. This function can be run with parallelisation using the doParallel flag.
}
\examples{
#get path to example network file
networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#create ccg
network <- CreateCCG(networkFile)
#get path to example experimental data
experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData.txt')
#read in experimetal data
experimentalData <- ReadExperimentalData(experimentalDataFile, network)
#run in single threaded mode
RankTheHypotheses(network, experimentalData, 2)
#run in parallel mode
RankTheHypotheses(network, experimentalData, 2, doParallel=TRUE, numCores=2)
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WriteExplainedNodesToSifFile.r
\name{WriteExplainedNodesToSifFile}
\alias{WriteExplainedNodesToSifFile}
\title{Write explained nodes to Sif file}
\usage{
WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network,
  experimentalData, delta, outputDir = getwd(), outputFilesName = "",
  correctlyExplainedOnly = FALSE, quiet = FALSE)
}
\arguments{
\item{hypothesisnode}{a hypothesis node}

\item{signOfHypothesis}{the direction of change of hypothesis node}

\item{network}{a computational causal graph}

\item{experimentalData}{The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.}

\item{delta}{the number of edges across which the hypothesis should be followed}

\item{outputDir}{the directory to output the files to. Default is the working directory}

\item{outputFilesName}{a character string to use for the name of the output files. Default value is "", which results in files using the default naming convention of "network file name-data file name-delta value-node name". Set to NA if not writing to file.}

\item{correctlyExplainedOnly}{if TRUE network files will only be produced for correctly explained nodes. If FALSE network files will be produced for each of correctly explained, incorrectly explained and ambiguously explained nodes.}

\item{quiet}{a flag to suppress output to console. FALSE by default.}
}
\value{
files containing paths from hypothesis node to explained nodes in sif format and corresponding annotation (_anno.txt) files
}
\description{
Outputs networks of explained nodes in .sif file format for producing visualisations using Cytoscape. Output will be to a
directory beginning with a timestamp,
}
\examples{
hypothesisnode <- "Node0"
signOfHypothesis <- +1
networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
network <- CreateCCG(networkFile)
experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
experimentalData <- ReadExperimentalData(experimentalDataFile, network)
delta <- 2
WriteExplainedNodesToSifFile(hypothesisnode, signOfHypothesis, network, experimentalData, delta,
                             outputFilesName=NA)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runSCANR.r
\name{runSCANR}
\alias{runSCANR}
\title{run ScanR}
\usage{
runSCANR(network, experimentalData, numberOfDeltaToScan = 5,
  topNumGenes = 150, correctPredictionsThreshold = Inf,
  writeResultFiles = TRUE, writeNetworkFiles = "all", doParallel = FALSE,
  numCores = NULL, quiet = FALSE, outputDir = getwd())
}
\arguments{
\item{network}{Computational Causal Graph, as an igraph.}

\item{experimentalData}{The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation.}

\item{numberOfDeltaToScan}{Iteratively scan for 1 to numberOfDeltaToScan delta values}

\item{topNumGenes}{A value to select top genes to report (typically top 100 genes)}

\item{correctPredictionsThreshold}{Minimal score for p-values calculation. Value is passed to RankTheHypothesis - scores below this value will get NAs for p-value and enrichment p-value. The default is Inf, so that no p-values are calculated.}

\item{writeResultFiles}{If set to TRUE the results of the scan will be written to two text files in the working directory. Default is TRUE.}

\item{writeNetworkFiles}{If set to "all" .sif files and corresponding _anno.txt files will be generated for the top correctly explained, incorrectly explained and ambiguously explained nodes. If set to "correct" they will only be calculated for correctly explained nodes. If set to "none", no networks will be generated. Default is "all".}

\item{doParallel}{A flag for running RankTheHypothesis in parallel mode. Default is FALSE.}

\item{numCores}{Number of cores to use if using parallel mode. If the default value of NULL is used, it will attempt to detect the number of cores available and use all of them bar one.}

\item{quiet}{a flag to suppress output to console. FALSE by default.}

\item{outputDir}{the directory to output the files to. Default is the working directory}
}
\value{
returns list of genes from each delta scan run
}
\description{
This function will return nodes regulated by the given hypothesisGene
}
\examples{
numberOfDeltaToScan <- 2
topNumGenes <- 4
#get path to example network file
networkFile <- system.file(package = 'CausalR', 'extdata', 'testNetwork.sif')
#create ccg
network <-  CreateCCG(networkFile)
#get path to example experimental data
experimentalDataFile <- system.file(package = 'CausalR', 'extdata', 'testData.txt')
#read in experimetal data
experimentalData <-  ReadExperimentalData(experimentalDataFile, network)
#run in single threaded mode
runSCANR(network, experimentalData, numberOfDeltaToScan, topNumGenes)
#run in parallel mode
runSCANR(network, experimentalData, numberOfDeltaToScan, topNumGenes,
         doParallel = TRUE, numCores = 2)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/OrderHypotheses.r
\name{OrderHypotheses}
\alias{OrderHypotheses}
\title{order hypotheses}
\usage{
OrderHypotheses(scoresMatrix)
}
\arguments{
\item{scoresMatrix}{a matrix containing the scores for each node of the network}
}
\value{
a ranked table of hypotheses
}
\description{
Ranks the hypotheses.  Takes a matrix containing the scores for each node of the network, and ranks them placing the hypothesis with the 
most correct predictions is at the top
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MakePredictionsFromCCG.r
\name{MakePredictionsFromCCG}
\alias{MakePredictionsFromCCG}
\title{make predictions from CCG}
\usage{
MakePredictionsFromCCG(hypothesisnode, signOfHypothesis, network, delta,
  nodesInExperimentalData = NULL)
}
\arguments{
\item{hypothesisnode}{a hypothesis node}

\item{signOfHypothesis}{the direction of change of hypothesis node}

\item{network}{a computational causal graph}

\item{delta}{the number of edges across which the hypothesis should be followed}

\item{nodesInExperimentalData}{the number of nodes in experimental data}
}
\value{
an matrix containing the relationship between each node and the hypothesis
}
\description{
Create a matrix of predictions for a particular hypothesis starting from a network 
with separate nodes for up- and down-regulation (+ve and -ve).
The output is an array containing the relationship between each node and the hypothesis.
The hypothesis provided will be the vertex id of one of the nodes in the network
(as an integer or name including + or - for up/down regulation).
The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation.
(It generally shouldn't be necessary to reverse the sign of a node when working from
a CCG, but this facility is included for consistency with MakePredictionsFromCG)
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg <- CreateCCG(network)
MakePredictionsFromCCG('NodeA', +1, ccg, 2)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CreateCCG.r
\name{CreateCCG}
\alias{CreateCCG}
\title{create a Computational Causal Graph (CCG)}
\usage{
CreateCCG(filename, nodeInclusionFile = NULL, excludeNodesInFile = TRUE)
}
\arguments{
\item{filename}{file name of the network file (in .sif file format)}

\item{nodeInclusionFile}{optional path to a text file listing nodes to exclude in the CCG (or include - see argument excludeNodesInFile).}

\item{excludeNodesInFile}{flag to determine if nodes in inclusion file should be taken as nodes to include or nodes to exclude. Default is TRUE to exclude.}
}
\value{
an igraph object containing the CCG.
}
\description{
Creates a computational causal graph from a network file.
}
\note{
CreateCG and CreateCCG create causal and computational causal graphs respectively.
}
\examples{
# get path to example .sif file
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#create ccg
ccg = CreateCCG(network)


}
\references{
L Chindelevitch et al.
Causal reasoning on biological networks: Interpreting transcriptional changes.
Bioinformatics, 28(8):1114-21, 2012.
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetRegulatedNodes.r
\name{GetRegulatedNodes}
\alias{GetRegulatedNodes}
\title{get regulated nodes}
\usage{
GetRegulatedNodes(PPInet, Expressiondata, delta, hypothesisGene = "",
  signOfHypothesis = 1, outputfile = "")
}
\arguments{
\item{PPInet}{a protein-protein interaction network}

\item{Expressiondata}{a table of observed gene expression data}

\item{delta}{the number of edges to follow along the network. This should typically be between 1 and 5 dependent on network size/topology}

\item{hypothesisGene}{the name of the hypothesis gene}

\item{signOfHypothesis}{the sign of action expected from the hypothesis, +1 for up regulation, -1 for down}

\item{outputfile}{the file to which the results should be written}
}
\value{
Nodes regulated by hypothesis gene
}
\description{
This function will compute the nodes regulated by the given hypothesis gene and write the results to a file
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetWeightsAboveHypothesisScoreAndTotalWeights.r
\name{GetWeightsAboveHypothesisScoreAndTotalWeights}
\alias{GetWeightsAboveHypothesisScoreAndTotalWeights}
\title{get weights above hypothesis score and total weights}
\usage{
GetWeightsAboveHypothesisScoreAndTotalWeights(r_p, r_m, c_p,
  predictionListStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, hypothesisScore, logepsDMax, logDMax)
}
\arguments{
\item{r_p}{the row sum r+}

\item{r_m}{the row sum r-}

\item{c_p}{the column sum c+}

\item{predictionListStats}{statistics for the  prediction list}

\item{experimentalDataStats}{statistics for the experimental data}

\item{logOfFactorialOfPredictionListStats}{log of factorial of prediction list stats}

\item{hypothesisScore}{the hypothesis score to be considered}

\item{logepsDMax}{Exponential of logD Maximum value}

\item{logDMax}{A logD Maximum value}
}
\value{
score data
}
\description{
Gets the score based on the values of n++, n+-, n-+ and n--. Used as part of a p-value calculation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateSignificance.r
\name{CalculateSignificance}
\alias{CalculateSignificance}
\title{calculate overall significance p-value}
\usage{
CalculateSignificance(hypothesisScore, predictionListStats,
  experimentalResultStats, epsilon = 1e-05, useCubicAlgorithm = TRUE,
  use1bAlgorithm = TRUE)
}
\arguments{
\item{hypothesisScore}{score for a particular hypothesis}

\item{predictionListStats}{numbers of predicted up-regulated, 
predicted down-regulated and ambiguous predictions predicted by the algorithm}

\item{experimentalResultStats}{numbers of up-regulated, 
down-regulated and not significantly changed transcripts in the 
experimental data}

\item{epsilon}{threshold that is used when calculating the p-value 
using the cubic algorithm}

\item{useCubicAlgorithm}{use the cubic algorithm, defaults to TRUE}

\item{use1bAlgorithm}{use the 1b version of the algorithm, defaults to TRUE
used to calculate the p-value}
}
\value{
the resulting  p-value
}
\description{
Calculates the p-value of a score given the hypothesis score and the 
distribution table, using either the quartic or the (faster) cubic algorithm
}
\examples{
CalculateSignificance(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReadSifFileToTable.r
\name{ReadSifFileToTable}
\alias{ReadSifFileToTable}
\title{read .sif to Table}
\usage{
ReadSifFileToTable(sifFile)
}
\arguments{
\item{sifFile}{the sifFile to be read in}
}
\value{
a R table containing the data from the .sif file
}
\description{
Reads a .sif file into a table in R
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FindMaximumDValue.r
\name{FindMaximumDValue}
\alias{FindMaximumDValue}
\title{find maximum D value}
\usage{
FindMaximumDValue(predictionListStats, experimentalDataStats,
  logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{predictionListStats}{a vector containing the predicted values q+, q- and q0:
numbers of positive, negative and non-significant/contradictory predictions}

\item{experimentalDataStats}{A vector containing the observed values n+, n- and n0:
numbers of positive, negative and non-significant/contradictory observations}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial value for
each entry in predictionListStats}

\item{returnlog}{should the result be returned as a log; default FALSE}
}
\value{
the maximum possible D value
}
\description{
computes the maximum possible D-value 
for given values q+, q-, q0 and n+, n-, n0.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateSignificanceUsingQuarticAlgorithm.r
\name{CalculateSignificanceUsingQuarticAlgorithm}
\alias{CalculateSignificanceUsingQuarticAlgorithm}
\title{calculate significance using the quartic algorithm}
\usage{
CalculateSignificanceUsingQuarticAlgorithm(hypothesisScore, predictionListStats,
  experimentalDataStats)
}
\arguments{
\item{hypothesisScore}{the score for which a  p-value is required}

\item{predictionListStats}{a vector containing the values q+, q- and q0 (the number of positive/negative/non-significant  or contradictory) predictions)}

\item{experimentalDataStats}{a vector containing the values n+, n- and n0 (the number of positive/negative/non-significant (or contradictory) transcripts in the results) (or contradictory) transcripts in the results)}
}
\value{
the corresponding p-value
}
\description{
Computes the significance of a given hypothesis. For a detailed description of the algorithm
see Causal reasoning on biological networks: interpreting transcriptional changes - Chindelevitch et al., section 2.
from which the methods and notation is taken.
}
\examples{
CalculateSignificance(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)
}
\references{
L Chindelevitch et al.
Causal reasoning on biological networks: Interpreting transcriptional changes.
Bioinformatics, 28(8):1114-21, 2012.
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FindApproximateValuesThatWillMaximiseDValue.r
\name{FindApproximateValuesThatWillMaximiseDValue}
\alias{FindApproximateValuesThatWillMaximiseDValue}
\title{find approximate values that will maximise D value}
\usage{
FindApproximateValuesThatWillMaximiseDValue(predictionListStats,
  experimentalDataStats)
}
\arguments{
\item{predictionListStats}{a vector containing the values q+, q- and q0: 
numbers of positive, negative and non-significant/contradictory predictions}

\item{experimentalDataStats}{a vector containing the values n+, n- and n0:
numbers of positive, negative and non-significant/contradictory predictions}
}
\value{
a 2x2 contingency table which approximately maximises D
}
\description{
Finds an approximate table values to maximise D.
Given the values of q+, q-, q0, n+, n- and n0 this function will produce the approximate values
of n++, n+-, n-+ and n-- that will maximise the D value. See Assessing statistical significance of 
casual graphs, page 6. The values are approximate since they need to be rounded, although the direction
of rounding is not clear at this stage.
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AddIDsToVertices.r
\name{AddIDsToVertices}
\alias{AddIDsToVertices}
\title{add IDs to vertices}
\usage{
AddIDsToVertices(network)
}
\arguments{
\item{network}{the network to which the IDs are to be added}
}
\value{
network with IDs added
}
\description{
Adds the IDs as a vertex property to 
the vertices in the network. Used when creating
sub-networks where the new nodes will retain the IDs from their original
network
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CheckRowAndColumnSumValuesAreValid.r
\name{CheckRowAndColumnSumValuesAreValid}
\alias{CheckRowAndColumnSumValuesAreValid}
\title{check row and column sum values are valid}
\usage{
CheckRowAndColumnSumValuesAreValid(rowAndColumnSumValues, predictionListStats,
  experimentalResultStats)
}
\arguments{
\item{rowAndColumnSumValues}{a 4x1 vector containing the row and column sum values (r+, r-, c+, c-) for a 2x2 contingency table}

\item{predictionListStats}{a vector containing the values q+, q- and q0}

\item{experimentalResultStats}{A vector containing the values n+, n- and n0}
}
\value{
TRUE if the table is valid; otherwise FALSE
}
\description{
Checkes to see if the values of r+, r-, c+ and c- which are stored in rowAndColumnSumValues define a valid contingency
table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetApproximateMaximumDValueFromThreeByTwoContingencyTable.r
\name{GetApproximateMaximumDValueFromThreeByTwoContingencyTable}
\alias{GetApproximateMaximumDValueFromThreeByTwoContingencyTable}
\title{returns approximate maximum D value or weight for a 3x2 superfamily}
\usage{

  GetApproximateMaximumDValueFromThreeByTwoContingencyTable(threeByTwoContingencyTable,
  predictionListStats, logOfFactorialOfPredictionListStats, returnlog = FALSE)
}
\arguments{
\item{threeByTwoContingencyTable}{approximate values of n++, n+-, n-+, n--, n0+ and n0-, these values are
calculated to optimise the D-value (see page 6 of Assessing
statistical significance of causal graphs)}

\item{predictionListStats}{a vector containing the values q+, q- and q0 
(the number of positive/negative/non-significant 
(or contradictory) predictions)}

\item{logOfFactorialOfPredictionListStats}{a vector containing the log of the factorial value for
each entry in predictionListStats}

\item{returnlog}{return the result as a log, default is FALSE}
}
\value{
an approximate maximum D value or weight
}
\description{
Computes an approximate maximum D value (or weight) for a superfamily (3x2 table).
The result is only approximate as only the first valid D value that is return. This has been
done to speed up the overall algorithm.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CreateCG.r
\name{CreateCG}
\alias{CreateCG}
\title{create a Computational Graph (CG)}
\usage{
CreateCG(sifFile)
}
\arguments{
\item{sifFile}{the path of the .sif file that contains all 
the information about the network Load in .sif file}
}
\value{
a CG network
}
\description{
Creates a CG network from a .sif file.
Takes in a .sif file output from Cytoscape, and creates an 'igraph' representing the network.
The edges will be annotated with the type of interaction and a weight (1 for activation and -1 for inhibition)
}
\examples{
# get path to example .sif file
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
#create cg
cg = CreateCG(network)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateSignificanceUsingCubicAlgorithm.r
\name{CalculateSignificanceUsingCubicAlgorithm}
\alias{CalculateSignificanceUsingCubicAlgorithm}
\title{calculate significance using the cubic algorithm}
\usage{
CalculateSignificanceUsingCubicAlgorithm(hypothesisScore, predictionListStats,
  experimentalDataStats, epsilon)
}
\arguments{
\item{hypothesisScore}{the score whose p-value we want to find.}

\item{predictionListStats}{numbers of predicted up-regulated, predicted down-regulated and ambiguous predictions.}

\item{experimentalDataStats}{numbers of up-regulated, down-regulated and not significantly changed transcripts in the experimental data.}

\item{epsilon}{an epsilon threshold that is used when calculating the p-value 
using the cubic algorithm. Defaults to 1e-5.}
}
\value{
p-value
}
\description{
Calculates the p-value of a score given the hypothesis score and the distribution table (calculated using the cubic algorithm)
}
\examples{
CalculateSignificance(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=TRUE)
CalculateSignificanceUsingQuarticAlgorithm(5, c(7,4,19), c(6,6,18))
CalculateSignificance(5, c(7,4,19), c(6,6,18), useCubicAlgorithm=FALSE)
CalculateSignificance(5, c(7,4,19), c(6,6,18), 1e-5)
CalculateSignificance(5, c(7,4,19), c(6,6,18), epsilon=1e-5, useCubicAlgorithm=TRUE)
CalculateSignificanceUsingCubicAlgorithm(5, c(7,4,19), c(6,6,18), 1e-5)
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PlotGraphWithNodeNames.r
\name{PlotGraphWithNodeNames}
\alias{PlotGraphWithNodeNames}
\title{plot graph with node names}
\usage{
PlotGraphWithNodeNames(igraph)
}
\arguments{
\item{igraph}{internal an igraph representation of an interaction network}
}
\value{
network visualisation
}
\description{
Plots an igraph with the node names. Plots a igraph to the screen displaying the names of the nodes input rather than R's internal numbering.
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
ccg <- CreateCCG(network)
PlotGraphWithNodeNames(ccg)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CalculateEnrichmentPValue.r
\name{CalculateEnrichmentPValue}
\alias{CalculateEnrichmentPValue}
\title{calculates an enrichment p-value}
\usage{
CalculateEnrichmentPValue(predictions, results)
}
\arguments{
\item{predictions}{predictions of changes from the CCG for a particular hypothesis}

\item{results}{gene changes observed in the experimental data}
}
\value{
an enrichment p-value
}
\description{
Calculate a enrichment p-value for a given hypothesis by comparing the corresponding predicted and observed gene changes
}
\examples{
predictions <- matrix(c(1,2,3,1,1,-1), ncol = 2)
results<- matrix(c(1,2,3,4,1,1,-1,1), ncol = 2)
CalculateEnrichmentPValue(predictions, results)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MakePredictionsFromCG.r
\name{MakePredictionsFromCG}
\alias{MakePredictionsFromCG}
\title{make predictions from CG}
\usage{
MakePredictionsFromCG(hypothesisnode, signOfHypothesis, network, delta,
  nodesInExperimentalData = NULL)
}
\arguments{
\item{hypothesisnode}{a hypothesis node}

\item{signOfHypothesis}{the direction of change of hypothesis node}

\item{network}{a computational causal graph}

\item{delta}{the number of edges across which the hypothesis should be followed}

\item{nodesInExperimentalData}{the number of nodes in experimental data}
}
\value{
an matrix containing the relationship between each node and the hypothesis
}
\description{
Create a matrix of predictions for a particular hypothesis - the output
is a matrix containing the relationship between each node and the hypothesis.
The hypothesis provided will be the vertex id of one of the nodes in the network (as an integer). 
The signOfHypothesis variable should be a 1 or -1, indicating up/down regulation
}
\examples{
network <- system.file(package='CausalR', 'extdata', 'testNetwork.sif')
cg <- CreateCG(network)
MakePredictionsFromCG('NodeA', +1, cg, 2)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AnalyseExperimentalData.r
\name{AnalyseExperimentalData}
\alias{AnalyseExperimentalData}
\title{analyse experimental data}
\usage{
AnalyseExperimentalData(experimentalData)
}
\arguments{
\item{experimentalData}{a dataframe containing a list of genes with corresponding direction of change (1 or -1)}
}
\value{
up and down regulation statistics for the experimental data
}
\description{
Returns the number of up- and down-regulated genes in the experimental data
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetCombinationsOfCorrectandIncorrectPredictions.r
\name{GetCombinationsOfCorrectandIncorrectPredictions}
\alias{GetCombinationsOfCorrectandIncorrectPredictions}
\title{returns table of correct and incorrect predictions}
\usage{
GetCombinationsOfCorrectandIncorrectPredictions(predictionDataStats,
  experimentalDataStats)
}
\arguments{
\item{predictionDataStats}{prediction data statistics table}

\item{experimentalDataStats}{Experimental data statistics table}
}
\value{
a matrix the  numbers of correct and incorrect positive and negative prediction
}
\description{
Returns the  numbers of correct and incorrect positive and negative predictions
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ScoreHypothesis.r
\name{ScoreHypothesis}
\alias{ScoreHypothesis}
\title{score hypothesis}
\usage{
ScoreHypothesis(matrixOfPredictions, matrixOfExperimentalData)
}
\arguments{
\item{matrixOfPredictions}{a matrix of predictions}

\item{matrixOfExperimentalData}{a matrix of experimentaldata}
}
\value{
scoreBreakdown a vector giving, in order, the overall score, and the numbers of correct, incorrect and ambigous predictions
}
\description{
Score a single hypothesis, using the predictions from the network and the experimental data returning a vector of scoring statistics
}
\examples{
predictions <- matrix(c(1,2,3,+1,0,-1),ncol=2)
experimentalData <- matrix(c(1,2,4,+1,+1,-1),ncol=2)
ScoreHypothesis(predictions,experimentalData)
CompareHypothesis(predictions,experimentalData)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetMaxDValueForAFamily.r
\name{GetMaxDValueForAFamily}
\alias{GetMaxDValueForAFamily}
\title{get maximun D value for a family}
\usage{
GetMaxDValueForAFamily(r_p, r_m, c_p, predictionListStats,
  experimentalDataStats, logOfFactorialOfPredictionListStats,
  returnlog = FALSE)
}
\arguments{
\item{r_p}{row sum r+}

\item{r_m}{row sum r-}

\item{c_p}{column sum c+}

\item{predictionListStats}{approximate values of n++, n+-, n-+ and n--}

\item{experimentalDataStats}{a vector containing the values q+, q- and q0:  number of positive, negative, non-significant/contradictory predictions}

\item{logOfFactorialOfPredictionListStats}{a vector containing the values n+, n- and n0:  number of positive, negative, non-significant/contradictory observations}

\item{returnlog}{return result as log, default value is FALSE}
}
\value{
the maximum DFam Value
}
\description{
Computes the maximum D value for a particular family - denoted as D_fam on page 6 of Assessing Statistical Signifcance of Causal Graphs
}
\references{
L Chindelevitch et al.
Assessing statistical significance in causal graphs.
BMC Bioinformatics, 13(35), 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WriteAllExplainedNodesToSifFile.r
\name{WriteAllExplainedNodesToSifFile}
\alias{WriteAllExplainedNodesToSifFile}
\title{Write all explained nodes to Sif file}
\usage{
WriteAllExplainedNodesToSifFile(scanResults, network, experimentalData, delta,
  correctlyExplainedOnly = TRUE, quiet = TRUE)
}
\arguments{
\item{scanResults}{a results object produced by ScanR}

\item{network}{a computational causal graph}

\item{experimentalData}{The experimental data read in using \link{ReadExperimentalData}.}

\item{delta}{the number of edges across which the hypothesis should be followed, the setting should be that used to generate the input ScanR object.}

\item{correctlyExplainedOnly}{if TRUE network files will only be produced for correctly explained nodes. If FALSE network files will be produced for each of correctly explained, incorrectly explained and ambiguously explained nodes.}

\item{quiet}{a flag to suppress output to console. FALSE by default.}
}
\value{
files containing paths from hypothesis node to explained nodes in sif format and corresponding annotation (_anno.txt) files
}
\description{
Outputs networks of all explained nodes in .sif file format, named by node name with sign of regulation, each with a corresponding annotation file for producing visualisations using Cytoscape.
}
\examples{
networkFile <- system.file(package='CausalR', 'extdata', 'testNetwork1.sif')
network <- CreateCCG(networkFile)
experimentalDataFile <- system.file(package='CausalR', 'extdata', 'testData1.txt')
experimentalData <- ReadExperimentalData(experimentalDataFile, network)
delta <- 2
scanResults <- runSCANR(network, experimentalData, numberOfDeltaToScan = delta,
    topNumGenes = 2, writeResultFiles = FALSE, writeNetworkFiles = "none",
    quiet = FALSE, doParallel = TRUE, numCores = 2)
WriteAllExplainedNodesToSifFile(scanResults, network, experimentalData, delta,
    correctlyExplainedOnly = TRUE, quiet = TRUE)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetExplainedNodesOfCCG.r
\name{GetExplainedNodesOfCCG}
\alias{GetExplainedNodesOfCCG}
\title{Get explained nodes of CCG}
\usage{
GetExplainedNodesOfCCG(hypothesisnode, signOfHypothesis, network,
  experimentalData, delta)
}
\arguments{
\item{hypothesisnode}{a hypothesis node}

\item{signOfHypothesis}{the direction of change of hypothesis node}

\item{network}{a computational causal graph}

\item{experimentalData}{The experimental data read in using \link{ReadExperimentalData}. The results is an n x 2 matrix; where the first column contains the node ids of the nodes in the network that the results refer to. The second column contains values indicating the direction of regulation in the results - (+)1 for up, -1 for down and 0 for insignificant amounts of regulation. The name of the first column is the filename the data was read from.}

\item{delta}{the number of edges across which the hypothesis should be followed}
}
\value{
vector of explained nodes
}
\description{
Returns a table of node names and values for explained nodes, I.e. nodes that appear in both network and data with the same sign.
The table contain the name in column 1 and the value (1 or -1) in column 2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PopulateTwoByTwoContingencyTable.r
\name{PopulateTwoByTwoContingencyTable}
\alias{PopulateTwoByTwoContingencyTable}
\title{Populate Two by Two Contingency Table}
\usage{
PopulateTwoByTwoContingencyTable(rowAndColumnSumValues, n_pp)
}
\arguments{
\item{rowAndColumnSumValues}{the row and column sums (r+, r-, c+, c-).}

\item{n_pp}{the value of n++.}
}
\value{
the completed 2x2 contingency table: n++, n+-, n-+, n--
}
\description{
Calculates a 2x2 contingency table.
Given the value of n++ and the row and column sums (r+, r-, c+, c-),
Calculates the remaining values in the 2x2 contingency table i.e.
n+-, n-+, and n--. See Chindelevich et al. BMC Bioinformatics (2012) paper 
'Assessing Statistical significance of causal graphs' for clarification on notation.
}
\references{
L Chindelevitch et al.
Causal reasoning on biological networks: Interpreting transcriptional changes.
Bioinformatics, 28(8):1114-21, 2012.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CompareHypothesis.r
\name{CompareHypothesis}
\alias{CompareHypothesis}
\title{compare hypothesis}
\usage{
CompareHypothesis(matrixOfPredictions, matrixOfExperimentalData, ccg = NULL,
  sourceNode = NULL)
}
\arguments{
\item{matrixOfPredictions}{a matrix of predictions}

\item{matrixOfExperimentalData}{a matrix of experimental data}

\item{ccg}{a CCG network (default=NULL)}

\item{sourceNode}{A starting node (default=NULL)}
}
\value{
a matrix containing predictions, observations and scores.
}
\description{
Compare the predictions from a hypothesis with the experimental data returning an matrix with columns for node ID, predictions, experimental results and the corresponding scores.
}
\examples{
predictions <- matrix(c(1,2,3,+1,0,-1),ncol=2)
experimentalData <- matrix(c(1,2,4,+1,+1,-1),ncol=2)
ScoreHypothesis(predictions,experimentalData)
CompareHypothesis(predictions,experimentalData)
}
\concept{
CausalR
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetWeightsFromInteractionInformation.r
\name{GetWeightsFromInteractionInformation}
\alias{GetWeightsFromInteractionInformation}
\title{get weights from interaction information}
\usage{
GetWeightsFromInteractionInformation(interactionInfo)
}
\arguments{
\item{interactionInfo}{a central column of the .sif file, giving the type of edge interaction}
}
\value{
a matrix of weights corresponding the the direction of regulation
}
\description{
Returns a matrix of weights (-1,0,+1) indicating the direction of regulation from the interaction information.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ComputePValueFromDistributionTable.r
\name{ComputePValueFromDistributionTable}
\alias{ComputePValueFromDistributionTable}
\title{compute a p-value from the distribution table}
\usage{
ComputePValueFromDistributionTable(scoreOfHypothesis, distributionMatrix,
  totalWeights)
}
\arguments{
\item{scoreOfHypothesis}{a score of hypothesis}

\item{distributionMatrix}{a distribution table presented as a matrix}

\item{totalWeights}{a matrix of total weights}
}
\value{
a p-value
}
\description{
Computes the p-value of the score of an hypothesis, based on a distribution table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CreateNetworkFromTable.r
\name{CreateNetworkFromTable}
\alias{CreateNetworkFromTable}
\title{create network from table}
\usage{
CreateNetworkFromTable(dataTable)
}
\arguments{
\item{dataTable}{the data table containing the information read in from the .sif file representing the network.}
}
\value{
an igraph network
}
\description{
Creates a network from an internal data table created from a .sif file:
this function converts the data read in from the .sif file into 
an igraph in R.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GetScoreForNumbersOfCorrectandIncorrectPredictions.r
\name{GetScoreForNumbersOfCorrectandIncorrectPredictions}
\alias{GetScoreForNumbersOfCorrectandIncorrectPredictions}
\title{returns the score for a given number of correct and incorrect predictions}
\usage{
GetScoreForNumbersOfCorrectandIncorrectPredictions(matrixRow)
}
\arguments{
\item{matrixRow}{a row af a matrix of correct and incorrect prediction scores}
}
\value{
the corresponding score for the given row
}
\description{
Returns the score based on the values of n++, n+-, n-+ and n--
}
